////////////////////////////////////////////////////////////////////////////////
// chaos.h : main

#ifndef CHAOS_H_INCLUDED
#define CHAOS_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// declare

class chaos;

////////////////////////////////////////////////////////////////////////////////
// include

#ifndef CHAOS_POS_H_INCLUDED
#include "chaos_pos.h"
#endif // CHAOS_POS_H_INCLUDED

#ifndef CHAOS_RECT_H_INCLUDED
#include "chaos_rect.h"
#endif // CHAOS_RECT_H_INCLUDED

#ifndef CHAOS_COLOR_H_INCLUDED
#include "chaos_color.h"
#endif // CHAOS_COLOR_H_INCLUDED

#ifndef CHAOS_STRING_H_INCLUDED
#include "chaos_string.h"
#endif // CHAOS_STRING_H_INCLUDED

#ifndef CHAOS_XML_H_INCLUDED
#include "chaos_xml.h"
#endif // CHAOS_XML_H_INCLUDED

#ifndef CHAOS_IMAGE_MANAGER_H_INCLUDED
#include "chaos_image_manager.h"
#endif // CHAOS_IMAGE_MANAGER_H_INCLUDED

#ifndef CHAOS_SYSTEM_H_INCLUDED
#include "chaos_system.h"
#endif // CHAOS_SYSTEM_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// chaos : main class

class chaos {
// enum
private:
	// limit
	enum {
		recollect_row		= 10,
		snapshot_page_count	= 10,
		snapshot_row_count	= 10,
		snapshot_count		= snapshot_page_count*snapshot_row_count,
		scroll_delta		= 120
	};
	// time
	enum {
		audio_notify_elapse	= 500
	};
	// arrow type
	enum arrow_type {
		up_arrow,
		down_arrow,
		left_arrow,
		right_arrow
	};
	// special mode type
	enum special_mode_type {
		spmode_none				= 0,
		spmode_main_menu		= 1,
		spmode_save				= 10,
		spmode_save_confirm		= 11,
		spmode_load				= 20,
		spmode_load_confirm		= 21,
		spmode_appreciate		= 30,
		spmode_recollect		= 40,
		spmode_rewind_confirm	= 41,
		spmode_restart_confirm	= 50,
		spmode_exit_confirm		= 60
	};

// attribute
private:
	// system
	chaos_system* system;
	// image manager
	chaos_image_manager* image_manager;
	// arguments
	std::vector<chaos_string> args;
	// last time
	chaos_uint32 last_time;
	// special mode
	special_mode_type special_mode;
	// special current selection
	int special_current_selection;
	// history offset
	int history_offset;
	// snapshot page
	int snapshot_page;
	// preview snapno
	int preview_snapno;
	// postpone preview snapno update
	int postpone_preview_snapno_update;
	// snapshots
	std::vector<chaos_xml*> snapshots;
	// enable appreciate
	bool enable_appreciate;
	// mouse wheel scroll remainder
	int mouse_scroll_remainder;
	// idling stop
	bool idling_stop;
	// full screen
	bool full_screen;
	// application quit
	bool app_quit;

	// main window
	chaos_wnd_handle main_wnd;
	// log window
	chaos_wnd_handle log_wnd;

#ifdef CHAOS_VIDEO_DIRECTX

	// direct draw module
	HMODULE ddraw_module;
	// direct draw
	IDirectDraw* ddraw;

#elif defined(CHAOS_VIDEO_SDL)

	SDL_Surface* screen;

#endif // CHAOS_VIDEO

#ifdef CHAOS_CDAUDIO_SDL

	// audio notify counter
	int audio_notify_counter;

#endif // CHAOS_CDAUDIO_SDL

// create & destroy
private:
	// default constructor
	chaos();

public:
	// destructor
	~chaos();

	// get singleton instance
	static chaos* instance();

// get & set
public:
	// get last time
	chaos_uint32 get_last_time() const {
		return last_time;
	}
	// set last time
	void set_last_time(chaos_uint32 new_time) {
		last_time = new_time;
	}
	// get main window haldle
	chaos_wnd_handle get_main_wnd() const {
		return main_wnd;
	}
	// set main window haldle
	void set_main_wnd(chaos_wnd_handle new_main_wnd) {
		main_wnd = new_main_wnd;
	}
	// get log window haldle
	chaos_wnd_handle get_log_wnd() const {
		return log_wnd;
	}
	// set log window haldle
	void set_log_wnd(chaos_wnd_handle new_log_wnd) {
		log_wnd = new_log_wnd;
	}
	// set arguments
	void set_args(int argc, char* argv[]);

// main window & screen
private:

#ifdef CHAOS_SCREEN_WIN32

	// main window procedure
	static LRESULT CALLBACK main_wnd_proc(
		HWND wnd, UINT message, WPARAM wparam, LPARAM lparam);
	// create main window
	HWND create_main_wnd();
	// close main window
	void close_main_wnd();

#elif defined(CHAOS_SCREEN_SDL)

	// sdl event
	void process_event(SDL_Event* ev);
	// clear events
	void clear_events();
	// create screen
	bool create_screen();

#endif // CHAOS_SCREEN

	// update screen(multi rect)
	void update_screen(
		const std::list<chaos_rect>& updates,
		bool update_now);
	// update screen(single rect)
	void update_screen(
		const chaos_rect& update_rect,
		bool update_now);
	// update screen(all)
	void update_screen(
		bool update_now);
	// is full screen
	bool is_full_screen() const;
	// set full screen
	bool set_full_screen(bool new_full_screen, bool prop_set);

// log window
private:

#ifdef CHAOS_GUI_WIN32

	// log window procedure
	static LRESULT CALLBACK log_wnd_proc(
		HWND wnd, UINT message, WPARAM wparam, LPARAM lparam);
	// create log window
	HWND create_log_wnd();
	// close log window
	void close_log_wnd();

#endif // CHAOS_GUI_WIN32

// special mode
private:
	// get special mode
	special_mode_type get_special_mode() const {
		return special_mode;
	}
	// set special mode
	void set_special_mode(special_mode_type new_mode) {
		special_mode = new_mode;
	}
	// change special mode
	void change_special_mode(special_mode_type new_mode);

// background
private:
	// remove message window info
	chaos_string remove_message_window_info(
		const chaos_string& oinfo) const;
	// get background info
	chaos_string get_background_info() const;

// selection
private:
	// get selection count
	int get_selection_count() const;
	// get current selection
	int get_current_selection() const;
	// set current selection
	void set_current_selection(int new_selno);
	// is enable selection
	bool is_enable_selection(int selno) const;
	// get selection rect
	chaos_rect get_selection_rect(int selno) const;
	// get selection color
	chaos_color get_selection_color(int selno) const;
	// get hit selection
	int get_hit_selection(chaos_pos pos) const;

// message
private:
	// get message text
	chaos_string get_message_text() const;

// image
private:
	// draw image
	void draw_image(
		chaos_rect* update_rect,
		const chaos_rect* clip_rect);

// fix
private:
	// set fix values
	void set_fixes() const;

// xml
private:
	// set xml date & time
	static void set_xml_date_time(chaos_xml* xml);
	// set xml info
	static void set_xml_info(chaos_xml* xml);

// snapshot
private:
	// get snapshot filename
	static chaos_string get_snapshot_filename(int datano);
	// is snapshot valid
	bool is_snapshot_valid(int snapno) const;
	// load snapshot from file
	bool load_snapshot_from_file(
		const chaos_string& filename,
		chaos_xml* snapshot) const;
	// save snapshot to file
	bool save_snapshot_to_file(
		const chaos_string& filename,
		chaos_xml* snapshot) const;
	// load all snapshots
	void load_all_snapshots();
	// clear all snapshots
	void clear_all_snapshots();
	// update preview snapno
	void update_preview_snapno();

// profile
private:
	// get profile filename
	static chaos_string get_profile_filename();
	// load profile from file
	bool load_profile_from_file(
		const chaos_string& filename,
		chaos_xml* profile) const;
	// save profile to file
	bool save_profile_to_file(
		const chaos_string& filename,
		chaos_xml* profile) const;
	// set default profile
	bool set_default_profile(
		chaos_xml* profile) const;
	// config profile
	void config_profile(
		const chaos_xml* profile);

// readflag
private:
	// get readflag filename
	static chaos_string get_readflag_filename();
	// load readflag from file
	bool load_readflag_from_file(
		const chaos_string& filename,
		chaos_xml* readflag) const;
	// save readflag to file
	bool save_readflag_to_file(
		const chaos_string& filename,
		chaos_xml* readflag) const;

// user input
private:
	// on mouse left-button pressed
	void on_mouse_lbutton_pressed(int x, int y);
	// on mouse right-button pressed
	void on_mouse_rbutton_pressed(int x, int y);
	// on mouse moved
	void on_mouse_moved(int x, int y);
	// on mouse scrolled
	void on_mouse_scrolled(int x, int y, int scroll);
	// on keyboard enter pressed
	void on_key_enter_pressed();
	// on keyboard escape pressed
	void on_key_escape_pressed();
	// on keyboard backspace pressed
	void on_key_backspace_pressed();
	// on keyboard arrow pressed
	void on_key_arrow_pressed(arrow_type arrow);
	// on keyboard next pressed
	void on_key_next_pressed();
	// on keyboard prior pressed
	void on_key_prior_pressed();
	// on keyboard home pressed
	void on_key_home_pressed();
	// on keyboard end pressed
	void on_key_end_pressed();
	// on keyboard delete pressed
	void on_key_delete_pressed();
	// on keyboard tab pressed
	void on_key_tab_pressed(bool shift);
	// on keyboard control pressed
	void on_key_control_pressed();
	// on keyboard control released
	void on_key_control_released();

// execute
private:
	// initialize
	bool initialize();
	// finalize
	bool finalize();
	// message loop
	int message_loop();
	// close application
	void close_app();
	// idle
	bool idle();
	// get ticks
	static chaos_uint32 get_ticks();

public:
	// is idling stop
	bool is_idling_stop() const {
		return idling_stop;
	}
	// set idling stop
	void set_idling_stop(bool new_idling_stop) {
		idling_stop = new_idling_stop;
	}
	// execute chaos
	int execute();
};

#endif // CHAOS_H_INCLUDED
