////////////////////////////////////////////////////////////////////////////////
// chaos_auto_array.h : auto array

#ifndef CHAOS_AUTO_ARRAY_H_INCLUDED
#define CHAOS_AUTO_ARRAY_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// declare

template <class element_type>
class chaos_auto_array;

////////////////////////////////////////////////////////////////////////////////
// auto array class template

template <class element_type>
class chaos_auto_array {
// typedef
public:
	// my type
	typedef chaos_auto_array<element_type> my_type;

// attribute:
private:
	// owns
	mutable bool owns;
	// array pointer
	element_type* array_ptr;

// create & destroy
public:
	// default constructor
	chaos_auto_array() :
		owns(false),
		array_ptr(NULL)
	{
	}
	// normal constructor
	explicit chaos_auto_array(element_type* new_array_ptr) :
		owns(true),
		array_ptr(new_array_ptr)
	{
	}
	// copy constructor
	chaos_auto_array(const my_type& org) :
		owns(org.owns),
		array_ptr(org.release())
	{
	}
	// destructor
	~chaos_auto_array() {
		if (owns) {
			delete [] array_ptr;
		}
	}

// operation
public:
	// get array
	element_type* get() const {
		return array_ptr;
	}
	// release & get array
	element_type* release() const {
		owns = false;
		return array_ptr;
	}

// operator
public:
	// let
	my_type& operator=(const my_type& org) {
		if (this != &org) {
			if (array_ptr != org.get()) {
				if (owns) {
					delete [] array_ptr;
				}
				owns = org.owns;
			} else if (org.owns) {
				owns = true;
			}
			array_ptr = org.release();
		}
		return (*this);
	}
	// ref
	element_type& operator*() const {
		return *get();
	}
	// pointer ref
	element_type* operator->() const {
		return get();
	}
	// indexer
	element_type& operator[](int index) const {
		return *(get()+index);
	}
};

#endif // CHAOS_AUTO_ARRAY_H_INCLUDED
