////////////////////////////////////////////////////////////////////////////////
// chaos_buffer.h : buffer

#ifndef CHAOS_BUFFER_H_INCLUDED
#define CHAOS_BUFFER_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// declare

class chaos_buffer;

////////////////////////////////////////////////////////////////////////////////
// chaos_buffer : buffer class

class chaos_buffer {
// typedef
public:
	// value type
	typedef chaos_uint8 value_type;
	// size type
	typedef size_t size_type;
	// iterator
	typedef value_type* iterator;
	// const iterator
	typedef const value_type* const_iterator;
	// reference
	typedef value_type& reference;
	// const reference
	typedef const value_type& const_reference;

// attribute
private:
	// data pointer
	value_type* data_ptr;
	// data size
	size_type data_size;
	// data capacity
	size_type data_capacity;

// create & destroy
public:
	// default constructor
	explicit chaos_buffer();
	// fill constructor
	explicit chaos_buffer(size_type fill, const value_type& value = 0);
	// copy constructor
	chaos_buffer(const chaos_buffer& org);
	// sequence constructor
	chaos_buffer(const_iterator first, const_iterator last);
	// destructor
	~chaos_buffer();

// buffer
private:
	// allocate data buffer
	void alloc_data(size_type new_data_capacity);
	// free data buffer
	void free_data();

// operation
public:
	// get size
	size_type size() const {
		return data_size;
	}
	// get capacity
	size_type capacity() const {
		return data_capacity;
	}
	// is empty
	bool empty() const {
		return data_size == 0;
	}
	// get data
	const value_type* data() const {
		return data_ptr;
	}
	value_type* data() {
		return data_ptr;
	}
	// get begin
	const_iterator begin() const {
		return data_ptr;
	}
	iterator begin() {
		return data_ptr;
	}
	// get end
	const_iterator end() const {
		return data_ptr+data_size;
	}
	iterator end() {
		return data_ptr+data_size;
	}
	// reference
	const_reference at(size_type index) const {
		ASSERT(index < data_size);
		return *(data_ptr+index);
	}
	reference at(size_type index) {
		ASSERT(index < data_size);
		return *(data_ptr+index);
	}
	// reference front
	const_reference front() const {
		ASSERT(data_size > 0);
		return *data_ptr;
	}
	reference front() {
		ASSERT(data_size > 0);
		return *data_ptr;
	}
	// reference back
	const_reference back() const {
		ASSERT(data_size > 0);
		return *(data_ptr+data_size-1);
	}
	reference back() {
		ASSERT(data_size > 0);
		return *(data_ptr+data_size-1);
	}
	// erase
	iterator erase(iterator first, iterator last);
	iterator erase(iterator target);
	// insert
	void insert(
		iterator target,
		size_type fill,
		const value_type& value);
	iterator insert(
		iterator target,
		const value_type& value = 0);
	// push back
	void push_back(const value_type& value);
	// pop back
	void pop_back();
	// reserve
	void reserve(size_type reserve_size);
	// resize
	void resize(
		size_type new_data_size,
		const value_type& value = 0);
	// clear
	void clear();
	// clear all
	void clear_all();

// operator
public:
	// copy
	chaos_buffer& operator=(const chaos_buffer& org);
	// indexer
	const_reference operator[](size_type index) const {
		ASSERT(index < data_size);
		return *(data_ptr+index);
	}
	reference operator[](size_type index) {
		ASSERT(index < data_size);
		return *(data_ptr+index);
	}
};

#endif // CHAOS_BUFFER_H_INCLUDED
