////////////////////////////////////////////////////////////////////////////////
// chaos_cd_player.h : cd player

#ifndef CHAOS_CD_PLAYER_H_INCLUDED
#define CHAOS_CD_PLAYER_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// declare

class chaos_cd_player;

////////////////////////////////////////////////////////////////////////////////
// include

#ifndef CHAOS_STRING_H_INCLUDED
#include "chaos_string.h"
#endif // CHAOS_STRING_H_INCLUDED

#ifndef CHAOS_SOUND_MANAGER_H_INCLUDED
#include "chaos_sound_manager.h"
#endif // CHAOS_SOUND_MANAGER_H_INCLUDED

#ifndef CHAOS_XML_H_INCLUDED
#include "chaos_xml.h"
#endif // CHAOS_XML_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// chaos_cd_player : cd player class

class chaos_cd_player {
// attribute
private:
	// opened player
	bool opened;
	// sound type
	chaos_string sound_type;
	// sound type cd-da
	bool cdda;
	// cd drive name
	chaos_string drive_name;
	// playing start track
	int playing_start_track;
	// playing end track
	int playing_end_track;
	// repeat
	bool repeat;
	// sound object
	chaos_sound sound;
	// bgm replace
	chaos_xml* bgm_replace;

#ifdef CHAOS_CDAUDIO_WIN32

	// cd device id
	MCIDEVICEID cd_device_id;
	// notify window
	HWND notify_wnd;

#elif defined(CHAOS_CDAUDIO_SDL)

	// cd device
	SDL_CD* cd_device;

#endif // CHAOS_CDAUDIO

// create & destroy
private:
	// default constructor
	chaos_cd_player();

public:
	// destructor
	~chaos_cd_player();
	// dispose
	void dispose();

	// get singleton instance
	static chaos_cd_player* instance();

// get & set
public:
	// is opened player
	bool is_opened() const {
		return opened;
	}
	// get sound type
	const chaos_string& get_sound_type() const {
		return sound_type;
	}
	// is sound type cd-da
	bool is_cdda() const {
		return cdda;
	}
	// get cd drive name
	const chaos_string& get_drive_name() const {
		return drive_name;
	}
	// is device open
	bool is_device_open() const {
		return !drive_name.empty();
	}
	// get playing start track
	int get_playing_start_track() const {
		return playing_start_track;
	}
	// get playing end track
	int get_playing_end_track() const {
		return playing_end_track;
	}
	// is repeat
	bool is_repeat() const {
		return repeat;
	}
	// is playing
	bool is_playing() const {
		return playing_start_track >= 1;
	}

#ifdef CHAOS_CDAUDIO_WIN32

	// get cd device id
	MCIDEVICEID get_cd_device_id() const {
		return cd_device_id;
	}
	// get notify window
	HWND get_notify_wnd() const {
		return notify_wnd;
	}
	// set notify window
	void set_notify_wnd(HWND new_wnd) {
		notify_wnd = new_wnd;
	}

#elif defined(CHAOS_CDAUDIO_SDL)

	// get cd device
	const SDL_CD* get_cd_device() const {
		return cd_device;
	}
	SDL_CD* get_cd_device() {
		return cd_device;
	}

#endif

// device
public:
	// open device
	bool open_device(
		const chaos_string& new_sound_type,
		const chaos_string& new_drive_name);
	// close device
	bool close_device();
	// set sound volume
	void set_sound_volume(int new_sound_volume);

// play
public:
	// play cd audio
	bool play(
		int new_start_track, int new_end_track,
		bool new_repeat);
	// stop cd audio
	bool stop();
	// notify cd audio
	bool notify();

// operation
public:
	// encode cd info
	static chaos_string encode_cd_info(
		const chaos_string& volume,
		const std::vector<int>& track_length);
	// decode cd info
	static bool decode_cd_info(
		const chaos_string& info,
		chaos_string& volume,
		std::vector<int>& track_length);
	// is match cd info
	static bool is_match_cd_info(
		const chaos_string& info1, const chaos_string& info2);
	// get cd drives
	static void get_cd_drives(
		std::vector<chaos_string>& cd_drives);
	// get cd info
	chaos_string get_cd_info();
	// open match device
	bool open_match_device(
		const chaos_string& new_sound_type,
		const chaos_string& fix_drive_name,
		const chaos_string& info);
	// get track filename
	chaos_string get_track_filename(
		int track);
};

#endif // CHAOS_CD_PLAYER_H_INCLUDED
