////////////////////////////////////////////////////////////////////////////////
// chaos_config_dlg.cpp : config dialog

////////////////////////////////////////////////////////////////////////////////
// include

#include "stdafx.h"

#include "chaos_config_dlg.h"
#include "chaos_message_imager.h"
#include "chaos_cd_player.h"
#include "chaos_auto_array.h"
#include "chaos_resource.h"
#include "chaos_utility.h"

#include "resource.h"

////////////////////////////////////////////////////////////////////////////////
// chaos_config_dlg : config dialog class

////////////////////////////////////////////////////////////////////////////////
// create & destroy

// default constructor

chaos_config_dlg::chaos_config_dlg() :
	chaos_dlg(),
	dirty(false),
	profile(NULL),
	sample_image(NULL)
{

#ifdef CHAOS_GUI_GTK

	sample_pixmap = NULL;
	font_dirty = false;

#endif // CHAOS_GUI

}

// destructor

chaos_config_dlg::~chaos_config_dlg() {
}

// get singleton instance

chaos_config_dlg* chaos_config_dlg::instance() {
	static chaos_config_dlg config_dlg;
	return &config_dlg;
}

////////////////////////////////////////////////////////////////////////////////
// dialog

#ifdef CHAOS_GUI_WIN32

// general page procedure

BOOL CALLBACK chaos_config_dlg::general_dlgproc(
	HWND dlg, UINT message, WPARAM wparam, LPARAM lparam)
{
	BOOL result = FALSE;
	switch (message) {
	case WM_INITDIALOG:
		{ // dummy block
			HWND prop_wnd = GetParent(dlg),
				main_wnd = GetParent(prop_wnd);
			if ((prop_wnd != NULL) && (main_wnd != NULL)) {
				RECT prop_rect, main_rect;
				GetWindowRect(prop_wnd, &prop_rect);
				GetWindowRect(main_wnd, &main_rect);
				SetWindowPos(
					prop_wnd, NULL,
					(main_rect.left+main_rect.right)/2-
						(prop_rect.right-prop_rect.left)/2,
					(main_rect.top+main_rect.bottom)/2-
						(prop_rect.bottom-prop_rect.top)/2,
					0,
					0,
					SWP_NOZORDER | SWP_NOSIZE | SWP_NOACTIVATE);
			}
			chaos_resource* res = chaos_resource::instance();
			SendDlgItemMessage(
				dlg, IDC_FULL_SCREEN,
				BM_SETCHECK,
				instance()->get_bool_profile("full_screen")?
					BST_CHECKED: BST_UNCHECKED,
				0);
			int i;
			for (i = 0; i < 6; i++) {
				chaos_resource::resource_item_type item =
					chaos_resource::item_none;
				switch (i) {
				case 0:
					item = chaos_resource::tspeed_superslow;
					break;
				case 1:
					item = chaos_resource::tspeed_slow;
					break;
				case 2:
					item = chaos_resource::tspeed_normal;
					break;
				case 3:
					item = chaos_resource::tspeed_fast;
					break;
				case 4:
					item = chaos_resource::tspeed_superfast;
					break;
				case 5:
					item = chaos_resource::tspeed_instant;
					break;
				}
				SendDlgItemMessage(
					dlg, IDC_TEXT_SPEED,
					CB_ADDSTRING,
					0,
					(LPARAM)(res->get_resource_text(
						item).c_str()));
			}
			SendDlgItemMessage(
				dlg, IDC_TEXT_SPEED,
				CB_SETCURSEL,
				instance()->get_int_profile("text_speed")-1,
				0);
			for (i = 0; i < 3; i++) {
				chaos_resource::resource_item_type item =
					chaos_resource::item_none;
				switch (i) {
				case 0:
					item = chaos_resource::sspeed_normal;
					break;
				case 1:
					item = chaos_resource::sspeed_fast;
					break;
				case 2:
					item = chaos_resource::sspeed_jump;
					break;
				}
				SendDlgItemMessage(
					dlg, IDC_SKIP_SPEED,
					CB_ADDSTRING,
					0,
					(LPARAM)(res->get_resource_text(item).c_str()));
			}
			SendDlgItemMessage(
				dlg, IDC_SKIP_SPEED,
				CB_SETCURSEL,
				instance()->get_int_profile("skip_speed")-1,
				0);
			SetDlgItemText(
				dlg, IDC_LASTNAME,
				instance()->get_str_profile("@S").c_str());
			SetDlgItemText(
				dlg, IDC_FIRSTNAME,
				instance()->get_str_profile("@M").c_str());
			SetDlgItemText(
				dlg, IDC_NICKNAME,
				instance()->get_str_profile("@N").c_str());
			SetDlgItemText(
				dlg, IDC_POSTFIX,
				instance()->get_str_profile("@K").c_str());
			result = FALSE;
		}
		break;
	case WM_NOTIFY:
		if (((NMHDR*)lparam)->code == PSN_APPLY) {
			enum {
				BUFSIZE = 256
			};
			chaos_auto_array<char> buf(new char[BUFSIZE]);
			bool dirty = false;
			dirty |= instance()->sync_bool_profile(
				"full_screen",
				SendDlgItemMessage(
					dlg, IDC_FULL_SCREEN,
					BM_GETCHECK,
					0, 0) == BST_CHECKED);
			dirty |= instance()->sync_int_profile(
				"text_speed",
				(int)SendDlgItemMessage(
					dlg, IDC_TEXT_SPEED,
					CB_GETCURSEL,
					0, 0)+1);
			dirty |= instance()->sync_int_profile(
				"skip_speed",
				(int)SendDlgItemMessage(
					dlg, IDC_SKIP_SPEED,
					CB_GETCURSEL,
					0, 0)+1);
			GetDlgItemText(dlg, IDC_LASTNAME, buf.get(), BUFSIZE);
			dirty |= instance()->sync_str_profile(
				"@S", buf.get());
			GetDlgItemText(dlg, IDC_FIRSTNAME, buf.get(), BUFSIZE);
			dirty |= instance()->sync_str_profile(
				"@M", buf.get());
			GetDlgItemText(dlg, IDC_NICKNAME, buf.get(), BUFSIZE);
			dirty |= instance()->sync_str_profile(
				"@N", buf.get());
			GetDlgItemText(dlg, IDC_POSTFIX, buf.get(), BUFSIZE);
			dirty |= instance()->sync_str_profile(
				"@K", buf.get());
			if (dirty) {
				instance()->set_dirty(true);
			}
		}
		break;
	case WM_COMMAND:
		switch (LOWORD(wparam)) {
		case IDC_DEFAULT_NAME:
			if (HIWORD(wparam) == BN_CLICKED) {
				chaos_resource* res = chaos_resource::instance();
				SetDlgItemText(
					dlg, IDC_LASTNAME,
					res->get_resource_text(
						chaos_resource::def_lastname).c_str());
				SetDlgItemText(
					dlg, IDC_FIRSTNAME,
					res->get_resource_text(
						chaos_resource::def_firstname).c_str());
				SetDlgItemText(
					dlg, IDC_NICKNAME,
					res->get_resource_text(
						chaos_resource::def_nickname).c_str());
				SetDlgItemText(
					dlg, IDC_POSTFIX,
					res->get_resource_text(
						chaos_resource::def_postfix).c_str());
			}
			break;
		}
		break;
	default:
		result = FALSE;
		break;
	}
	return result;
}

// sound page procedure

BOOL CALLBACK chaos_config_dlg::sound_dlgproc(
	HWND dlg, UINT message, WPARAM wparam, LPARAM lparam)
{
	BOOL result = FALSE;
	switch (message) {
	case WM_INITDIALOG:
		{ // dummy block
			chaos_resource* res = chaos_resource::instance();
			static const char* sound_types[] = {
				"CD-DA", ".mp3", ".ogg", ".wav", ".wma"
			};
			int i;
			for (i = 0; i < sizeof(sound_types)/sizeof(sound_types[0]); i++) {
				SendDlgItemMessage(
					dlg, IDC_BGM_SOUND_TYPE,
					CB_ADDSTRING,
					0,
					(LPARAM)(sound_types[i]));
			}
			chaos_string bgm_sound_type =
				instance()->get_str_profile("bgm_sound_type");
			if (SendDlgItemMessage(
					dlg, IDC_BGM_SOUND_TYPE,
					CB_SELECTSTRING,
					(WPARAM)-1,
					(LPARAM)(bgm_sound_type.c_str())) == CB_ERR)
			{
				SetDlgItemText(
					dlg, IDC_BGM_SOUND_TYPE,
					bgm_sound_type.c_str());
			}
			chaos_string cd_drive_auto =
				res->get_resource_text(chaos_resource::cd_drive_auto);
			SendDlgItemMessage(
				dlg, IDC_CD_DRIVE,
				CB_ADDSTRING,
				0,
				(LPARAM)(cd_drive_auto.c_str()));
			chaos_string cd_drive;
			for (i = 'a'; i <= 'z'; i++) {
				cd_drive = (char)i;
				cd_drive += ':';
				UINT drive_type = GetDriveType((cd_drive+'\\').c_str());
				if (drive_type == DRIVE_CDROM) {
					cd_drive = chaos_utility::to_upper_str(cd_drive);
					SendDlgItemMessage(
						dlg, IDC_CD_DRIVE,
						CB_ADDSTRING,
						0,
						(LPARAM)(cd_drive.c_str()));
				}
			}
			cd_drive = instance()->get_str_profile("cd_drive");
			if (cd_drive.empty()) {
				cd_drive = cd_drive_auto;
			}
			if (SendDlgItemMessage(
					dlg, IDC_CD_DRIVE,
					CB_SELECTSTRING,
					(WPARAM)-1,
					(LPARAM)(cd_drive.c_str())) == CB_ERR)
			{
				SetDlgItemText(
					dlg, IDC_CD_DRIVE,
					cd_drive.c_str());
			}
			for (i = 0; i < 2; i++) {
				int id;
				chaos_string sound_volume_name;
				if (i == 0) {
					id = IDC_BGM_SOUND_VOLUME;
					sound_volume_name = "bgm_sound_volume";
				} else {
					id = IDC_EFFECT_SOUND_VOLUME;
					sound_volume_name = "effect_sound_volume";
				}
				SendDlgItemMessage(
					dlg, id,
					TBM_SETRANGE,
					TRUE,
					MAKELPARAM(0, 100));
				SendDlgItemMessage(
					dlg, id,
					TBM_SETTICFREQ,
					25,
					0);
				SendDlgItemMessage(
					dlg, id,
					TBM_SETPAGESIZE,
					0,
					5);
				int sound_volume = instance()->get_int_profile(sound_volume_name);
				SendDlgItemMessage(
					dlg, id,
					TBM_SETPOS,
					TRUE,
					sound_volume);
			}
			PostMessage(
				dlg,
				WM_COMMAND,
				MAKELONG(IDC_BGM_SOUND_TYPE, CBN_SELCHANGE),
				(LPARAM)GetDlgItem(dlg, IDC_BGM_SOUND_TYPE));
			result = FALSE;
		}
		break;
	case WM_NOTIFY:
		if (((NMHDR*)lparam)->code == PSN_APPLY) {
			enum {
				BUFSIZE = 256
			};
			chaos_auto_array<char> buf(new char[BUFSIZE]);
			bool dirty = false;
			GetDlgItemText(dlg, IDC_BGM_SOUND_TYPE, buf.get(), BUFSIZE);
			chaos_string bgm_sound_type = buf.get();
			dirty |= instance()->sync_str_profile(
				"bgm_sound_type", bgm_sound_type);
			chaos_resource* res = chaos_resource::instance();
			chaos_string cd_drive_auto =
				res->get_resource_text(chaos_resource::cd_drive_auto);
			GetDlgItemText(dlg, IDC_CD_DRIVE, buf.get(), BUFSIZE);
			chaos_string cd_drive = buf.get();
			if (cd_drive == cd_drive_auto) {
				cd_drive = "";
			}
			dirty |= instance()->sync_str_profile(
				"cd_drive", cd_drive);
			for (int i = 0; i < 2; i++) {
				int id;
				chaos_string sound_volume_name;
				if (i == 0) {
					id = IDC_BGM_SOUND_VOLUME;
					sound_volume_name = "bgm_sound_volume";
				} else {
					id = IDC_EFFECT_SOUND_VOLUME;
					sound_volume_name = "effect_sound_volume";
				}
				dirty |= instance()->sync_int_profile(
					sound_volume_name,
					(int)SendDlgItemMessage(
						dlg, id,
						TBM_GETPOS,
						0,
						0));
			}
			if (dirty) {
				instance()->set_dirty(true);
			}
		}
		break;
	case WM_COMMAND:
		switch (LOWORD(wparam)) {
		case IDC_BGM_SOUND_TYPE:
			if ((HIWORD(wparam) == CBN_SELCHANGE) ||
				(HIWORD(wparam) == CBN_EDITCHANGE))
			{
				enum {
					BUFSIZE = 256
				};
				chaos_auto_array<char> buf(new char[BUFSIZE]);
				*buf = '\0';
				chaos_string bgm_sound_type;
				if (HIWORD(wparam) == CBN_SELCHANGE) {
					int sel = SendDlgItemMessage(
						dlg, IDC_BGM_SOUND_TYPE,
						CB_GETCURSEL,
						0, 0);
					SendDlgItemMessage(
						dlg, IDC_BGM_SOUND_TYPE,
						CB_GETLBTEXT,
						sel,
						(LPARAM)buf.get());
					bgm_sound_type = buf.get();
				} else {
					GetDlgItemText(dlg, IDC_BGM_SOUND_TYPE, buf.get(), BUFSIZE);
					bgm_sound_type = buf.get();
				}
				bool cdda = (chaos_utility::icompare(bgm_sound_type, "CD-DA") == 0);
				HWND cd_drive_combo = GetDlgItem(dlg, IDC_CD_DRIVE),
					drive_label = NULL,
					volume_slider = GetDlgItem(dlg, IDC_BGM_SOUND_VOLUME),
					volume_label = NULL,
					volume_min = NULL,
					volume_max = NULL;
				if (cd_drive_combo != NULL) {
					EnableWindow(cd_drive_combo, cdda? TRUE: FALSE);
					drive_label = GetWindow(cd_drive_combo, GW_HWNDPREV);
				}
				if (drive_label != NULL) {
					EnableWindow(drive_label, cdda? TRUE: FALSE);
				}
				if (volume_slider != NULL) {
					EnableWindow(volume_slider, !cdda? TRUE: FALSE);
					volume_label = GetWindow(volume_slider, GW_HWNDPREV);
					volume_min = GetWindow(volume_slider, GW_HWNDNEXT);
				}
				if (volume_label != NULL) {
					EnableWindow(volume_label, !cdda? TRUE: FALSE);
				}
				if (volume_min != NULL) {
					EnableWindow(volume_min, !cdda? TRUE: FALSE);
					volume_max = GetWindow(volume_min, GW_HWNDNEXT);
				}
				if (volume_max != NULL) {
					EnableWindow(volume_max, !cdda? TRUE: FALSE);
				}
			}
			break;
		}
		break;
	default:
		result = FALSE;
		break;
	}
	return result;
}

// font page procedure

BOOL CALLBACK chaos_config_dlg::font_dlgproc(
	HWND dlg, UINT message, WPARAM wparam, LPARAM lparam)
{
	BOOL result = FALSE;
	switch (message) {
	case WM_INITDIALOG:
		{ // dummy block
			LOGFONT lf;
			memset(&lf, 0, sizeof(lf));
			lf.lfCharSet = SHIFTJIS_CHARSET;
			HDC dc = GetDC(dlg);
			EnumFontFamiliesEx(
				dc,
				&lf,
				(FONTENUMPROC)enum_font_family_proc,
				(LPARAM)dlg,
				0);
			ReleaseDC(dlg, dc);
			chaos_string font_name =
				instance()->get_str_profile("font_name").c_str();
			if (SendDlgItemMessage(
					dlg, IDC_FONT_NAME,
					CB_SELECTSTRING,
					(WPARAM)-1,
					(LPARAM)font_name.c_str()) == CB_ERR)
			{
				SetDlgItemText(
					dlg, IDC_FONT_NAME,
					font_name.c_str());
			}
			SendDlgItemMessage(
				dlg, IDC_FONT_BOLD,
				BM_SETCHECK,
				instance()->get_bool_profile("font_bold")?
					BST_CHECKED: BST_UNCHECKED,
				0);
			SendDlgItemMessage(
				dlg, IDC_FONT_ANTI_ALIAS,
				BM_SETCHECK,
				instance()->get_bool_profile("font_anti_alias")?
					BST_CHECKED: BST_UNCHECKED,
				0);
			HWND sample_static = GetDlgItem(dlg, IDC_FONT_SAMPLE);
			if (sample_static != NULL) {
				SetWindowLong(
					sample_static, GWL_STYLE,
					(GetWindowLong(sample_static, GWL_STYLE) &
						~SS_TYPEMASK) | SS_OWNERDRAW);
			}
			result = FALSE;
		}
		break;
	case WM_NOTIFY:
		if (((NMHDR*)lparam)->code == PSN_APPLY) {
			enum {
				BUFSIZE = 256
			};
			chaos_auto_array<char> buf(new char[BUFSIZE]);
			bool dirty = false;
			GetDlgItemText(dlg, IDC_FONT_NAME, buf.get(), BUFSIZE);
			dirty |= instance()->sync_str_profile(
				"font_name", buf.get());
			dirty |= instance()->sync_bool_profile(
				"font_bold",
				SendDlgItemMessage(
					dlg, IDC_FONT_BOLD,
					BM_GETCHECK,
					0, 0) == BST_CHECKED);
			bool font_anti_alias =
				(SendDlgItemMessage(
					dlg, IDC_FONT_ANTI_ALIAS,
					BM_GETCHECK,
					0, 0) == BST_CHECKED);
			dirty |= instance()->sync_bool_profile(
				"font_anti_alias",
				font_anti_alias);
			dirty |= instance()->sync_bool_profile(
				"sys_font_anti_alias",
				font_anti_alias);
			if (dirty) {
				instance()->set_dirty(true);
			}
		}
		break;
	case WM_COMMAND:
		switch (LOWORD(wparam)) {
		case IDC_FONT_NAME:
			if ((HIWORD(wparam) == CBN_SELCHANGE) ||
				(HIWORD(wparam) == CBN_EDITCHANGE))
			{
				instance()->get_sample_image()->destroy();
				HWND sample_static = GetDlgItem(dlg, IDC_FONT_SAMPLE);
				if (sample_static != NULL) {
					InvalidateRect(sample_static, NULL, TRUE);
				}
			}
			break;
		case IDC_FONT_BOLD:
		case IDC_FONT_ANTI_ALIAS:
			if (HIWORD(wparam) == BN_CLICKED) {
				instance()->get_sample_image()->destroy();
				HWND sample_static = GetDlgItem(dlg, IDC_FONT_SAMPLE);
				if (sample_static != NULL) {
					InvalidateRect(sample_static, NULL, TRUE);
				}
			}
			break;
		case IDC_DEFAULT_FONT:
			if (HIWORD(wparam) == BN_CLICKED) {
				chaos_string font_name =
					chaos_resource::instance()->get_resource_text(
						chaos_resource::def_font_name);
				if (SendDlgItemMessage(
						dlg, IDC_FONT_NAME,
						CB_SELECTSTRING,
						(WPARAM)-1,
						(LPARAM)font_name.c_str()) == CB_ERR)
				{
					SetDlgItemText(
						dlg, IDC_FONT_NAME,
						font_name.c_str());
				}
				SendDlgItemMessage(
					dlg, IDC_FONT_BOLD,
					BM_SETCHECK,
					BST_CHECKED,
					0);
				SendDlgItemMessage(
					dlg, IDC_FONT_ANTI_ALIAS,
					BM_SETCHECK,
					BST_CHECKED,
					0);
				instance()->get_sample_image()->destroy();
				HWND sample_static = GetDlgItem(dlg, IDC_FONT_SAMPLE);
				if (sample_static != NULL) {
					InvalidateRect(sample_static, NULL, TRUE);
				}
			}
			break;
		}
		break;
	case WM_DRAWITEM:
		if (wparam == IDC_FONT_SAMPLE) {
			DRAWITEMSTRUCT* dis = (DRAWITEMSTRUCT*)lparam;
			chaos_image* image = instance()->get_sample_image();
			if (!image->is_valid()) {
				chaos_dim sample_dim(
					dis->rcItem.right-dis->rcItem.left,
					dis->rcItem.bottom-dis->rcItem.top);
				chaos_string font_name;
				{ // dummy block
					enum {
						BUFSIZE = 256
					};
					chaos_auto_array<char> buf(new char[BUFSIZE]);
					GetDlgItemText(dlg, IDC_FONT_NAME, buf.get(), BUFSIZE);
					font_name = buf.get();
				}
				bool font_bold = (SendDlgItemMessage(
						dlg, IDC_FONT_BOLD,
						BM_GETCHECK,
						0, 0) == BST_CHECKED),
					font_anti_alias = (SendDlgItemMessage(
						dlg, IDC_FONT_ANTI_ALIAS,
						BM_GETCHECK,
						0, 0) == BST_CHECKED);
				instance()->build_sample_image(
					sample_dim,
					font_name, font_bold, font_anti_alias);
			}
			if (image->is_valid()) {
				BITMAPINFO bmi;
				memset(&bmi, 0, sizeof(bmi));
				bmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
				bmi.bmiHeader.biWidth = image->get_width();
				bmi.bmiHeader.biHeight = -image->get_height();
				bmi.bmiHeader.biBitCount = 32;
				bmi.bmiHeader.biPlanes = 1;
				bmi.bmiHeader.biCompression = BI_RGB;
				SetDIBitsToDevice(
					dis->hDC,
					0, 0,
					image->get_width(), image->get_height(),
					0, 0,
					0, image->get_height(),
					image->get_bits(),
					&bmi, DIB_RGB_COLORS);
			}
		}
		break;
	default:
		result = FALSE;
		break;
	}
	return result;
}

// else page procedure

BOOL CALLBACK chaos_config_dlg::else_dlgproc(
	HWND dlg, UINT message, WPARAM /*wparam*/, LPARAM lparam)
{
	BOOL result = FALSE;
	switch (message) {
	case WM_INITDIALOG:
		{ // dummy block
			chaos_resource* res = chaos_resource::instance();
			SendDlgItemMessage(
				dlg, IDC_NOT_SHOW_STARTUP,
				BM_SETCHECK,
				!instance()->get_bool_profile("show_startup")?
					BST_CHECKED: BST_UNCHECKED,
				0);
			int i;
			for (i = 0; i < 2; i++) {
				chaos_resource::resource_item_type item =
					chaos_resource::item_none;
				switch (i) {
				case 0:
					item = chaos_resource::logmode_file;
					break;
				case 1:
					item = chaos_resource::logmode_window;
					break;
				}
				SendDlgItemMessage(
					dlg, IDC_LOG_MODE,
					CB_ADDSTRING,
					0,
					(LPARAM)(res->get_resource_text(
						item).c_str()));
			}
			SendDlgItemMessage(
				dlg, IDC_LOG_MODE,
				CB_SETCURSEL,
				instance()->get_int_profile("log_mode")-1,
				0);
			for (i = 0; i < 4; i++) {
				chaos_resource::resource_item_type item =
					chaos_resource::item_none;
				switch (i) {
				case 0:
					item = chaos_resource::loglv_none;
					break;
				case 1:
					item = chaos_resource::loglv_error;
					break;
				case 2:
					item = chaos_resource::loglv_warning;
					break;
				case 3:
					item = chaos_resource::loglv_report;
					break;
				}
				SendDlgItemMessage(
					dlg, IDC_LOG_LEVEL,
					CB_ADDSTRING,
					0,
					(LPARAM)(res->get_resource_text(
						item).c_str()));
			}
			SendDlgItemMessage(
				dlg, IDC_LOG_LEVEL,
				CB_SETCURSEL,
				instance()->get_int_profile("log_level"),
				0);
			HWND buf_spin = GetDlgItem(dlg, IDC_IMAGE_BUFFER_SPIN);
			SendMessage(
				buf_spin,
				UDM_SETRANGE,
				0, MAKELPARAM(20, 2));
			SetDlgItemInt(
				dlg, IDC_IMAGE_BUFFER,
				instance()->get_int_profile("sys_image_buffer_count"),
				FALSE);
			HWND step_spin = GetDlgItem(dlg, IDC_EFFECT_STEP_SPIN);
			SendMessage(
				step_spin,
				UDM_SETRANGE,
				0, MAKELPARAM(10, 1));
			int effect_step = instance()->get_int_profile("effect_step");
			if (effect_step < 1) {
				effect_step = 1;
			} else if (effect_step > 10) {
				effect_step = 10;
			}
			SetDlgItemInt(
				dlg, IDC_EFFECT_STEP,
				effect_step,
				FALSE);
			SendDlgItemMessage(
				dlg, IDC_EFFECT_CUT,
				BM_SETCHECK,
				instance()->get_bool_profile("effect_cut")?
					BST_CHECKED: BST_UNCHECKED,
				0);
		}
		result = FALSE;
		break;
	case WM_NOTIFY:
		if (((NMHDR*)lparam)->code == PSN_APPLY) {
			enum {
				BUFSIZE = 256
			};
			chaos_auto_array<char> buf(new char[BUFSIZE]);
			bool dirty = false;
			bool show_startup =
				(SendDlgItemMessage(
					dlg, IDC_NOT_SHOW_STARTUP,
					BM_GETCHECK,
					0, 0) != BST_CHECKED);
			dirty |= instance()->sync_bool_profile(
				"show_startup",
				show_startup);
			dirty |= instance()->sync_int_profile(
				"log_mode",
				(int)SendDlgItemMessage(
					dlg, IDC_LOG_MODE,
					CB_GETCURSEL,
					0, 0)+1);
			dirty |= instance()->sync_int_profile(
				"log_level",
				(int)SendDlgItemMessage(
					dlg, IDC_LOG_LEVEL,
					CB_GETCURSEL,
					0, 0));
			int buf_count = (int)GetDlgItemInt(
				dlg, IDC_IMAGE_BUFFER,
				NULL, TRUE);
			if (buf_count < 2) {
				buf_count = 2;
			} else if (buf_count > 20) {
				buf_count = 20;
			}
			dirty |= instance()->sync_int_profile(
				"sys_image_buffer_count",
				buf_count);
			int effect_step = (int)GetDlgItemInt(
				dlg, IDC_EFFECT_STEP,
				NULL, TRUE);
			if (effect_step < 1) {
				effect_step = 1;
			} else if (effect_step > 10) {
				effect_step = 10;
			}
			dirty |= instance()->sync_int_profile(
				"effect_step",
				effect_step);
			bool effect_cut =
				(SendDlgItemMessage(
					dlg, IDC_EFFECT_CUT,
					BM_GETCHECK,
					0, 0) == BST_CHECKED);
			dirty |= instance()->sync_bool_profile(
				"effect_cut",
				effect_cut);
			if (dirty) {
				instance()->set_dirty(true);
			}
		}
		break;
	default:
		result = FALSE;
		break;
	}
	return result;
}

#elif defined(CHAOS_GUI_GTK)

// font entry focus in event

gboolean chaos_config_dlg::font_focus_in_event(
	GtkWidget* /*font_entry*/,
	GdkEventFocus* /*event*/,
	gpointer data)
{
	chaos_config_dlg* dlg = (chaos_config_dlg*)data;
	dlg->set_font_dirty(false);
	return FALSE;
}

// font entry focus out event

gboolean chaos_config_dlg::font_focus_out_event(
	GtkWidget* /*font_entry*/,
	GdkEventFocus* /*event*/,
	gpointer data)
{
	chaos_config_dlg* dlg = (chaos_config_dlg*)data;
	if (dlg->is_font_dirty()) {
		dlg->get_sample_image()->destroy();
		GtkWidget* font_sample_area = dlg->get_widget("font_sample");
		if (font_sample_area != NULL) {
			gtk_widget_queue_draw(font_sample_area);
		}
	}
	return FALSE;
}

// font entry changed

void chaos_config_dlg::font_changed(
	GtkWidget* /*font_entry*/,
	gpointer data)
{
	chaos_config_dlg* dlg = (chaos_config_dlg*)data;
	dlg->set_font_dirty(true);
}

// default name button clicked

void chaos_config_dlg::default_name_clicked(
	GtkWidget* /*default_name_button*/,
	gpointer data)
{
	chaos_config_dlg* dlg = (chaos_config_dlg*)data;
	chaos_resource* res = chaos_resource::instance();
	GtkWidget* lastname_entry = dlg->get_widget("lastname");
	if (lastname_entry != NULL) {
		gtk_entry_set_text(
			GTK_ENTRY(lastname_entry),
			chaos_utility::sjis_to_gui(
				res->get_resource_text(
					chaos_resource::def_lastname)).c_str());
	}
	GtkWidget* firstname_entry = dlg->get_widget("firstname");
	if (firstname_entry != NULL) {
		gtk_entry_set_text(
			GTK_ENTRY(firstname_entry),
			chaos_utility::sjis_to_gui(
				res->get_resource_text(
					chaos_resource::def_firstname)).c_str());
	}
	GtkWidget* nickname_entry = dlg->get_widget("nickname");
	if (nickname_entry != NULL) {
		gtk_entry_set_text(
			GTK_ENTRY(nickname_entry),
			chaos_utility::sjis_to_gui(
				res->get_resource_text(
					chaos_resource::def_nickname)).c_str());
	}
	GtkWidget* postfix_entry = dlg->get_widget("postfix");
	if (postfix_entry != NULL) {
		gtk_entry_set_text(
			GTK_ENTRY(postfix_entry),
			chaos_utility::sjis_to_gui(
				res->get_resource_text(
					chaos_resource::def_postfix)).c_str());
	}
}

// sound bgm sound type changed

void chaos_config_dlg::sound_bgm_sound_type_changed(
	GtkWidget* sound_bgm_sound_type_entry,
	gpointer data)
{
	chaos_config_dlg* dlg = (chaos_config_dlg*)data;
	chaos_string text = chaos_utility::gui_to_sjis(
		gtk_entry_get_text(GTK_ENTRY(sound_bgm_sound_type_entry)));
	bool cdda = (chaos_utility::icompare(text, "CD-DA") == 0);
	GtkWidget* cd_drive_combo =
		dlg->get_widget("cd_drive");
	if (cd_drive_combo != NULL) {
		gtk_widget_set_sensitive(
			cd_drive_combo,
			cdda? TRUE: FALSE);
	}
	GtkWidget* cd_drive_label =
		dlg->get_widget("cd_drive_label");
	if (cd_drive_label != NULL) {
		gtk_widget_set_sensitive(
			cd_drive_label,
			cdda? TRUE: FALSE);
	}
	GtkWidget* bgm_sound_volume_scale =
		dlg->get_widget("bgm_sound_volume");
	if (bgm_sound_volume_scale != NULL) {
		gtk_widget_set_sensitive(
			bgm_sound_volume_scale,
			!cdda? TRUE: FALSE);
	}
	GtkWidget* bgm_sound_volume_label =
		dlg->get_widget("bgm_sound_volume_label");
	if (bgm_sound_volume_label != NULL) {
		gtk_widget_set_sensitive(
			bgm_sound_volume_label,
			!cdda? TRUE: FALSE);
	}
}

// font select button clicked

void chaos_config_dlg::font_select_clicked(
	GtkWidget* /*font_select_button*/,
	gpointer data)
{
	chaos_resource* res = chaos_resource::instance();
	chaos_config_dlg* dlg = (chaos_config_dlg*)data;
	dlg->set_selected_font("");

#ifdef CHAOS_FONT_GTK

	chaos_string font_kind_name = "font_xlfd";

#elif defined(CHAOS_FONT_PANGO)

	chaos_string font_kind_name = "font_pango";

#endif // CHAOS_FONT
	GtkWidget* font_entry = dlg->get_widget(font_kind_name);
	chaos_dlg font_dlg;
	GtkWidget* fontsel_dialog = gtk_font_selection_dialog_new("");
	gtk_widget_ref(fontsel_dialog);
	font_dlg.set_dialog(fontsel_dialog);
	font_dlg.register_widget(
		"cancel",
		GTK_FONT_SELECTION_DIALOG(fontsel_dialog)->cancel_button);
	font_dlg.init_dialog_default(
		res->get_resource_text(
			chaos_resource::caption_font_select));
	gtk_signal_connect(
		GTK_OBJECT(GTK_FONT_SELECTION_DIALOG(fontsel_dialog)->ok_button),
			"clicked",
		GTK_SIGNAL_FUNC(fontsel_dialog_ok_clicked), &font_dlg);
	gtk_font_selection_dialog_set_preview_text(
		GTK_FONT_SELECTION_DIALOG(fontsel_dialog),
		chaos_utility::sjis_to_gui(
			res->get_resource_text(
				chaos_resource::font_sample_text)).c_str());
	if (font_entry != NULL) {
		chaos_string font =
			chaos_utility::gui_to_sjis(
				gtk_entry_get_text(
					GTK_ENTRY(font_entry)));

#ifdef CHAOS_FONT_GTK

		font = chaos_message_imager::normalize_font_xlfd(
			font,
			dlg->get_int_profile("font_size"));

#elif defined(CHAOS_FONT_PANGO)

		font = chaos_message_imager::normalize_font_pango(
			font,
			dlg->get_int_profile("font_size"));

#endif // CHAOS_FONT

		gtk_font_selection_dialog_set_font_name(
			GTK_FONT_SELECTION_DIALOG(fontsel_dialog),
			chaos_utility::sjis_to_gui(font).c_str());
	}
	gtk_widget_show(fontsel_dialog);
	gtk_grab_add(fontsel_dialog);
	push_current_modaling(fontsel_dialog);
	gtk_main();
	pop_current_modaling();
	bool result = font_dlg.get_dialog_result();
	if (result) {
		if (font_entry != NULL) {
			chaos_string font = dlg->get_selected_font();

#ifdef CHAOS_FONT_GTK

			font = chaos_message_imager::normalize_font_xlfd(
				font,
				-1);

#elif defined(CHAOS_FONT_PANGO)

			font = chaos_message_imager::normalize_font_pango(
				font,
				-1);

#endif // CHAOS_FONT

			gtk_entry_set_text(
				GTK_ENTRY(font_entry),
				chaos_utility::sjis_to_gui(font).c_str());
			dlg->get_sample_image()->destroy();
			GtkWidget* font_sample_area = dlg->get_widget("font_sample");
			if (font_sample_area != NULL) {
				gtk_widget_queue_draw(font_sample_area);
			}
		}
	}
	font_dlg.destroy_widgets();
	gtk_grab_remove(fontsel_dialog);
	gtk_widget_unref(fontsel_dialog);
}

// font anti alias check button toggled

void chaos_config_dlg::font_anti_alias_toggled(
	GtkWidget* /*font_anti_alias_check*/,
	gpointer data)
{
	chaos_config_dlg* dlg = (chaos_config_dlg*)data;
	dlg->get_sample_image()->destroy();
	GtkWidget* font_sample_area = dlg->get_widget("font_sample");
	if (font_sample_area != NULL) {
		gtk_widget_queue_draw(font_sample_area);
	}
}

// default font button clicked

void chaos_config_dlg::default_font_clicked(
	GtkWidget* /*default_font_button*/,
	gpointer data)
{
	chaos_config_dlg* dlg = (chaos_config_dlg*)data;

#ifdef CHAOS_FONT_GTK

	chaos_string
		font = chaos_resource::instance()->get_resource_text(
			chaos_resource::def_font_xlfd),
		font_kind_name = "font_xlfd";

#elif defined(CHAOS_FONT_PANGO)

	chaos_string
		font = chaos_resource::instance()->get_resource_text(
			chaos_resource::def_font_pango),
		font_kind_name = "font_pango";

#endif // CHAOS_FONT

	GtkWidget* font_entry = dlg->get_widget(font_kind_name);
	if (font_entry != NULL) {
		gtk_entry_set_text(
			GTK_ENTRY(font_entry),
			chaos_utility::sjis_to_gui(
				font).c_str());
	}
	GtkWidget* font_anti_alias_check = dlg->get_widget("font_anti_alias");
	if (font_anti_alias_check != NULL) {
		gtk_toggle_button_set_active(
			GTK_TOGGLE_BUTTON(font_anti_alias_check),
			TRUE);
	}
	dlg->get_sample_image()->destroy();
	GtkWidget* font_sample_area = dlg->get_widget("font_sample");
	if (font_sample_area != NULL) {
		gtk_widget_queue_draw(font_sample_area);
	}
}

// font sample expose event

gboolean chaos_config_dlg::font_sample_expose_event(
	GtkWidget* font_sample_area,
	GdkEventExpose* /*event*/,
	gpointer data)
{
	chaos_config_dlg* dlg = (chaos_config_dlg*)data;
	chaos_dim area_dim(
		font_sample_area->allocation.width,
		font_sample_area->allocation.height);
	chaos_image* image = dlg->get_sample_image();
	GdkPixmap* pixmap = dlg->get_sample_pixmap();
	if (image->is_valid()) {
		if (image->get_dim() != area_dim) {
			image->destroy();
		}
	}
	if (!image->is_valid()) {
		chaos_dim sample_dim = area_dim;
		chaos_string font;

#ifdef CHAOS_FONT_GTK

		chaos_string font_kind_name = "font_xlfd";

#elif defined(CHAOS_FONT_PANGO)

		chaos_string font_kind_name = "font_pango";

#endif // CHAOS_FONT

		GtkWidget* font_entry = dlg->get_widget(font_kind_name);
		if (font_entry != NULL) {
			font = chaos_utility::gui_to_sjis(
				gtk_entry_get_text(GTK_ENTRY(font_entry)));
		}
		bool font_anti_alias = true;
		GtkWidget* font_anti_alias_check = dlg->get_widget("font_anti_alias");
		if (font_anti_alias_check != NULL) {
			font_anti_alias = (gtk_toggle_button_get_active(
				GTK_TOGGLE_BUTTON(font_anti_alias_check)) != FALSE);
		}
		dlg->build_sample_image(
			sample_dim,
			font, false, font_anti_alias);
		if (pixmap != NULL) {
			gdk_pixmap_unref(pixmap);
			pixmap = NULL;
			dlg->set_sample_pixmap(pixmap);
		}
	}
	GdkGC* gc = gdk_gc_new(font_sample_area->window);
	if ((pixmap == NULL) && image->is_valid()) {
		pixmap = gdk_pixmap_new(
			font_sample_area->window,
			image->get_width(), image->get_height(), -1);
		if (pixmap != NULL) {
			GdkImage* img = gdk_image_new(
				GDK_IMAGE_NORMAL,
				gdk_window_get_visual(font_sample_area->window),
				image->get_width(), image->get_height());
			if (img != NULL) {
				for (int y = 0; y < image->get_height(); y++) {
					for (int x = 0; x < image->get_width(); x++) {
						chaos_uint32 col = image->get_pixel(x, y);
						GdkColor color;
						color.red =
							(gushort)(CHAOS_RGBA_RED(col)*0x0101);
						color.green =
							(gushort)(CHAOS_RGBA_GREEN(col)*0x0101);
						color.blue =
							(gushort)(CHAOS_RGBA_BLUE(col)*0x0101);
						gdk_color_alloc(
							gdk_colormap_get_system(),
							&color);
						gdk_image_put_pixel(
							img,
							x, y,
							color.pixel);
					}
				}
				gdk_draw_image(
					pixmap,
					gc,
					img,
					0, 0,
					0, 0,
					image->get_width(), image->get_height());
				gdk_image_destroy(img);
			}
			dlg->set_sample_pixmap(pixmap);
		}
	}
	if ((pixmap != NULL) && image->is_valid()) {
		gdk_draw_pixmap(
			font_sample_area->window,
			gc,
			pixmap,
			0, 0,
			0, 0,
			image->get_width(), image->get_height());
	}
	return FALSE;
}

// font selection dialog ok button clicked

void chaos_config_dlg::fontsel_dialog_ok_clicked(
	GtkWidget* /*ok_button*/,
	gpointer data)
{
	chaos_dlg* dlg = (chaos_dlg*)data;
	GtkWidget* fontsel_dialog = dlg->get_dialog();
	chaos_string font =
		chaos_utility::gui_to_sjis(
			gtk_font_selection_dialog_get_font_name(
				GTK_FONT_SELECTION_DIALOG(fontsel_dialog)));
	instance()->set_selected_font(font);
	dlg->set_dialog_result(true);
	dlg->on_ok_clicked();
	gtk_widget_destroy(dlg->get_dialog());
}

#endif // CHAOS_GUI

////////////////////////////////////////////////////////////////////////////////
// font

#ifdef CHAOS_GUI_WIN32

// enum font family procedure

int CALLBACK chaos_config_dlg::enum_font_family_proc(
	ENUMLOGFONTEX* pelfe,
	NEWTEXTMETRICEX* /*pntme*/,
	int /*font_type*/,
	LPARAM lparam)
{
	if (pelfe->elfLogFont.lfFaceName[0] != '@') {
		HWND dlg = (HWND)lparam;
		if (SendDlgItemMessage(
				dlg, IDC_FONT_NAME,
				CB_FINDSTRINGEXACT,
				(WPARAM)-1,
				(LPARAM)pelfe->elfLogFont.lfFaceName) == CB_ERR)
		{
			SendDlgItemMessage(
				dlg, IDC_FONT_NAME,
				CB_ADDSTRING,
				0,
				(LPARAM)pelfe->elfLogFont.lfFaceName);
		}
	}
	return TRUE;
}

#endif // CHAOS_GUI

// build sample image

bool chaos_config_dlg::build_sample_image(
	chaos_dim sample_dim,
	const chaos_string& font_name,
	bool font_bold,
	bool font_anti_alias)
{
	if (sample_dim.empty()) {
		return false;
	}
	if (sample_image == NULL) {
		return false;
	}
	if (!sample_image->create(
			sample_dim,
			true,
			CHAOS_RGBA(64, 64, 64, 0)))
	{
		return false;
	}
	chaos_message_imager message_imager;
	if (!message_imager.initialize(sample_dim)) {
		return false;
	}
	int font_size = get_int_profile("font_size");
	if (font_size < 1) {
		font_size = 28;
	}
	int length = (sample_dim.width)/font_size-1;
	if (length < 1) {
		length = 1;
	}
	message_imager.set_font_name(font_name);
	message_imager.set_font_size(font_size);
	message_imager.set_font_bold(font_bold);
	message_imager.set_font_anti_alias(font_anti_alias);
	chaos_string message_text;
	message_text = chaos_message_imager::encode_length_info(
		length);
	message_text += chaos_message_imager::encode_position_info(
		font_size/2, font_size/2);
	message_text += chaos_resource::instance()->get_resource_text(
		chaos_resource::font_sample_text),
	message_imager.update_message_text(
		message_text,
		NULL);
	sample_image->alpha_blt(
		message_imager.get_message_image(),
		chaos_pos(0, 0));
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// operation

// modal

bool chaos_config_dlg::do_modal(chaos_wnd_handle parent_wnd) {
	ASSERT(profile != NULL);
	if (profile == NULL) {
		return false;
	}
	set_dirty(false);
	sample_image = new chaos_image;
	bool result;

#ifdef CHAOS_GUI_WIN32

	HINSTANCE process_instance = (HINSTANCE)GetModuleHandle(NULL);
	PROPSHEETPAGE apsp[4];
	memset(apsp, 0x00, sizeof(apsp));
	int page_count = sizeof(apsp)/sizeof(apsp[0]);
	for (int i = 0; i < page_count; i++) {
		apsp[i].dwSize = sizeof(PROPSHEETPAGE);
		apsp[i].hInstance = process_instance;
	}
	apsp[0].pszTemplate = MAKEINTRESOURCE(IDD_CONFIG_GENERAL);
	apsp[0].pfnDlgProc = general_dlgproc;
	apsp[1].pszTemplate = MAKEINTRESOURCE(IDD_CONFIG_SOUND);
	apsp[1].pfnDlgProc = sound_dlgproc;
	apsp[2].pszTemplate = MAKEINTRESOURCE(IDD_CONFIG_FONT);
	apsp[2].pfnDlgProc = font_dlgproc;
	apsp[3].pszTemplate = MAKEINTRESOURCE(IDD_CONFIG_ELSE);
	apsp[3].pfnDlgProc = else_dlgproc;
	PROPSHEETHEADER psh;
	memset(&psh, 0x00, sizeof(psh));
	psh.dwSize = sizeof(PROPSHEETHEADER);
	psh.dwFlags = PSH_PROPSHEETPAGE | PSH_NOAPPLYNOW;
	psh.hwndParent = parent_wnd;
	psh.hInstance = process_instance;
	psh.pszCaption =
		chaos_resource::instance()->get_resource_text(
			chaos_resource::main_menu_config).c_str();
	psh.nPages = page_count;
	psh.ppsp = apsp;
	result = (PropertySheet(&psh) == IDOK);

#elif defined(CHAOS_GUI_GTK)

	sample_pixmap = NULL;
	font_dirty = false;
	widgets.clear();
	dialog = gtk_dialog_new();
	gtk_widget_ref(dialog);
	build_widgets(
		GTK_DIALOG(dialog)->action_area,
		chaos_resource::instance()->get_resource_text(
			chaos_resource::dlg_std_action_area));
	build_widgets(
		GTK_DIALOG(dialog)->vbox,
		chaos_resource::instance()->get_resource_text(
			chaos_resource::dlg_config));
	init_dialog_default(
		chaos_resource::instance()->get_resource_text(
			chaos_resource::main_menu_config));
	initialize_widgets();
	gtk_widget_show(dialog);
	gtk_grab_add(dialog);
	gtk_window_present(GTK_WINDOW(dialog));
	push_current_modaling(dialog);
	gtk_main();
	pop_current_modaling();
	result = dialog_result;
	destroy_widgets();
	gtk_grab_remove(dialog);
	gtk_widget_unref(dialog);
	dialog = NULL;
	if (sample_pixmap != NULL) {
		gdk_pixmap_unref(sample_pixmap);
		sample_pixmap = NULL;
	}
	parent_wnd;

#endif // CHAOS_GUI

	delete sample_image;
	sample_image = NULL;
	return result;
}

#ifdef CHAOS_GUI_GTK

// on ok clicked

// initialize widgets

void chaos_config_dlg::initialize_widgets() {
	int i;
	chaos_resource* res = chaos_resource::instance();
	// general
	GtkWidget* full_screen_check = get_widget("full_screen");
	if (full_screen_check != NULL) {
		gtk_toggle_button_set_active(
			GTK_TOGGLE_BUTTON(full_screen_check),
			get_bool_profile("full_screen")?
				TRUE: FALSE);
	}
	GtkWidget* text_speed_combo = get_widget("text_speed");
	if (text_speed_combo != NULL) {
#if !GTK_CHECK_VERSION(2, 4, 0)
		for (i = 0; i < 6; i++) {
			chaos_resource::resource_item_type item =
				chaos_resource::item_none;
			switch (i) {
			case 0:
				item = chaos_resource::tspeed_superslow;
				break;
			case 1:
				item = chaos_resource::tspeed_slow;
				break;
			case 2:
				item = chaos_resource::tspeed_normal;
				break;
			case 3:
				item = chaos_resource::tspeed_fast;
				break;
			case 4:
				item = chaos_resource::tspeed_superfast;
				break;
			case 5:
				item = chaos_resource::tspeed_instant;
				break;
			}
			GtkWidget* label = gtk_list_item_new_with_label(
				chaos_utility::sjis_to_gui(
					res->get_resource_text(item)).c_str());
			gtk_container_add(
				GTK_CONTAINER(GTK_COMBO(text_speed_combo)->list),
				label);
			gtk_widget_show(label);
		}
		int sel = get_int_profile("text_speed")-1;
		if (sel < 0) {
			sel = 0;
		} else if (sel > 5) {
			sel = 5;
		}
		gtk_list_select_item(
			GTK_LIST(GTK_COMBO(text_speed_combo)->list),
			sel);
#else // GTK_CHECK_VERSION(2, 4, 0)
		for (i = 0; i < 6; i++) {
			chaos_resource::resource_item_type item =
				chaos_resource::item_none;
			switch (i) {
			case 0:
				item = chaos_resource::tspeed_superslow;
				break;
			case 1:
				item = chaos_resource::tspeed_slow;
				break;
			case 2:
				item = chaos_resource::tspeed_normal;
				break;
			case 3:
				item = chaos_resource::tspeed_fast;
				break;
			case 4:
				item = chaos_resource::tspeed_superfast;
				break;
			case 5:
				item = chaos_resource::tspeed_instant;
				break;
			}
			gtk_combo_box_append_text(
				GTK_COMBO_BOX(text_speed_combo),
				chaos_utility::sjis_to_gui(
					res->get_resource_text(item)).c_str());
		}
		int sel = get_int_profile("text_speed")-1;
		if (sel < 0) {
			sel = 0;
		} else if (sel > 5) {
			sel = 5;
		}
		gtk_combo_box_set_active(
			GTK_COMBO_BOX(text_speed_combo),
			sel);
#endif // GTK_CHECK_VERSION
	}
	GtkWidget* skip_speed_combo = get_widget("skip_speed");
	if (skip_speed_combo != NULL) {
#if !GTK_CHECK_VERSION(2, 4, 0)
		for (i = 0; i < 3; i++) {
			chaos_resource::resource_item_type item =
				chaos_resource::item_none;
			switch (i) {
			case 0:
				item = chaos_resource::sspeed_normal;
				break;
			case 1:
				item = chaos_resource::sspeed_fast;
				break;
			case 2:
				item = chaos_resource::sspeed_jump;
				break;
			}
			GtkWidget* label = gtk_list_item_new_with_label(
				chaos_utility::sjis_to_gui(
					res->get_resource_text(item)).c_str());
			gtk_container_add(
				GTK_CONTAINER(GTK_COMBO(skip_speed_combo)->list),
				label);
			gtk_widget_show(label);
		}
		int sel = get_int_profile("skip_speed")-1;
		if (sel < 0) {
			sel = 0;
		} else if (sel > 2) {
			sel = 2;
		}
		gtk_list_select_item(
			GTK_LIST(GTK_COMBO(skip_speed_combo)->list),
			sel);
#else // GTK_CHECK_VERSION(2, 4, 0)
		for (i = 0; i < 3; i++) {
			chaos_resource::resource_item_type item =
				chaos_resource::item_none;
			switch (i) {
			case 0:
				item = chaos_resource::sspeed_normal;
				break;
			case 1:
				item = chaos_resource::sspeed_fast;
				break;
			case 2:
				item = chaos_resource::sspeed_jump;
				break;
			}
			gtk_combo_box_append_text(
				GTK_COMBO_BOX(skip_speed_combo),
				chaos_utility::sjis_to_gui(
					res->get_resource_text(item)).c_str());
		}
		int sel = get_int_profile("skip_speed")-1;
		if (sel < 0) {
			sel = 0;
		} else if (sel > 2) {
			sel = 2;
		}
		gtk_combo_box_set_active(
			GTK_COMBO_BOX(skip_speed_combo),
			sel);
#endif // GTK_CHECK_VERSION
	}
	GtkWidget* lastname_entry = get_widget("lastname");
	if (lastname_entry != NULL) {
		gtk_entry_set_text(
			GTK_ENTRY(lastname_entry),
			chaos_utility::sjis_to_gui(
				get_str_profile("@S")).c_str());
	}
	GtkWidget* firstname_entry = get_widget("firstname");
	if (firstname_entry != NULL) {
		gtk_entry_set_text(
			GTK_ENTRY(firstname_entry),
			chaos_utility::sjis_to_gui(
				get_str_profile("@M")).c_str());
	}
	GtkWidget* nickname_entry = get_widget("nickname");
	if (nickname_entry != NULL) {
		gtk_entry_set_text(
			GTK_ENTRY(nickname_entry),
			chaos_utility::sjis_to_gui(
				get_str_profile("@N")).c_str());
	}
	GtkWidget* postfix_entry = get_widget("postfix");
	if (postfix_entry != NULL) {
		gtk_entry_set_text(
			GTK_ENTRY(postfix_entry),
			chaos_utility::sjis_to_gui(
				get_str_profile("@K")).c_str());
	}
	GtkWidget* default_name_button = get_widget("default_name");
	if (default_name_button != NULL) {
		gtk_signal_connect(
			GTK_OBJECT(default_name_button), "clicked",
			GTK_SIGNAL_FUNC(default_name_clicked), this);
	}
	// sound
	GtkWidget* bgm_sound_type_combo =
		get_widget("bgm_sound_type");
	if (bgm_sound_type_combo != NULL) {
		static const char* sound_types[] = {
			"CD-DA", ".mp3", ".ogg", ".wav"
		};
#if !GTK_CHECK_VERSION(2, 4, 0)
		gtk_combo_set_value_in_list(
			GTK_COMBO(bgm_sound_type_combo),
			FALSE,
			TRUE);
		for (
			int i = 0;
			i < sizeof(sound_types)/sizeof(sound_types[0]);
			i++)
		{
			GtkWidget* label = gtk_list_item_new_with_label(
				chaos_utility::sjis_to_gui(sound_types[i]).c_str());
			gtk_container_add(
				GTK_CONTAINER(GTK_COMBO(bgm_sound_type_combo)->list),
				label);
			gtk_widget_show(label);
		}
		chaos_string bgm_sound_type =
			get_str_profile("bgm_sound_type");
		gtk_entry_set_text(
			GTK_ENTRY(GTK_COMBO(bgm_sound_type_combo)->entry),
			chaos_utility::sjis_to_gui(bgm_sound_type).c_str());
		gtk_signal_connect(
			GTK_OBJECT(GTK_COMBO(bgm_sound_type_combo)->entry), "changed",
			GTK_SIGNAL_FUNC(sound_bgm_sound_type_changed), this);
#else // GTK_CHECK_VERSION(2, 4, 0)
		for (
			int i = 0;
			i < sizeof(sound_types)/sizeof(sound_types[0]);
			i++)
		{
			gtk_combo_box_append_text(
				GTK_COMBO_BOX(bgm_sound_type_combo),
				chaos_utility::sjis_to_gui(sound_types[i]).c_str());
		}
		chaos_string bgm_sound_type =
			get_str_profile("bgm_sound_type");
		gtk_entry_set_text(
			GTK_ENTRY(gtk_bin_get_child(GTK_BIN(bgm_sound_type_combo))),
			chaos_utility::sjis_to_gui(bgm_sound_type).c_str());
		gtk_signal_connect(
			GTK_OBJECT(gtk_bin_get_child(GTK_BIN(bgm_sound_type_combo))),
				"changed",
			GTK_SIGNAL_FUNC(sound_bgm_sound_type_changed), this);
#endif // GTK_CHECK_VERSION
	}
	GtkWidget* cd_drive_combo = get_widget("cd_drive");
	if (cd_drive_combo != NULL) {
#if !GTK_CHECK_VERSION(2, 4, 0)
		gtk_combo_set_value_in_list(
			GTK_COMBO(cd_drive_combo),
			FALSE,
			TRUE);
		chaos_string cd_drive_auto =
			res->get_resource_text(chaos_resource::cd_drive_auto);
		std::vector<chaos_string> cd_drives;
		chaos_cd_player::get_cd_drives(cd_drives);
		chaos_string cd_drive;
		for (i = 0; i <= (int)cd_drives.size(); i++) {
			if (i == 0) {
				cd_drive = cd_drive_auto;
			} else {
				cd_drive = cd_drives[i-1];
			}
			GtkWidget* label = gtk_list_item_new_with_label(
				chaos_utility::sjis_to_gui(cd_drive).c_str());
			gtk_container_add(
				GTK_CONTAINER(GTK_COMBO(cd_drive_combo)->list),
				label);
			gtk_widget_show(label);
		}
		cd_drive = get_str_profile("cd_drive");
		if (cd_drive.empty()) {
			cd_drive = cd_drive_auto;
		}
		gtk_entry_set_text(
			GTK_ENTRY(GTK_COMBO(cd_drive_combo)->entry),
			chaos_utility::sjis_to_gui(cd_drive).c_str());
#else // GTK_CHECK_VERSION(2, 4, 0)
		chaos_string cd_drive_auto =
			res->get_resource_text(chaos_resource::cd_drive_auto);
		std::vector<chaos_string> cd_drives;
		chaos_cd_player::get_cd_drives(cd_drives);
		chaos_string cd_drive;
		for (i = 0; i <= (int)cd_drives.size(); i++) {
			if (i == 0) {
				cd_drive = cd_drive_auto;
			} else {
				cd_drive = cd_drives[i-1];
			}
			gtk_combo_box_append_text(
				GTK_COMBO_BOX(cd_drive_combo),
				chaos_utility::sjis_to_gui(cd_drive).c_str());
		}
		cd_drive = get_str_profile("cd_drive");
		if (cd_drive.empty()) {
			cd_drive = cd_drive_auto;
		}
		gtk_entry_set_text(
			GTK_ENTRY(gtk_bin_get_child(GTK_BIN(cd_drive_combo))),
			chaos_utility::sjis_to_gui(cd_drive).c_str());
#endif // GTK_CHECK_VERSION
	}
	for (int types = 0; types < 2; types++) {
		chaos_string sound_volume_name = (types == 0)?
			"bgm_sound_volume":
			"effect_sound_volume";
		GtkWidget* sound_volume_scale =
			get_widget(sound_volume_name);
		if (sound_volume_scale != NULL) {
			GtkAdjustment* adjustment =
				(GtkAdjustment*)gtk_adjustment_new(
					get_int_profile(sound_volume_name),
					0, 101,
					1, 5,
					1);
			gtk_scale_set_digits(
				GTK_SCALE(sound_volume_scale),
				0);
			gtk_range_set_adjustment(
				GTK_RANGE(sound_volume_scale),
				adjustment);
		}
	}
	if (bgm_sound_type_combo != NULL) {
#if !GTK_CHECK_VERSION(2, 4, 0)
		sound_bgm_sound_type_changed(
			GTK_COMBO(bgm_sound_type_combo)->entry,
			this);
#else // GTK_CHECK_VERSION(2, 4, 0)
		sound_bgm_sound_type_changed(
			gtk_bin_get_child(GTK_BIN(bgm_sound_type_combo)),
			this);
#endif // GTK_CHECK_VERSION
	}
	// font

#ifdef CHAOS_FONT_GTK

	chaos_string font_kind_name = "font_xlfd";

#elif defined(CHAOS_FONT_PANGO)

	chaos_string font_kind_name = "font_pango";

#endif // CHAOS_FONT

	GtkWidget* font_entry = get_widget(font_kind_name);
	if (font_entry != NULL) {
		gtk_entry_set_text(
			GTK_ENTRY(font_entry),
			chaos_utility::sjis_to_gui(
				get_str_profile(font_kind_name)).c_str());
		gtk_signal_connect_after(
			GTK_OBJECT(font_entry), "focus_in_event",
			GTK_SIGNAL_FUNC(font_focus_in_event), this);
		gtk_signal_connect_after(
			GTK_OBJECT(font_entry), "focus_out_event",
			GTK_SIGNAL_FUNC(font_focus_out_event), this);
		gtk_signal_connect(
			GTK_OBJECT(font_entry), "changed",
			GTK_SIGNAL_FUNC(font_changed), this);
	}
	GtkWidget* font_select_button = get_widget("font_select");
	if (font_select_button != NULL) {
		gtk_signal_connect(
			GTK_OBJECT(font_select_button), "clicked",
			GTK_SIGNAL_FUNC(font_select_clicked), this);
	}
	GtkWidget* font_anti_alias_check = get_widget("font_anti_alias");
	if (font_anti_alias_check != NULL) {
		gtk_toggle_button_set_active(
			GTK_TOGGLE_BUTTON(font_anti_alias_check),
			get_bool_profile("font_anti_alias")?
				TRUE: FALSE);
		gtk_signal_connect(
			GTK_OBJECT(font_anti_alias_check),
			"toggled",
			GTK_SIGNAL_FUNC(font_anti_alias_toggled),
			this);
	}
	GtkWidget* default_font_button = get_widget("default_font");
	if (default_font_button != NULL) {
		gtk_signal_connect(
			GTK_OBJECT(default_font_button), "clicked",
			GTK_SIGNAL_FUNC(default_font_clicked), this);
	}
	GtkWidget* font_sample_area = get_widget("font_sample");
	if (font_sample_area != NULL) {
		gtk_signal_connect(
			GTK_OBJECT(font_sample_area), "expose_event",
			GTK_SIGNAL_FUNC(font_sample_expose_event), this);
	}
	// else
	GtkWidget* not_show_startup_check = get_widget("not_show_startup");
	if (not_show_startup_check != NULL) {
		gtk_toggle_button_set_active(
			GTK_TOGGLE_BUTTON(not_show_startup_check),
			!get_bool_profile("show_startup")?
				TRUE: FALSE);
	}
	GtkWidget* log_mode_combo = get_widget("log_mode");
	if (log_mode_combo != NULL) {
#if !GTK_CHECK_VERSION(2, 4, 0)
		for (i = 0; i < 2; i++) {
			chaos_resource::resource_item_type item =
				chaos_resource::item_none;
			switch (i) {
			case 0:
				item = chaos_resource::logmode_file;
				break;
			case 1:
				item = chaos_resource::logmode_window;
				break;
			}
			GtkWidget* label = gtk_list_item_new_with_label(
				chaos_utility::sjis_to_gui(
					res->get_resource_text(item)).c_str());
			gtk_container_add(
				GTK_CONTAINER(GTK_COMBO(log_mode_combo)->list),
				label);
			gtk_widget_show(label);
		}
		int sel = get_int_profile("log_mode")-1;
		if (sel < 0) {
			sel = 0;
		} else if (sel > 1) {
			sel = 1;
		}
		gtk_list_select_item(
			GTK_LIST(GTK_COMBO(log_mode_combo)->list),
			sel);
#else // GTK_CHECK_VERSION(2, 4, 0)
		for (i = 0; i < 2; i++) {
			chaos_resource::resource_item_type item =
				chaos_resource::item_none;
			switch (i) {
			case 0:
				item = chaos_resource::logmode_file;
				break;
			case 1:
				item = chaos_resource::logmode_window;
				break;
			}
			gtk_combo_box_append_text(
				GTK_COMBO_BOX(log_mode_combo),
				chaos_utility::sjis_to_gui(
					res->get_resource_text(item)).c_str());
		}
		int sel = get_int_profile("log_mode")-1;
		if (sel < 0) {
			sel = 0;
		} else if (sel > 1) {
			sel = 1;
		}
		gtk_combo_box_set_active(
			GTK_COMBO_BOX(log_mode_combo),
			sel);
#endif // GTK_CHECK_VERSION
	}
	GtkWidget* log_level_combo = get_widget("log_level");
	if (log_level_combo != NULL) {
#if !GTK_CHECK_VERSION(2, 4, 0)
		for (i = 0; i < 4; i++) {
			chaos_resource::resource_item_type item =
				chaos_resource::item_none;
			switch (i) {
			case 0:
				item = chaos_resource::loglv_none;
				break;
			case 1:
				item = chaos_resource::loglv_error;
				break;
			case 2:
				item = chaos_resource::loglv_warning;
				break;
			case 3:
				item = chaos_resource::loglv_report;
				break;
			}
			GtkWidget* label = gtk_list_item_new_with_label(
				chaos_utility::sjis_to_gui(
					res->get_resource_text(item)).c_str());
			gtk_container_add(
				GTK_CONTAINER(GTK_COMBO(log_level_combo)->list),
				label);
			gtk_widget_show(label);
		}
		int sel = get_int_profile("log_level");
		if (sel < 0) {
			sel = 0;
		} else if (sel > 3) {
			sel = 3;
		}
		gtk_list_select_item(
			GTK_LIST(GTK_COMBO(log_level_combo)->list),
			sel);
#else // GTK_CHECK_VERSION(2, 4, 0)
		for (i = 0; i < 4; i++) {
			chaos_resource::resource_item_type item =
				chaos_resource::item_none;
			switch (i) {
			case 0:
				item = chaos_resource::loglv_none;
				break;
			case 1:
				item = chaos_resource::loglv_error;
				break;
			case 2:
				item = chaos_resource::loglv_warning;
				break;
			case 3:
				item = chaos_resource::loglv_report;
				break;
			}
			gtk_combo_box_append_text(
				GTK_COMBO_BOX(log_level_combo),
				chaos_utility::sjis_to_gui(
					res->get_resource_text(item)).c_str());
		}
		int sel = get_int_profile("log_level");
		if (sel < 0) {
			sel = 0;
		} else if (sel > 3) {
			sel = 3;
		}
		gtk_combo_box_set_active(
			GTK_COMBO_BOX(log_level_combo),
			sel);
#endif // GTK_CHECK_VERSION
	}
	GtkWidget* image_buffer_spin = get_widget("image_buffer");
	if (image_buffer_spin != NULL) {
		GtkAdjustment* adjustment = gtk_spin_button_get_adjustment(
			GTK_SPIN_BUTTON(image_buffer_spin));
		adjustment->lower = 2;
		adjustment->upper = 20;
		gtk_spin_button_set_value(
			GTK_SPIN_BUTTON(image_buffer_spin),
			(gfloat)get_int_profile("sys_image_buffer_count"));
	}
	GtkWidget* effect_step_spin = get_widget("effect_step");
	if (effect_step_spin != NULL) {
		GtkAdjustment* adjustment = gtk_spin_button_get_adjustment(
			GTK_SPIN_BUTTON(effect_step_spin));
		adjustment->lower = 1;
		adjustment->upper = 10;
		int effect_step = get_int_profile("effect_step");
		if (effect_step < 1) {
			effect_step = 1;
		} else if (effect_step > 10) {
			effect_step = 10;
		}
		gtk_spin_button_set_value(
			GTK_SPIN_BUTTON(effect_step_spin),
			(gfloat)effect_step);
	}
	GtkWidget* effect_cut_check = get_widget("effect_cut");
	if (effect_cut_check != NULL) {
		gtk_toggle_button_set_active(
			GTK_TOGGLE_BUTTON(effect_cut_check),
			get_bool_profile("effect_cut")?
				TRUE: FALSE);
	}
}

void chaos_config_dlg::on_ok_clicked() {
	chaos_resource* res = chaos_resource::instance();
	// general
	GtkWidget* full_screen_check = get_widget("full_screen");
	if (full_screen_check != NULL) {
		dirty |= sync_bool_profile(
			"full_screen",
			gtk_toggle_button_get_active(
				GTK_TOGGLE_BUTTON(full_screen_check)) != FALSE);
	}
	GtkWidget* text_speed_combo = get_widget("text_speed");
	if (text_speed_combo != NULL) {
#if !GTK_CHECK_VERSION(2, 4, 0)
		GtkList* list = GTK_LIST(GTK_COMBO(text_speed_combo)->list);
		dirty |= sync_int_profile(
			"text_speed",
			(list->selection != NULL)?
				(gtk_list_child_position(
					list,
					GTK_WIDGET(list->selection->data))+1):
				1);
#else // GTK_CHECK_VERSION(2, 4, 0)
		dirty |= sync_int_profile(
			"text_speed",
			gtk_combo_box_get_active(
				GTK_COMBO_BOX(text_speed_combo))+1);
#endif // GTK_CHECK_VERSION
	}
	GtkWidget* skip_speed_combo = get_widget("skip_speed");
	if (skip_speed_combo != NULL) {
#if !GTK_CHECK_VERSION(2, 4, 0)
		GtkList* list = GTK_LIST(GTK_COMBO(skip_speed_combo)->list);
		dirty |= sync_int_profile(
			"skip_speed",
			(list->selection != NULL)?
				(gtk_list_child_position(
					list,
					GTK_WIDGET(list->selection->data))+1):
				1);
#else // GTK_CHECK_VERSION(2, 4, 0)
		dirty |= sync_int_profile(
			"skip_speed",
			gtk_combo_box_get_active(
				GTK_COMBO_BOX(skip_speed_combo))+1);
#endif // GTK_CHECK_VERSION
	}
	GtkWidget* lastname_entry = get_widget("lastname");
	if (lastname_entry != NULL) {
		dirty |= sync_str_profile(
			"@S",
			chaos_utility::gui_to_sjis(
				gtk_entry_get_text(GTK_ENTRY(lastname_entry))));
	}
	GtkWidget* firstname_entry = get_widget("firstname");
	if (firstname_entry != NULL) {
		dirty |= sync_str_profile(
			"@M",
			chaos_utility::gui_to_sjis(
				gtk_entry_get_text(GTK_ENTRY(firstname_entry))));
	}
	GtkWidget* nickname_entry = get_widget("nickname");
	if (nickname_entry != NULL) {
		dirty |= sync_str_profile(
			"@N",
			chaos_utility::gui_to_sjis(
				gtk_entry_get_text(GTK_ENTRY(nickname_entry))));
	}
	GtkWidget* postfix_entry = get_widget("postfix");
	if (postfix_entry != NULL) {
		dirty |= sync_str_profile(
			"@K",
			chaos_utility::gui_to_sjis(
				gtk_entry_get_text(GTK_ENTRY(postfix_entry))));
	}
	// sound
	GtkWidget* bgm_sound_type_combo =
		get_widget("bgm_sound_type");
	if (bgm_sound_type_combo != NULL) {
#if !GTK_CHECK_VERSION(2, 4, 0)
		chaos_string bgm_sound_type =
			chaos_utility::gui_to_sjis(
				gtk_entry_get_text(
					GTK_ENTRY(GTK_COMBO(bgm_sound_type_combo)->entry)));
		dirty |= sync_str_profile(
			"bgm_sound_type",
			bgm_sound_type);
#else // GTK_CHECK_VERSION(2, 4, 0)
		chaos_string bgm_sound_type =
			chaos_utility::gui_to_sjis(
				gtk_combo_box_get_active_text(
					GTK_COMBO_BOX(bgm_sound_type_combo)));
		dirty |= sync_str_profile(
			"bgm_sound_type",
			bgm_sound_type);
#endif // GTK_CHECK_VERSION
	}
	GtkWidget* cd_drive_combo = get_widget("cd_drive");
	if (cd_drive_combo != NULL) {
#if !GTK_CHECK_VERSION(2, 4, 0)
		chaos_string cd_drive =
			chaos_utility::gui_to_sjis(
				gtk_entry_get_text(
					GTK_ENTRY(GTK_COMBO(cd_drive_combo)->entry)));
		chaos_string cd_drive_auto =
			res->get_resource_text(chaos_resource::cd_drive_auto);
		if (cd_drive == cd_drive_auto) {
			cd_drive = "";
		}
		dirty |= sync_str_profile(
			"cd_drive",
			cd_drive);
#else // GTK_CHECK_VERSION(2, 4, 0)
		chaos_string cd_drive =
			chaos_utility::gui_to_sjis(
				gtk_combo_box_get_active_text(
					GTK_COMBO_BOX(cd_drive_combo)));
		chaos_string cd_drive_auto =
			res->get_resource_text(chaos_resource::cd_drive_auto);
		if (cd_drive == cd_drive_auto) {
			cd_drive = "";
		}
		dirty |= sync_str_profile(
			"cd_drive",
			cd_drive);
#endif // GTK_CHECK_VERSION
	}
	for (int types = 0; types < 2; types++) {
		chaos_string sound_volume_name = (types == 0)?
			"bgm_sound_volume":
			"effect_sound_volume";
		GtkWidget* sound_volume_scale =
			get_widget(sound_volume_name);
		if (sound_volume_scale != NULL) {
			int sound_volume = (int)(
					gtk_range_get_value(
						GTK_RANGE(sound_volume_scale))+0.5);
			if (sound_volume < 0) {
				sound_volume = 0;
			} else if (sound_volume > 100) {
				sound_volume = 100;
			}
			dirty |= sync_int_profile(
				sound_volume_name,
				sound_volume);
		}
	}
	// font

#ifdef CHAOS_FONT_GTK

	chaos_string font_kind_name = "font_xlfd",
		sys_font_kind_name = "sys_font_xlfd";

#elif defined(CHAOS_FONT_PANGO)

	chaos_string font_kind_name = "font_pango",
		sys_font_kind_name = "sys_font_pango";

#endif // CHAOS_FONT

	GtkWidget* font_entry = get_widget(font_kind_name);
	if (font_entry != NULL) {
		chaos_string font = chaos_utility::gui_to_sjis(
				gtk_entry_get_text(GTK_ENTRY(font_entry)));
		dirty |= sync_str_profile(
			font_kind_name,
			font);
		dirty |= sync_str_profile(
			sys_font_kind_name,
			font);
	}
	GtkWidget* font_anti_alias_check = get_widget("font_anti_alias");
	if (font_anti_alias_check != NULL) {
		bool font_anti_alias = (gtk_toggle_button_get_active(
			GTK_TOGGLE_BUTTON(font_anti_alias_check)) != FALSE);
		dirty |= sync_bool_profile(
			"font_anti_alias",
			font_anti_alias);
		dirty |= sync_bool_profile(
			"sys_font_anti_alias",
			font_anti_alias);
	}
	// else
	GtkWidget* not_show_startup_check = get_widget("not_show_startup");
	if (not_show_startup_check != NULL) {
		bool show_startup = (gtk_toggle_button_get_active(
			GTK_TOGGLE_BUTTON(not_show_startup_check)) == FALSE);
		dirty |= sync_bool_profile(
			"show_startup",
			show_startup);
	}
	GtkWidget* log_mode_combo = get_widget("log_mode");
	if (log_mode_combo != NULL) {
#if !GTK_CHECK_VERSION(2, 4, 0)
		GtkList* list = GTK_LIST(GTK_COMBO(log_mode_combo)->list);
		dirty |= sync_int_profile(
			"log_mode",
			(list->selection != NULL)?
				(gtk_list_child_position(
					list,
					GTK_WIDGET(list->selection->data))+1):
				1);
#else // GTK_CHECK_VERSION(2, 4, 0)
		dirty |= sync_int_profile(
			"log_mode",
			gtk_combo_box_get_active(
				GTK_COMBO_BOX(log_mode_combo))+1);
#endif // GTK_CHECK_VERSION
	}
	GtkWidget* log_level_combo = get_widget("log_level");
	if (log_level_combo != NULL) {
#if !GTK_CHECK_VERSION(2, 4, 0)
		GtkList* list = GTK_LIST(GTK_COMBO(log_level_combo)->list);
		dirty |= sync_int_profile(
			"log_level",
			(list->selection != NULL)?
				gtk_list_child_position(
					list,
					GTK_WIDGET(list->selection->data)):
				0);
#else // GTK_CHECK_VERSION(2, 4, 0)
		dirty |= sync_int_profile(
			"log_level",
			gtk_combo_box_get_active(
				GTK_COMBO_BOX(log_level_combo)));
#endif // GTK_CHECK_VERSION
	}
	GtkWidget* image_buffer_spin = get_widget("image_buffer");
	if (image_buffer_spin != NULL) {
		int buf_count = gtk_spin_button_get_value_as_int(
			GTK_SPIN_BUTTON(image_buffer_spin));
		if (buf_count < 2) {
			buf_count = 2;
		} else if (buf_count > 20) {
			buf_count = 20;
		}
		dirty |= sync_int_profile(
			"sys_image_buffer_count",
			buf_count);
	}
	GtkWidget* effect_step_spin = get_widget("effect_step");
	if (effect_step_spin != NULL) {
		int effect_step = gtk_spin_button_get_value_as_int(
			GTK_SPIN_BUTTON(effect_step_spin));
		if (effect_step < 1) {
			effect_step = 1;
		} else if (effect_step > 10) {
			effect_step = 10;
		}
		dirty |= sync_int_profile(
			"effect_step",
			effect_step);
	}
	GtkWidget* effect_cut_check = get_widget("effect_cut");
	if (effect_cut_check != NULL) {
		bool effect_cut = (gtk_toggle_button_get_active(
			GTK_TOGGLE_BUTTON(effect_cut_check)) != FALSE);
		dirty |= sync_bool_profile(
			"effect_cut",
			effect_cut);
	}
}

#endif // CHAOS_GUI
