////////////////////////////////////////////////////////////////////////////////
// chaos_config_dlg.h : config dialog

#ifndef CHAOS_CONFIG_DLG_H_INCLUDED
#define CHAOS_CONFIG_DLG_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// declare

class chaos_config_dlg;

////////////////////////////////////////////////////////////////////////////////
// include

#ifndef CHAOS_COLOR_H_INCLUDED
#include "chaos_color.h"
#endif // CHAOS_COLOR_H_INCLUDED

#ifndef CHAOS_STRING_H_INCLUDED
#include "chaos_string.h"
#endif // CHAOS_STRING_H_INCLUDED

#ifndef CHAOS_XML_H_INCLUDED
#include "chaos_xml.h"
#endif // CHAOS_XML_H_INCLUDED

#ifndef CHAOS_IMAGE_H_INCLUDED
#include "chaos_image.h"
#endif // CHAOS_IMAGE_H_INCLUDED

#ifndef CHAOS_DLG_H_INCLUDED
#include "chaos_dlg.h"
#endif // CHAOS_DLG_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// chaos_config_dlg : config dialog class

class chaos_config_dlg :
	public chaos_dlg
{
// attribute
private:
	// dirty
	bool dirty;
	// profile
	chaos_xml* profile;
	// sample image
	chaos_image* sample_image;

#ifdef CHAOS_GUI_GTK

	// sample pixmap
	GdkPixmap* sample_pixmap;
	// font dirty
	bool font_dirty;
	// selected font
	chaos_string selected_font;

#endif // CHAOS_GUI

// create & destroy
private:
	// default constructor
	chaos_config_dlg();

public:
	// destructor
	~chaos_config_dlg();

	// get singleton instance
	static chaos_config_dlg* instance();

// get & set
public:
	// is dirty
	bool is_dirty() const {
		return dirty;
	}
	// set dirty
	void set_dirty(bool new_dirty) {
		dirty = new_dirty;
	}
	// get profile
	const chaos_xml* get_profile() const {
		return profile;
	}
	chaos_xml* get_profile() {
		return profile;
	}
	// set profile
	void set_profile(chaos_xml* new_profile) {
		profile = new_profile;
	}
	// get int profile
	int get_int_profile(const chaos_string& name) const {
		ASSERT(profile != NULL);
		return profile->get_values()->get_value(name).get_integer();
	}
	// sync int profile
	bool sync_int_profile(const chaos_string& name, int value) {
		ASSERT(profile != NULL);
		bool dirty = false;
		if ((chaos_int32)value !=
				profile->get_values()->get_value(name).get_integer())
		{
			profile->get_values()->set_value(
				name, chaos_value((chaos_int32)value));
			dirty = true;
		}
		return dirty;
	}
	// get boolean profile
	bool get_bool_profile(const chaos_string& name) const {
		ASSERT(profile != NULL);
		return profile->get_values()->get_value(name).get_boolean();
	}
	// sync boolean profile
	bool sync_bool_profile(const chaos_string& name, bool value) {
		ASSERT(profile != NULL);
		bool dirty = false;
		if (value !=
				profile->get_values()->get_value(name).get_boolean())
		{
			profile->get_values()->set_value(
				name, chaos_value(value));
			dirty = true;
		}
		return dirty;
	}
	// get color profile
	chaos_color get_col_profile(const chaos_string& name) const {
		ASSERT(profile != NULL);
		return profile->get_values()->get_value(name).get_color();
	}
	// sync color profile
	bool sync_col_profile(const chaos_string& name, chaos_color value) {
		ASSERT(profile != NULL);
		bool dirty = false;
		if (value !=
				profile->get_values()->get_value(name).get_color())
		{
			profile->get_values()->set_value(
				name, chaos_value(value));
			dirty = true;
		}
		return dirty;
	}
	// get string profile
	chaos_string get_str_profile(const chaos_string& name) const {
		ASSERT(profile != NULL);
		return profile->get_values()->get_value(name).get_string();
	}
	// sync string profile
	bool sync_str_profile(
		const chaos_string& name, const chaos_string& value) const
	{
		ASSERT(profile != NULL);
		bool dirty = false;
		if (value !=
				profile->get_values()->get_value(name).get_string())
		{
			profile->get_values()->set_value(
				name, chaos_value(value));
			dirty = true;
		}
		return dirty;
	}
	// get sample image
	const chaos_image* get_sample_image() const {
		return sample_image;
	}
	chaos_image* get_sample_image() {
		return sample_image;
	}

#ifdef CHAOS_GUI_GTK

	// get sample pixmap
	const GdkPixmap* get_sample_pixmap() const {
		return sample_pixmap;
	}
	GdkPixmap* get_sample_pixmap() {
		return sample_pixmap;
	}
	// set sample pixmap
	void set_sample_pixmap(GdkPixmap* new_sample_pixmap) {
		sample_pixmap = new_sample_pixmap;
	}
	// is font dirty
	bool is_font_dirty() const {
		return font_dirty;
	}
	// set font dirty
	void set_font_dirty(bool new_dirty) {
		font_dirty = new_dirty;
	}
	// get selected font
	const chaos_string& get_selected_font() const {
		return selected_font;
	}
	// set selected font
	void set_selected_font(const chaos_string& new_font) {
		selected_font = new_font;
	}

#endif // CHAOS_GUI

// dialog
private:

#ifdef CHAOS_GUI_WIN32

	// general page procedure
	static BOOL CALLBACK general_dlgproc(
		HWND dlg, UINT message, WPARAM wparam, LPARAM lparam);
	// sound page procedure
	static BOOL CALLBACK sound_dlgproc(
		HWND dlg, UINT message, WPARAM wparam, LPARAM lparam);
	// font page procedure
	static BOOL CALLBACK font_dlgproc(
		HWND dlg, UINT message, WPARAM wparam, LPARAM lparam);
	// else page procedure
	static BOOL CALLBACK else_dlgproc(
		HWND dlg, UINT message, WPARAM wparam, LPARAM lparam);

#elif defined(CHAOS_GUI_GTK)

	// font entry focus in event
	static gboolean font_focus_in_event(
		GtkWidget* font_entry,
		GdkEventFocus* event,
		gpointer data);
	// font entry focus out event
	static gboolean font_focus_out_event(
		GtkWidget* font_entry,
		GdkEventFocus* event,
		gpointer data);
	// font entry changed
	static void font_changed(
		GtkWidget* font_entry,
		gpointer data);
	// default name button clicked
	static void default_name_clicked(
		GtkWidget* default_name_button,
		gpointer data);
	// sound bgm sound type changed
	static void sound_bgm_sound_type_changed(
		GtkWidget* sound_bgm_sound_type_entry,
		gpointer data);
	// font select button clicked
	static void font_select_clicked(
		GtkWidget* font_select_button,
		gpointer data);
	// font anti alias check button toggled
	static void font_anti_alias_toggled(
		GtkWidget* font_anti_alias_check,
		gpointer data);
	// default font button clicked
	static void default_font_clicked(
		GtkWidget* default_font_button,
		gpointer data);
	// font sample expose event
	static gboolean font_sample_expose_event(
		GtkWidget* font_sample_area,
		GdkEventExpose* event,
		gpointer data);
	// font selection dialog ok button clicked
	static void fontsel_dialog_ok_clicked(
		GtkWidget* ok_button,
		gpointer data);

#endif // CHAOS_GUI

// font
private:

#ifdef CHAOS_GUI_WIN32

	// enum font family procedure
	static int CALLBACK enum_font_family_proc(
		ENUMLOGFONTEX *pelfe,
		NEWTEXTMETRICEX* pntme,
		int font_type,
		LPARAM lparam);

#endif // CHAOS_GUI

	// build sample image
	bool build_sample_image(
		chaos_dim sample_dim,
		const chaos_string& font_name,
		bool font_bold,
		bool font_anti_alias);

// operation
public:
	// modal
	bool do_modal(chaos_wnd_handle parent_wnd);

#ifdef CHAOS_GUI_GTK

	// initialize widgets
	void initialize_widgets();
	// on ok clicked
	virtual void on_ok_clicked();

#endif // CHAOS_GUI

};

#endif // CHAOS_CONFIG_DLG_H_INCLUDED
