////////////////////////////////////////////////////////////////////////////////
// chaos_dim.h : 2d dimension

#ifndef CHAOS_DIM_H_INCLUDED
#define CHAOS_DIM_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// declare

class chaos_dim;

////////////////////////////////////////////////////////////////////////////////
// include

#ifndef CHAOS_POS_H_INCLUDED
#include "chaos_pos.h"
#endif // CHAOS_POS_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// chaos_dim : 2d dimension class

class chaos_dim {
// attribute
public:
	// dimension width
	int width;
	// dimension height
	int height;

// create & destroy
public:
	// default constructor
	chaos_dim() :
		width(0), height(0)
	{
	}
	// normal constructor
	chaos_dim(int new_width, int new_height) :
		width(new_width), height(new_height)
	{
	}
	// copy constructor
	chaos_dim(const chaos_dim& org) :
		width(org.width), height(org.height)
	{
	}
	// 2d position constructor
	chaos_dim(const chaos_pos& org);

// operation
public:
	// is empty
	bool empty() const {
		return (width == 0) || (height == 0);
	}
	// clear
	void clear() {
		width = height = 0;
	}

// operator
public:
	// copy
	chaos_dim& operator=(const chaos_dim& org) {
		width = org.width;
		height = org.height;
		return *this;
	}
	// add let
	chaos_dim& operator+=(const chaos_dim& org) {
		width += org.width;
		height += org.height;
		return *this;
	}
	// sub let
	chaos_dim& operator-=(const chaos_dim& org) {
		width -= org.width;
		height -= org.height;
		return *this;
	}
	// add
	chaos_dim operator+(const chaos_dim& org) const {
		chaos_dim tmp = *this;
		tmp += org;
		return tmp;
	}
	// sub
	chaos_dim operator-(const chaos_dim& org) const {
		chaos_dim tmp = *this;
		tmp -= org;
		return tmp;
	}
	// equal
	bool operator==(const chaos_dim& org) const {
		return (width == org.width) && (height == org.height);
	}
	// not equal
	bool operator!=(const chaos_dim& org) const {
		return (width != org.width) || (height != org.height);
	}
};

#endif // CHAOS_DIM_H_INCLUDED
