////////////////////////////////////////////////////////////////////////////////
// chaos_dlg.cpp : dialog

////////////////////////////////////////////////////////////////////////////////
// include

#include "stdafx.h"

#include "chaos_dlg.h"
#include "chaos_value.h"
#include "chaos_value_manager.h"
#include "chaos_utility.h"

////////////////////////////////////////////////////////////////////////////////
// chaos_dlg : dialog base class

////////////////////////////////////////////////////////////////////////////////
// attribute

#ifdef CHAOS_GUI_GTK

// current modaling

std::list<GtkWidget*> chaos_dlg::current_modaling;

#endif // CHAOS_GUI

////////////////////////////////////////////////////////////////////////////////
// utility

#if defined(CHAOS_FONT_GTK) && (GTK_MAJOR_VERSION < 2)

static GdkFont* gtk_style_get_font(GtkStyle* style) {
	return style->font;
}

#endif // defined(CHAOS_FONT_GTK) && (GTK_MAJOR_VERSION < 2)

////////////////////////////////////////////////////////////////////////////////
// create & destroy

// default constructor

chaos_dlg::chaos_dlg() {

#ifdef CHAOS_GUI_GTK

	dialog_result = false;
	dialog = NULL;

#endif // CHAOS_GUI

}

// destructor

chaos_dlg::~chaos_dlg() {
}

////////////////////////////////////////////////////////////////////////////////
// dialog

#ifdef CHAOS_GUI_GTK

// delete event

gboolean chaos_dlg::delete_event(
	GtkWidget* /*dialog*/,
	GdkEvent* /*event*/,
	gpointer data)
{
	chaos_dlg* dlg = (chaos_dlg*)data;
	dlg->set_dialog_result(false);
	dlg->on_cancel_clicked();
	return FALSE;
}

// destroy

void chaos_dlg::destroy(
	GtkWidget* /*dialog*/,
	gpointer /*data*/)
{
	gtk_main_quit();
}

// key press event

gboolean chaos_dlg::key_press_event(
	GtkWidget* dialog,
	GdkEventKey* event,
	gpointer* data)
{
	if (event->keyval == GDK_Escape) {
		chaos_dlg* dlg = (chaos_dlg*)data;
		dlg->set_dialog_result(false);
		dlg->on_cancel_clicked();
		gtk_widget_destroy(dialog);
		return TRUE;
	}
	return FALSE;
}

// ok button clicked

void chaos_dlg::ok_clicked(
	GtkWidget* /*ok_button*/,
	gpointer data)
{
	chaos_dlg* dlg = (chaos_dlg*)data;
	dlg->set_dialog_result(true);
	dlg->on_ok_clicked();
	gtk_widget_destroy(dlg->get_dialog());
}

// cancel button clicked

void chaos_dlg::cancel_clicked(
	GtkWidget* /*cancel_button*/,
	gpointer data)
{
	chaos_dlg* dlg = (chaos_dlg*)data;
	dlg->set_dialog_result(false);
	dlg->on_cancel_clicked();
	gtk_widget_destroy(dlg->get_dialog());
}

#endif // CHAOS_GUI

////////////////////////////////////////////////////////////////////////////////
// operation

#ifdef CHAOS_GUI_GTK

// push current modaling

void chaos_dlg::push_current_modaling(GtkWidget* modaling_dialog) {
	current_modaling.push_back(modaling_dialog);
}

// pop current modaling

void chaos_dlg::pop_current_modaling() {
	current_modaling.pop_back();
}

// skip space in info

void chaos_dlg::skip_info_space(const char*& infostr) {
	while (chaos_utility::is_space(*infostr)) {
		infostr++;
	}
}

// get word from info

chaos_string chaos_dlg::get_info_word(const char*& infostr) {
	chaos_string words;
	char ch;
	while (
		((ch = *infostr) != '\0') &&
		!chaos_utility::is_space(ch) &&
		((ch == '_') ||
			chaos_utility::is_alpha(ch) ||
			chaos_utility::is_digit(ch)))
	{
		words += ch;
		infostr++;
	}
	return words;
}

// build widgets

GtkWidget* chaos_dlg::build_widgets(
	GtkWidget* parent,
	const chaos_string& widget_info)
{
	#define ATTR_INT(nm, df) \
		(attrs.exist_value(#nm)? \
			(int)attrs.get_value(#nm).get_integer(): \
			(int)(df))
	#define ATTR_BOOL(nm, df) \
		(attrs.exist_value(#nm)? \
			attrs.get_value(#nm).get_boolean(): \
			(df))
	#define ATTR_STR(nm, df) \
		(attrs.exist_value(#nm)? \
			attrs.get_value(#nm).get_string(): \
			chaos_string(df))
	std::stack< GtkWidget*, std::list<GtkWidget*> > wstack;
	if (parent != NULL) {
		wstack.push(parent);
	}
	GtkWidget* result_widget = NULL;
	GSList* radio_group = NULL;
	GtkAdjustment* adjustment = NULL;
	int dialog_unit = 16;
	GtkStyle* style = gtk_widget_get_default_style();
	GdkFont* style_font = NULL;
	if (style != NULL) {
		style_font = gtk_style_get_font(style);
	}
	if (style_font != NULL) {
		dialog_unit = style_font->ascent+style_font->descent;
	}
	const char* infostr = widget_info.c_str();
	while (true) {
		skip_info_space(infostr);
		while (*infostr == '}') {
			infostr++;
			if (wstack.size() <= 0) {
				ASSERT(false);
				return NULL;
			}
			wstack.pop();
			skip_info_space(infostr);
		}
		chaos_string types =
			chaos_utility::to_lower_str(get_info_word(infostr));
		if (types.empty()) {
			break;
		}
		chaos_value_manager<chaos_string> attrs;
		skip_info_space(infostr);
		if (*infostr == '(') {
			infostr++;
			while (true) {
				skip_info_space(infostr);
				if (*infostr == ')') {
					infostr++;
					break;
				}
				chaos_string attr_name =
					chaos_utility::to_lower_str(get_info_word(infostr));
				if (attr_name.empty()) {
					ASSERT(false);
					return NULL;
				}
				chaos_value attr_value;
				skip_info_space(infostr);
				if (*infostr == '=') {
					infostr++;
					skip_info_space(infostr);
					attr_value = chaos_value::from_string(infostr);
				}
				attrs.set_value(attr_name, attr_value);
				skip_info_space(infostr);
				if (*infostr == ',') {
					infostr++;
				}
			}
		}
		GtkWidget* new_widget = NULL;
		if (types == "vbox") {
			new_widget = gtk_vbox_new(
				ATTR_INT(homogeneous, FALSE),
				(ATTR_INT(spacing, 1)*dialog_unit)/8);
		} else if (types == "hbox") {
			new_widget = gtk_hbox_new(
				ATTR_INT(homogeneous, FALSE),
				(ATTR_INT(spacing, 1)*dialog_unit)/8);
		} else if (types == "frame") {
			new_widget = gtk_frame_new(
				chaos_utility::sjis_to_gui(
					ATTR_STR(text, "")).c_str());
		} else if (types == "scrolled_window") {
			new_widget = gtk_scrolled_window_new(
				NULL, NULL);
			if (new_widget != NULL) {
				gtk_scrolled_window_set_policy(
					GTK_SCROLLED_WINDOW(new_widget),
					(ATTR_INT(hscrollbar_policy, 1) != 0)?
						GTK_POLICY_ALWAYS: GTK_POLICY_AUTOMATIC,
					(ATTR_INT(vscrollbar_policy, 1) != 0)?
						GTK_POLICY_ALWAYS: GTK_POLICY_AUTOMATIC);
			}
		} else if (types == "notebook") {
			new_widget = gtk_notebook_new();
		} else if (types == "label") {
			new_widget = gtk_label_new(
				chaos_utility::sjis_to_gui(
					ATTR_STR(text, "")).c_str());
		} else if (types == "button") {
			new_widget = gtk_button_new_with_label(
				chaos_utility::sjis_to_gui(
					ATTR_STR(text, "")).c_str());
		} else if (types == "check_button") {
			new_widget = gtk_check_button_new_with_label(
				chaos_utility::sjis_to_gui(
					ATTR_STR(text, "")).c_str());
		} else if (types == "radio_button") {
			bool lead = ATTR_BOOL(lead, false);
			if (lead) {
				radio_group = NULL;
			}
			new_widget = gtk_radio_button_new_with_label(
				radio_group,
				chaos_utility::sjis_to_gui(
					ATTR_STR(text, "")).c_str());
			if (new_widget != NULL) {
				if (lead) {
					radio_group = gtk_radio_button_group(
						GTK_RADIO_BUTTON(new_widget));
				}
			}
		} else if (types == "entry") {
			new_widget = gtk_entry_new();
			gtk_entry_set_editable(
				GTK_ENTRY(new_widget),
				ATTR_BOOL(editable, true)? TRUE: FALSE);
		} else if (types == "text") {
#if !GTK_CHECK_VERSION(2, 0, 0)
			new_widget = gtk_text_new(NULL, NULL);
			if (new_widget != NULL) {
				if (ATTR_BOOL(vadj, false)) {
					adjustment = GTK_TEXT(new_widget)->vadj;
				}
				gtk_text_set_editable(
					GTK_TEXT(new_widget),
					ATTR_BOOL(editable, true)? TRUE: FALSE);
			}
#else // GTK_CHECK_VERSION(2, 0, 0)
			new_widget = gtk_text_view_new();
			if (new_widget != NULL) {
				gtk_text_view_set_wrap_mode(
					GTK_TEXT_VIEW(new_widget),
					GTK_WRAP_CHAR);
				if (ATTR_BOOL(vadj, false)) {
					adjustment = (GtkAdjustment*)gtk_adjustment_new(
						0,
						0, 100,
						1, 10,
						1);
					gtk_widget_set_scroll_adjustments(
						GTK_WIDGET(new_widget),
						NULL,
						adjustment);
				}
				gtk_text_view_set_editable(
					GTK_TEXT_VIEW(new_widget),
					ATTR_BOOL(editable, true)? TRUE: FALSE);
			}
#endif // GTK_CHECK_VERSION
		} else if (types == "combo") {
#if !GTK_CHECK_VERSION(2, 4, 0)
			new_widget = gtk_combo_new();
			if (new_widget != NULL) {
				gtk_combo_set_use_arrows(
					GTK_COMBO(new_widget),
					ATTR_BOOL(use_arrows, true)? TRUE: FALSE);
				gtk_entry_set_editable(
					GTK_ENTRY(GTK_COMBO(new_widget)->entry),
					ATTR_BOOL(editable, true)? TRUE: FALSE);
			}
#else // GTK_CHECK_VERSION(2, 4, 0)
			if (!ATTR_BOOL(editable, true)) {
				new_widget = gtk_combo_box_new_text();
			} else {
				new_widget = gtk_combo_box_entry_new_text();
			}
#endif // GTK_CHECK_VERSION
		} else if (types == "hscrollbar") {
			GtkAdjustment* adjustment2 = NULL;
			if (ATTR_BOOL(adjustment, false)) {
				adjustment2 = adjustment;
			}
			new_widget = gtk_hscrollbar_new(adjustment2);
		} else if (types == "vscrollbar") {
			GtkAdjustment* adjustment2 = NULL;
			if (ATTR_BOOL(adjustment, false)) {
				adjustment2 = adjustment;
			}
			new_widget = gtk_vscrollbar_new(adjustment2);
		} else if (types == "hscale") {
			GtkAdjustment* adjustment2 = NULL;
			if (ATTR_BOOL(adjustment, false)) {
				adjustment2 = adjustment;
			}
			new_widget = gtk_hscale_new(adjustment2);
		} else if (types == "vscale") {
			GtkAdjustment* adjustment2 = NULL;
			if (ATTR_BOOL(adjustment, false)) {
				adjustment2 = adjustment;
			}
			new_widget = gtk_vscale_new(adjustment2);
		} else if (types == "spin_button") {
			GtkAdjustment* adjustment2 =
				(GtkAdjustment*)gtk_adjustment_new(
					(gfloat)ATTR_INT(value, 0),
					(gfloat)ATTR_INT(lower, 0),
					(gfloat)ATTR_INT(upper, 100),
					(gfloat)ATTR_INT(stepincrement, 1),
					(gfloat)ATTR_INT(pageincrement, 10),
					(gfloat)ATTR_INT(pagesize, 0));
			new_widget = gtk_spin_button_new(
				adjustment2,
				(gfloat)ATTR_INT(climb_rate, 0),
				ATTR_INT(digits, 0));
		} else if (types == "drawing_area") {
			new_widget = gtk_drawing_area_new();
			if (new_widget != NULL) {
				int dx = ATTR_INT(dx, 0),
					dy = ATTR_INT(dy, 0);
				if ((dx > 0) && (dy > 0)) {
					gtk_drawing_area_size(
						GTK_DRAWING_AREA(new_widget),
						dx, dy);
				}
				if (ATTR_BOOL(exposure, true)) {
					gtk_widget_set_events(
						new_widget,
						GDK_EXPOSURE_MASK);
				}
			}
		}
		if (new_widget == NULL) {
			ASSERT(false);
			break;
		}
		if (result_widget == NULL) {
			result_widget = new_widget;
		}
		chaos_string names = ATTR_STR(name, "");
		if (!names.empty()) {
			register_widget(names, new_widget);
		}
		int width = ATTR_INT(width, 0),
			height = ATTR_INT(height, 0);
		if (width > 0) {
			width = (width*dialog_unit)/8;
		} else if (width < 0) {
			width = -width;
		}
		if (height > 0) {
			height = (height*dialog_unit)/8;
		} else if (height < 0) {
			height = -height;
		}
		if ((width > 0) || (height > 0)) {
			gtk_widget_set_usize(
				new_widget,
				width, height);
		}
		int border = ATTR_INT(border, 0);
		if ((border > 0) &&
			GTK_IS_CONTAINER(new_widget))
		{
			gtk_container_set_border_width(
				GTK_CONTAINER(new_widget),
				(border*dialog_unit)/8);
		}
		if (wstack.size() > 0) {
			GtkWidget* container = wstack.top();
			if (GTK_IS_BOX(container)) {
				if (!ATTR_BOOL(end_pack, false)) {
					gtk_box_pack_start(
						GTK_BOX(container),
						new_widget,
						ATTR_BOOL(expand, false)? TRUE: FALSE,
						ATTR_BOOL(fill, false)? TRUE: FALSE,
						ATTR_INT(padding, 1));
				} else {
					gtk_box_pack_end(
						GTK_BOX(container),
						new_widget,
						ATTR_BOOL(expand, false)? TRUE: FALSE,
						ATTR_BOOL(fill, false)? TRUE: FALSE,
						ATTR_INT(padding, 1));
				}
			} else if (GTK_IS_SCROLLED_WINDOW(container)) {
				gtk_scrolled_window_add_with_viewport(
					GTK_SCROLLED_WINDOW(container),
					new_widget);
			} else if (GTK_IS_NOTEBOOK(container)) {
				GtkWidget* label = gtk_label_new(
					chaos_utility::sjis_to_gui(
						ATTR_STR(tag_name, "")).c_str());
				gtk_notebook_append_page(
					GTK_NOTEBOOK(container),
					new_widget,
					label);
			} else if (GTK_IS_CONTAINER(container)) {
				gtk_container_add(
					GTK_CONTAINER(container),
					new_widget);
			}
		}
		if (ATTR_BOOL(default_button, false)) {
			GTK_WIDGET_SET_FLAGS(
				new_widget,
				GTK_CAN_DEFAULT);
			gtk_widget_grab_default(new_widget);
		}
		gtk_widget_show(new_widget);
		skip_info_space(infostr);
		if (*infostr == '{') {
			infostr++;
			wstack.push(new_widget);
		}
	}
	#undef ATTR_INT
	#undef ATTR_BOOL
	#undef ATTR_STR
	return result_widget;
}

// destroy widgets

void chaos_dlg::destroy_widgets() {
	widget_map::const_iterator it;
	for (it = widgets.begin(); it != widgets.end(); it++) {
		gtk_widget_unref((*it).second);
	}
	widgets.clear();
}

// register widget

void chaos_dlg::register_widget(
	const chaos_string& name, GtkWidget* widget)
{
	gtk_widget_ref(widget);
	widgets.insert(
		std::make_pair(name, widget));
}

// get widget

GtkWidget* chaos_dlg::get_widget(const chaos_string& name) const {
	widget_map::const_iterator it = widgets.find(name);
	return (it != widgets.end())?
		(*it).second:
		(GtkWidget*)NULL;
}

// init dialog default

void chaos_dlg::init_dialog_default(const chaos_string& title) {
	ASSERT(dialog != NULL);
	ASSERT(GTK_IS_DIALOG(dialog));
	gtk_signal_connect(
		GTK_OBJECT(dialog), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);
	gtk_signal_connect(
		GTK_OBJECT(dialog), "destroy",
		GTK_SIGNAL_FUNC(destroy), this);
	GtkWidget* ok_button = get_widget("ok");
	if (ok_button != NULL) {
		gtk_signal_connect(
			GTK_OBJECT(ok_button), "clicked",
			GTK_SIGNAL_FUNC(ok_clicked), this);
	}
	GtkWidget* cancel_button = get_widget("cancel");
	if (cancel_button != NULL) {
		gtk_signal_connect(
			GTK_OBJECT(cancel_button), "clicked",
			GTK_SIGNAL_FUNC(cancel_clicked), this);
	}
	gtk_signal_connect_after(
		GTK_OBJECT(dialog), "key_press_event",
		GTK_SIGNAL_FUNC(key_press_event), this);
	gtk_window_set_title(
		GTK_WINDOW(dialog),
		chaos_utility::sjis_to_gui(
			title).c_str());
	dialog_result = false;
}

// on ok clicked

void chaos_dlg::on_ok_clicked() {
	// nothing
}

// on cancel clicked

void chaos_dlg::on_cancel_clicked() {
	// nothing
}

#endif // CHAOS_GUI
