////////////////////////////////////////////////////////////////////////////////
// chaos_dlg.h : dialog

#ifndef CHAOS_DLG_H_INCLUDED
#define CHAOS_DLG_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// declare

class chaos_dlg;

////////////////////////////////////////////////////////////////////////////////
// include

#ifndef CHAOS_STRING_H_INCLUDED
#include "chaos_string.h"
#endif // CHAOS_STRING_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// chaos_dlg : dialog base class

class chaos_dlg {
// typedef
protected:

#ifdef CHAOS_GUI_GTK

	// widget map
	typedef std::map<chaos_string, GtkWidget*> widget_map;

#endif // CHAOS_GUI

// attribute
protected:

#ifdef CHAOS_GUI_GTK

	// dialog result
	bool dialog_result;
	// dialog widget
	GtkWidget* dialog;
	// widgets
	widget_map widgets;
	// current modaling
	static std::list<GtkWidget*> current_modaling;

#endif // CHAOS_GUI

// create & destroy
public:
	// default constructor
	chaos_dlg();
	// destructor
	~chaos_dlg();

// dialog
private:

#ifdef CHAOS_GUI_GTK

	// delete event
	static gboolean delete_event(
		GtkWidget* dialog,
		GdkEvent* event,
		gpointer data);
	// destroy
	static void destroy(
		GtkWidget* dialog,
		gpointer data);
	// key press event
	static gboolean key_press_event(
		GtkWidget* dialog,
		GdkEventKey* event,
		gpointer* data);
	// ok button clicked
	static void ok_clicked(
		GtkWidget* ok_button,
		gpointer data);
	// cancel button clicked
	static void cancel_clicked(
		GtkWidget* cancel_button,
		gpointer data);

#endif // CHAOS_GUI

// get & set
public:

#ifdef CHAOS_GUI_GTK

	// get dialog result
	bool get_dialog_result() const {
		return dialog_result;
	}
	// set dialog result
	void set_dialog_result(bool new_result) {
		dialog_result = new_result;
	}
	// get dialog widget
	const GtkWidget* get_dialog() const {
		return dialog;
	}
	GtkWidget* get_dialog() {
		return dialog;
	}
	// set dialog widget
	void set_dialog(GtkWidget* new_dialog) {
		dialog = new_dialog;
	}
	// get current modaling
	static GtkWidget* get_current_modaling() {
		return (current_modaling.size() > 0)?
			current_modaling.back():
			NULL;
	}

#endif // CHAOS_GUI

// operation
public:

#ifdef CHAOS_GUI_GTK

	// push current modaling
	static void push_current_modaling(GtkWidget* modaling_dialog);
	// pop current modaling
	static void pop_current_modaling();
	// skip space in info
	static void skip_info_space(const char*& infostr);
	// get word from info
	static chaos_string get_info_word(const char*& infostr);
	// build widgets
	GtkWidget* build_widgets(
		GtkWidget* parent,
		const chaos_string& widget_info);
	// destroy widgets
	void destroy_widgets();
	// register widget
	void register_widget(
		const chaos_string& name, GtkWidget* widget);
	// get widget
	GtkWidget* get_widget(const chaos_string& name) const;
	// init dialog default
	void init_dialog_default(const chaos_string& title);
	// on ok clicked
	virtual void on_ok_clicked();
	// on cancel clicked
	virtual void on_cancel_clicked();

#endif // CHAOS_GUI

};

#endif // CHAOS_DLG_H_INCLUDED
