////////////////////////////////////////////////////////////////////////////////
// chaos_file_data.cpp : file data

////////////////////////////////////////////////////////////////////////////////
// include

#include "stdafx.h"

#include "chaos_file_data.h"

////////////////////////////////////////////////////////////////////////////////
// chaos_file_data : file data class

////////////////////////////////////////////////////////////////////////////////
// create & destroy

// constructor

chaos_file_data::chaos_file_data() :
	cur_pos(0)
{
}

// destructor

chaos_file_data::~chaos_file_data() {
}

////////////////////////////////////////////////////////////////////////////////
// operation

// get 1 byte

int chaos_file_data::get() {
	int bt = -1;
	if (cur_pos < databuf.size()) {
		bt = (int)databuf[cur_pos++];
	}
	return bt;
}

// get line

chaos_string chaos_file_data::get_line() {
	chaos_string line;
	int bt;
	while ((bt = get()) != -1) {
		line += (char)bt;
		if (bt == '\n') {
			break;
		}
	}
	return line;
}

// read multibytes

size_t chaos_file_data::read(void* buf, size_t size) {
	size_t result = databuf.size()-cur_pos;
	if (result > size) {
		result = size;
	}
	if (result > 0) {
		memcpy(buf, databuf.data()+cur_pos, result);
	}
	cur_pos += result;
	return result;
}

// put 1 byte

void chaos_file_data::put(int bt) {
	databuf.push_back((chaos_uint8)bt);
	cur_pos = databuf.size();
}

// put line

void chaos_file_data::put_line(const chaos_string& line) {
	for (size_t i = 0; i < line.length(); i++) {
		put(line[i]);
	}
}

// write multibytes

void chaos_file_data::write(const void* buf, size_t size) {
	cur_pos = databuf.size();
	databuf.resize(cur_pos+size);
	memcpy(databuf.data()+cur_pos, buf, size);
	cur_pos = databuf.size();
}

// seek

void chaos_file_data::seek(int seeks, seek_type types) {
	switch (types) {
	case seek_set:
		cur_pos = 0;
		break;
	case seek_end:
		cur_pos = databuf.size();
		break;
	}
	if (seeks < 0) {
		if ((size_t)(-seeks) > cur_pos) {
			cur_pos = 0;
		} else {
			cur_pos += seeks;
		}
	} else {
		if ((size_t)seeks > databuf.size()-cur_pos) {
			cur_pos = databuf.size();
		} else {
			cur_pos += seeks;
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// load & store

// load data from file

bool chaos_file_data::load_from_file(FILE* fpt, size_t ofs, size_t size) {
	if (fseek(fpt, ofs, SEEK_SET) != 0) {
		return false;
	}
	databuf.resize(size);
	size_t reads = fread(databuf.data(), 1, size, fpt);
	if (reads != size) {
		return false;
	}
	cur_pos = 0;
	return true;
}

// load data from string

bool chaos_file_data::load_from_string(const chaos_string& str) {
	databuf.resize(str.length());
	for (size_t i = 0; i < str.length(); i++) {
		databuf[i] = (chaos_uint8)str[i];
	}
	cur_pos = 0;
	return true;
}

// store data from file

bool chaos_file_data::store_to_file(FILE* fpt, size_t ofs) const {
	if (fseek(fpt, ofs, SEEK_SET) != 0) {
		return false;
	}
	size_t write = fwrite(databuf.data(), 1, databuf.size(), fpt);
	if (write != databuf.size()) {
		return false;
	}
	return true;
}

// store data from string

bool chaos_file_data::store_to_string(chaos_string& str) const {
	str.resize(databuf.size());
	for (size_t i = 0; i < databuf.size(); i++) {
		str[i] = (char)databuf[i];
	}
	return true;
}

// xor data

void chaos_file_data::xor_data(chaos_uint8 xor_key) {
	chaos_uint8* data = databuf.data();
	size_t size = databuf.size();
	for (size_t i = 0; i < size; i++) {
		*(data++) ^= xor_key;
	}
}
