////////////////////////////////////////////////////////////////////////////////
// chaos_file_data.h : file data

#ifndef CHAOS_FILE_DATA_H_INCLUDED
#define CHAOS_FILE_DATA_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// declare

class chaos_file_data;

////////////////////////////////////////////////////////////////////////////////
// include

#ifndef CHAOS_STRING_H_INCLUDED
#include "chaos_string.h"
#endif // CHAOS_STRING_H_INCLUDED

#ifndef CHAOS_BUFFER_H_INCLUDED
#include "chaos_buffer.h"
#endif // CHAOS_BUFFER_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// chaos_file_data : file data class

class chaos_file_data {
// enum
public:
	// seek position
	enum seek_type {
		seek_set,
		seek_cur,
		seek_end
	};

// attribute
private:
	// filename
	chaos_string filename;
	// data buffer
	chaos_buffer databuf;
	// current position
	size_t cur_pos;

// create & destroy
public:
	// constructor
	chaos_file_data();
	// destructor
	~chaos_file_data();

// operation
public:
	// get filename
	const chaos_string& get_filename() const {
		return filename;
	}
	// set filename
	void set_filename(const chaos_string& new_filename) {
		filename = new_filename;
	}
	// get size
	size_t size() const {
		return databuf.size();
	}
	// is eof
	bool eof() const {
		return cur_pos >= databuf.size();
	}
	// is empty
	bool empty() const {
		return databuf.empty();
	}
	// tell current position
	size_t tell() const {
		return cur_pos;
	}
	// clear data buffer
	void clear() {
		databuf.clear();
	}
	// reserve data buffer
	void reserve(size_t reserve_size) {
		databuf.reserve(reserve_size);
	}
	// get 1 byte
	int get();
	// get line
	chaos_string get_line();
	// read multibytes
	size_t read(void* buf, size_t size);
	// put 1 byte
	void put(int bt);
	// put line
	void put_line(const chaos_string& line);
	// write multibytes
	void write(const void* buf, size_t size);
	// seek
	void seek(int seeks, seek_type types);

// load & store
public:
	// load data from file
	bool load_from_file(FILE* fpt, size_t ofs, size_t size);
	// load data from string
	bool load_from_string(const chaos_string& str);
	// store data from file
	bool store_to_file(FILE* fpt, size_t ofs) const;
	// store data from string
	bool store_to_string(chaos_string& str) const;
	// xor data
	void xor_data(chaos_uint8 xor_key);
};

#endif // CHAOS_FILE_DATA_H_INCLUDED
