////////////////////////////////////////////////////////////////////////////////
// chaos_file_serializer.cpp : file serializer

////////////////////////////////////////////////////////////////////////////////
// include

#include "stdafx.h"

#include "chaos_file_serializer.h"
#include "chaos_exception.h"
#include "chaos_utility.h"

////////////////////////////////////////////////////////////////////////////////
// chaos_file_serializer : file serializer class

////////////////////////////////////////////////////////////////////////////////
// create & destroy

// default constructor

chaos_file_serializer::chaos_file_serializer() {
}

// destructor

chaos_file_serializer::~chaos_file_serializer() {
	clear();
}

// add pack block

bool chaos_file_serializer::add_pack_block(const chaos_string& pack_index) {
	if (pack_index.length() <= 0) {
		return false;
	}
	pack_block_type* block = NULL;
	chaos_string new_info;
	try {
		block = new pack_block_type;
		block->pack_index = pack_index;
		block->pack_file = NULL;
		if (chaos_utility::is_str_sjistrail(
				pack_index, pack_index.length()-1) ||
			(pack_index[pack_index.length()-1] != get_dir_separator()))
		{
			chaos_file_data filedata;
			if (!load_file_data(&filedata, pack_index, fpos_base_dir)) {
				throw chaos_exception(chaos_exception::io_error);
			}
			int index_size = (int)filedata.size();
			filedata.xor_data(encrypt_key);
			if ((filedata.get() != '\0') ||
				(filedata.get() != '@'))
			{
				throw chaos_exception(chaos_exception::bad_format);
			}
			chaos_string pack_substance;
			int ch;
			while ((ch = filedata.get()) != '#') {
				if (ch == -1) {
					throw chaos_exception(chaos_exception::bad_format);
				}
				pack_substance += (char)ch;
			}
			block->pack_substance = pack_substance;
			do {
				chaos_string pack_filename;
				while ((ch = filedata.get()) != '$') {
					if (ch == -1) {
						throw chaos_exception(chaos_exception::bad_format);
					}
					pack_filename += (char)ch;
				}
				pack_pos_type pack_pos;
				pack_pos.offset = 0;
				pack_pos.length = 0;
				while ((ch = filedata.get()) != '*') {
					if ((ch < '0') || (ch > '9')) {
						throw chaos_exception(chaos_exception::bad_format);
					}
					pack_pos.offset = pack_pos.offset*10+(ch-'0');
				}
				while ((ch = filedata.get()) != '+') {
					if ((ch < '0') || (ch > '9')) {
						throw chaos_exception(chaos_exception::bad_format);
					}
					pack_pos.length = pack_pos.length*10+(ch-'0');
				}
				block->pack_poses.insert(
					pack_pos_map_type::value_type(
						get_pure_filename(pack_filename),
						pack_pos));
			} while ((ch = filedata.get()) == '#');
			if ((block->pack_file = fopen(
					chaos_utility::sjis_to_filename(
						base_dir+pack_substance).c_str(),
					"rb")) == NULL)
			{
				throw chaos_exception(chaos_exception::io_error);
			}
			if (fseek(block->pack_file, 0, SEEK_END) != 0) {
				throw chaos_exception(chaos_exception::io_error);
			}
			int substance_size = (int)ftell(block->pack_file);
			if (substance_size == -1) {
				throw chaos_exception(chaos_exception::io_error);
			}
			if (fseek(block->pack_file, 0, SEEK_SET) != 0) {
				throw chaos_exception(chaos_exception::io_error);
			}
			new_info = chaos_utility::format_str(
				"%s,%d,%s,%d",
				chaos_utility::str_to_str(pack_index).c_str(),
				index_size,
				chaos_utility::str_to_str(pack_substance).c_str(),
				substance_size);
		} else {
			new_info = chaos_utility::format_str(
				"%s,%d",
				chaos_utility::str_to_str(pack_index).c_str(),
				-1);
		}
	} catch (std::exception& e) {
		LOG_ERR(
			"exception : file serializer::add pack index - "
			"%s\n",
			e.what());
		if (block != NULL) {
			if (block->pack_file != NULL) {
				fclose(block->pack_file);
			}
			delete block;
		}
		return false;
	}
	pack_blocks.push_front(block);
	if (!new_info.empty()) {
		if (!pack_info.empty()) {
			pack_info += ',';
		}
		pack_info += new_info;
	}
	return true;
}

// clear

void chaos_file_serializer::clear() {
	base_dir = "";
	chaos_dir = "";
	while (pack_blocks.size() > 0) {
		delete pack_blocks.front();
		pack_blocks.pop_front();
	}
	pack_info = "";
}

// get singleton instance

chaos_file_serializer* chaos_file_serializer::instance() {
	static chaos_file_serializer serializer;
	return &serializer;
}

////////////////////////////////////////////////////////////////////////////////
// file data

// exist file

bool chaos_file_serializer::exist_file(
	const chaos_string& filename,
	file_pos_type fpos) const
{
	if (filename.empty()) {
		return false;
	}
	bool exist = false;
	FILE* fpt = NULL;
	if (fpos == fpos_packed) {
		for (
			pack_block_list_type::const_iterator
				it = pack_blocks.begin();
			it != pack_blocks.end();
			it++)
		{
			if ((*it)->pack_file == NULL) {
				chaos_string filepath = base_dir+
					(*it)->pack_index+
					repleace_dir_separator(filename);
				fpt = fopen(
					chaos_utility::sjis_to_filename(filepath).c_str(),
					"rb");
				if (fpt != NULL) {
					exist = true;
					break;
				}
			} else {
				pack_pos_map_type::iterator pos_it =
					(*it)->pack_poses.find(
						get_pure_filename(filename));
				if (pos_it != (*it)->pack_poses.end()) {
					exist = true;
					break;
				}
			}
		}
	} else if (fpos == fpos_chaos_dir) {
		chaos_string filepath = chaos_dir+
			repleace_dir_separator(filename);
		fpt = fopen(
			chaos_utility::sjis_to_filename(filepath).c_str(),
			"rb");
		if (fpt != NULL) {
			exist = true;
		}
	} else {
		chaos_string filepath = base_dir+
			repleace_dir_separator(filename);
		fpt = fopen(
			chaos_utility::sjis_to_filename(filepath).c_str(),
			"rb");
		if (fpt != NULL) {
			exist = true;
		}
	}
	if (fpt != NULL) {
		fclose(fpt);
	}
	return exist;
}

// load file data

bool chaos_file_serializer::load_file_data(
	chaos_file_data* filedata,
	const chaos_string& filename,
	file_pos_type fpos) const
{
	ASSERT(filedata != NULL);
	if (filename.empty()) {
		return false;
	}
	FILE* fpt = NULL;
	bool result = false;
	bool encrypted = false;
	try {
		size_t fileoffset = 0,
			filesize = (size_t)-1;
		if (fpos == fpos_packed) {
			for (
				pack_block_list_type::const_iterator
					it = pack_blocks.begin();
				it != pack_blocks.end();
				it++)
			{
				if ((*it)->pack_file == NULL) {
					chaos_string filepath = base_dir+
						(*it)->pack_index+
						repleace_dir_separator(filename);
					fpt = fopen(
						chaos_utility::sjis_to_filename(filepath).c_str(),
						"rb");
					if (fpt != NULL) {
						break;
					}
				} else {
					pack_pos_map_type::iterator pos_it =
						(*it)->pack_poses.find(
							get_pure_filename(filename));
					if (pos_it != (*it)->pack_poses.end()) {
						fpt = (*it)->pack_file;
						fileoffset = (*pos_it).second.offset;
						filesize = (*pos_it).second.length;
						encrypted = true;
						break;
					}
				}
			}
		} else if (fpos == fpos_chaos_dir) {
			chaos_string filepath = chaos_dir+
				repleace_dir_separator(filename);
			fpt = fopen(
				chaos_utility::sjis_to_filename(filepath).c_str(),
				"rb");
		} else {
			chaos_string filepath = base_dir+
				repleace_dir_separator(filename);
			fpt = fopen(
				chaos_utility::sjis_to_filename(filepath).c_str(),
				"rb");
		}
		if (fpt == NULL) {
			throw chaos_exception(chaos_exception::io_error);
		}
		if (filesize == (size_t)-1) {
			if (fseek(fpt, 0, SEEK_END) != 0) {
				throw chaos_exception(chaos_exception::io_error);
			}
			filesize = (size_t)ftell(fpt);
			if (filesize == (size_t)-1) {
				throw chaos_exception(chaos_exception::io_error);
			}
			if (fseek(fpt, 0, SEEK_SET) != 0) {
				throw chaos_exception(chaos_exception::io_error);
			}
		}
		if (filesize > 0) {
			if (!filedata->load_from_file(fpt, fileoffset, filesize)) {
				throw chaos_exception(chaos_exception::io_error);
			}
			if (encrypted) {
				filedata->xor_data(encrypt_key);
			}
		}
		filedata->set_filename(filename);
		result = true;
	} catch (std::exception& e) {
		LOG_ERR(
			"exception : file serializer::load file data - "
			"%s\n",
			e.what());
	}
	if (!encrypted && (fpt != NULL)) {
		fclose(fpt);
	}
	return result;
}

// save file data

bool chaos_file_serializer::save_file_data(
	const chaos_file_data* filedata,
	const chaos_string& filename,
	file_pos_type fpos) const
{
	if (fpos == fpos_packed) {
		return false;
	}
	ASSERT(filedata != NULL);
	if (filename.empty()) {
		return false;
	}
	FILE* fpt = NULL;
	bool result = false;
	try {
		if (fpos == fpos_chaos_dir) {
			chaos_string filepath = chaos_dir+
				repleace_dir_separator(filename);
			fpt = fopen(
				chaos_utility::sjis_to_filename(filepath).c_str(),
				"wb");
		} else {
			chaos_string filepath = base_dir+
				repleace_dir_separator(filename);
			fpt = fopen(
				chaos_utility::sjis_to_filename(filepath).c_str(),
				"wb");
		}
		if (fpt == NULL) {
			throw chaos_exception(chaos_exception::io_error);
		}
		if (filedata->size() > 0) {
			if (!filedata->store_to_file(fpt, 0)) {
				throw chaos_exception(chaos_exception::io_error);
			}
		}
		if (fflush(fpt) != 0) {
			throw chaos_exception(chaos_exception::io_error);
		}
		result = true;
	} catch (std::exception& e) {
		LOG_ERR(
			"exception : file serializer::save file data - "
			"%s\n",
			e.what());
	}
	if (fpt != NULL) {
		fclose(fpt);
	}
	return result;
}

////////////////////////////////////////////////////////////////////////////////
// operation

// replace directory separator

chaos_string chaos_file_serializer::repleace_dir_separator(
	const chaos_string& org_filename)
{
	chaos_string rep_filename = org_filename;
	bool sjislead = false;
	for (size_t i = 0; i < rep_filename.length(); i++) {
		char ch = rep_filename[i];
		if (!sjislead) {
			sjislead = chaos_utility::is_sjislead(ch);
			if (ch == '/') {
				rep_filename[i] = get_dir_separator();
			}
		} else {
			sjislead = false;
		}
	}
	return rep_filename;
}

// get pure filename

chaos_string chaos_file_serializer::get_pure_filename(
	const chaos_string& org_filename)
{
	chaos_string pure_filename = org_filename;
	if ((org_filename.length() >= 2) &&
		(org_filename[0] == '.') &&
		(org_filename[1] == '/'))
	{
		pure_filename = pure_filename.substr(2);
	}
	pure_filename = chaos_utility::to_lower_str(pure_filename);
	return pure_filename;
}
