////////////////////////////////////////////////////////////////////////////////
// chaos_file_serializer.h : file serializer

#ifndef CHAOS_FILE_SERIALIZER_H_INCLUDED
#define CHAOS_FILE_SERIALIZER_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// declare

class chaos_file_serializer;

////////////////////////////////////////////////////////////////////////////////
// include

#ifndef CHAOS_STRING_H_INCLUDED
#include "chaos_string.h"
#endif // CHAOS_STRING_H_INCLUDED

#ifndef CHAOS_FILE_DATA_H_INCLUDED
#include "chaos_file_data.h"
#endif // CHAOS_FILE_DATA_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// chaos_file_serializer : file serializer class

class chaos_file_serializer {
// enum
private:
	// encrypt
	enum {
		encrypt_key = 0x61
	};

public:
	// file position type
	enum file_pos_type {
		fpos_base_dir,
		fpos_chaos_dir,
		fpos_packed
	};

// struct & typedef
private:
	// pack position type
	struct pack_pos_type {
		size_t offset;
		size_t length;
	};
	// pack position map type
	typedef std::map<chaos_string, pack_pos_type> pack_pos_map_type;
	// pack block type
	struct pack_block_type {
		// pack index
		chaos_string pack_index;
		// pack substance
		chaos_string pack_substance;
		// pack file
		FILE* pack_file;
		// pack position map
		pack_pos_map_type pack_poses;
	};
	// pack block list type
	typedef std::list<pack_block_type*> pack_block_list_type;

// attribute
private:
	// base directory
	chaos_string base_dir;
	// chaos directory
	chaos_string chaos_dir;
	// pack block list
	pack_block_list_type pack_blocks;
	// pack info
	chaos_string pack_info;

// create & destroy
private:
	// default constructor
	chaos_file_serializer();

public:
	// destructor
	~chaos_file_serializer();
	// add pack block
	bool add_pack_block(const chaos_string& pack_index);
	// clear
	void clear();

	// get singleton instance
	static chaos_file_serializer* instance();

// get
public:
	// get base directory
	const chaos_string& get_base_dir() const {
		return base_dir;
	}
	// set base directory
	void set_base_dir(const chaos_string& new_dir) {
		base_dir = new_dir;
	}
	// get chaos directory
	const chaos_string& get_chaos_dir() const {
		return chaos_dir;
	}
	// get chaos directory
	void set_chaos_dir(const chaos_string& new_dir) {
		chaos_dir = new_dir;
	}
	// get pack block count
	int get_pack_block_count() const {
		return pack_blocks.size();
	}
	// get pack info
	const chaos_string& get_pack_info() const {
		return pack_info;
	}

// file data
public:
	// exist file
	bool exist_file(
		const chaos_string& filename,
		file_pos_type fpos) const;
	// load file data
	bool load_file_data(
		chaos_file_data* filedata,
		const chaos_string& filename,
		file_pos_type fpos) const;
	// save file data
	bool save_file_data(
		const chaos_file_data* filedata,
		const chaos_string& filename,
		file_pos_type fpos) const;

// operation
private:
	// replace directory separator
	static chaos_string repleace_dir_separator(
		const chaos_string& org_filename);

public:
	// get directory separator
	static char get_dir_separator() {

#ifdef CHAOS_PLATFORM_WIN32

		return '\\';

#elif defined(CHAOS_PLATFORM_UNIX)

		return '/';

#endif // CHAOS_PLATFORM

	}
	// get pure filename
	static chaos_string get_pure_filename(
		const chaos_string& org_filename);
};

#endif // CHAOS_FILE_SERIALIZER_H_INCLUDED
