////////////////////////////////////////////////////////////////////////////////
// chaos_func_alphablt.cpp : alpha blt function

////////////////////////////////////////////////////////////////////////////////
// include

#include "stdafx.h"

#include "chaos_func_alphablt.h"
#include "chaos_image_realizer.h"
#include "chaos_exception.h"

////////////////////////////////////////////////////////////////////////////////
// chaos_func_alphablt : alpha blt function class

////////////////////////////////////////////////////////////////////////////////
// create & destroy

// default constructor

chaos_func_alphablt::chaos_func_alphablt() :
	chaos_func()
{
}

// destructor

chaos_func_alphablt::~chaos_func_alphablt() {
}

////////////////////////////////////////////////////////////////////////////////
// function

// get name

chaos_string chaos_func_alphablt::get_name() const {
	return "alphablt";
}

// execute

chaos_value chaos_func_alphablt::execute(
	chaos_system* system,
	chaos_interpreter* /*interpreter*/,
	params_type* params) const
{
	// format:
	//   #AlphaBlt(image1, x, y, image2, image3)
	// params:
	//   image1 : target image number(integer)
	//   image2 : rgb source image number(integer)
	//   image3 : alpha source image number(integer)
	//   x      : blt x position(integer)
	//   y      : blt x position(integer)
	// example:
	//   #AlphaBlt(1, 15, 380 , 11, 12);
	if (params->size() < 5) {
		LOG_ERR(
			"error : func alphablt::execute - "
			"too few params\n");
		throw chaos_exception(chaos_exception::bad_format);
	}
	chaos_func::params_type::iterator it = params->begin();
	int image1 = (int)(*(it++)).get_integer();
	int x = (int)(*(it++)).get_integer();
	int y = (int)(*(it++)).get_integer();
	int image2 = (int)(*(it++)).get_integer();
	int image3 = (int)(*it).get_integer();
	chaos_string info =
		chaos_image_realizer::encode_alpha_blt_info(
			x, y,
			system->get_image_info(image1),
			system->get_image_info(image2),
			system->get_image_info(image3));
	system->set_image_info(image1, info);
	return chaos_value();
}
