////////////////////////////////////////////////////////////////////////////////
// chaos_func_blt.cpp : blt function

////////////////////////////////////////////////////////////////////////////////
// include

#include "stdafx.h"

#include "chaos_func_blt.h"
#include "chaos_image_realizer.h"
#include "chaos_exception.h"

////////////////////////////////////////////////////////////////////////////////
// chaos_func_blt : blt function class

////////////////////////////////////////////////////////////////////////////////
// create & destroy

// default constructor

chaos_func_blt::chaos_func_blt() :
	chaos_func()
{
}

// destructor

chaos_func_blt::~chaos_func_blt() {
}

////////////////////////////////////////////////////////////////////////////////
// function

// get name

chaos_string chaos_func_blt::get_name() const {
	return "blt";
}

// execute

chaos_value chaos_func_blt::execute(
	chaos_system* system,
	chaos_interpreter* /*interpreter*/,
	params_type* params) const
{
	// format:
	//   #Blt(image1, x, y, image2)
	// params:
	//   image1 : target image number(integer)
	//   image2 : source image number(integer)
	//   x      : blt x position(integer)
	//   y      : blt x position(integer)
	// example:
	//   #Blt(1, 0, 0, 2);
	if (params->size() < 4) {
		LOG_ERR(
			"error : func blt::execute - "
			"too few params\n");
		throw chaos_exception(chaos_exception::bad_format);
	}
	chaos_func::params_type::iterator it = params->begin();
	int image1 = (int)(*(it++)).get_integer();
	int x = (int)(*(it++)).get_integer();
	int y = (int)(*(it++)).get_integer();
	int image2 = (int)(*(it++)).get_integer();
	chaos_string subinfo1 = system->get_image_info(image1),
		subinfo2 = system->get_image_info(image2);
	// blt optimization
	if ((x == 0) && (y == 0) && !subinfo1.empty()) {
		subinfo1 = "";
	}
	chaos_string info =
		chaos_image_realizer::encode_blt_info(
			x, y,
			subinfo1,
			subinfo2);
	system->set_image_info(image1, info);
	return chaos_value();
}
