////////////////////////////////////////////////////////////////////////////////
// chaos_func_gosub.cpp : gosub function

////////////////////////////////////////////////////////////////////////////////
// include

#include "stdafx.h"

#include "chaos_func_gosub.h"
#include "chaos_exception.h"

////////////////////////////////////////////////////////////////////////////////
// chaos_func_gosub : gosub function class

////////////////////////////////////////////////////////////////////////////////
// create & destroy

// default constructor

chaos_func_gosub::chaos_func_gosub() :
	chaos_func()
{
}

// destructor

chaos_func_gosub::~chaos_func_gosub() {
}

////////////////////////////////////////////////////////////////////////////////
// function

// get name

chaos_string chaos_func_gosub::get_name() const {
	return "gosub";
}

// execute

chaos_value chaos_func_gosub::execute(
	chaos_system* system,
	chaos_interpreter* interpreter,
	params_type* params) const
{
	// format:
	//   #Gosub(label [, filename])
	// params:
	//   label    : subroutine script label(string)
	//   filename : subroutine script filename(string)
	// example:
	//   #Gosub("_Printbg", "tool.txt");
	if (params->size() < 1) {
		LOG_ERR(
			"error : func gosub::execute - "
			"too few params\n");
		throw chaos_exception(chaos_exception::bad_format);
	}
	params_type::iterator it = params->begin();
	chaos_string label = (*(it++)).get_string();
	chaos_string filename;
	int id;
	if (params->size() >= 2) {
		filename = (*it).get_string();
		id = -1;
	} else {
		filename = interpreter->get_script_filename();
		id = interpreter->get_script_id();
	}
	system->request_jump(
		chaos_system::jmp_gosub,
		filename,
		id,
		1, 1,
		label);
	return chaos_value();
}
