////////////////////////////////////////////////////////////////////////////////
// chaos_func_goto.cpp : goto function

////////////////////////////////////////////////////////////////////////////////
// include

#include "stdafx.h"

#include "chaos_func_goto.h"
#include "chaos_file_serializer.h"
#include "chaos_exception.h"
#include "chaos_utility.h"

////////////////////////////////////////////////////////////////////////////////
// chaos_func_goto : goto function class

////////////////////////////////////////////////////////////////////////////////
// create & destroy

// default constructor

chaos_func_goto::chaos_func_goto() :
	chaos_func()
{
}

// destructor

chaos_func_goto::~chaos_func_goto() {
}

////////////////////////////////////////////////////////////////////////////////
// function

// get name

chaos_string chaos_func_goto::get_name() const {
	return "goto";
}

// execute

chaos_value chaos_func_goto::execute(
	chaos_system* system,
	chaos_interpreter* interpreter,
	params_type* params) const
{
	// format:
	//   #Goto(label [, filename])
	// params:
	//   label    : jump script label(string)
	//   filename : jump script filename(string)
	// example:
	//   #Goto("_INPUT");
	//   #Goto("_Restart", "./menu.txt");
	if (params->size() < 1) {
		LOG_ERR(
			"error : func goto::execute - "
			"too few params\n");
		throw chaos_exception(chaos_exception::bad_format);
	}
	params_type::iterator it = params->begin();
	chaos_string label = (*(it++)).get_string();
	chaos_string filename;
	int id;
	if (params->size() >= 2) {
		filename = (*it).get_string();
		id = -1;
	} else {
		filename = interpreter->get_script_filename();
		id = interpreter->get_script_id();
	}
	system->request_jump(
		chaos_system::jmp_goto,
		filename,
		id,
		1, 1,
		label);
	if ((chaos_utility::icompare(
			chaos_file_serializer::get_pure_filename(filename),
			system->get_start_script_filename()) == 0) &&
		(chaos_utility::icompare(
			label,
			system->get_start_script_label()) == 0))
	{
		system->clear_histories();
	}
	return chaos_value();
}
