////////////////////////////////////////////////////////////////////////////////
// chaos_func_if.cpp : if function

////////////////////////////////////////////////////////////////////////////////
// include

#include "stdafx.h"

#include "chaos_func_if.h"
#include "chaos_exception.h"

////////////////////////////////////////////////////////////////////////////////
// chaos_func_if : if function class

////////////////////////////////////////////////////////////////////////////////
// create & destroy

// default constructor

chaos_func_if::chaos_func_if() :
	chaos_func()
{
}

// destructor

chaos_func_if::~chaos_func_if() {
}

////////////////////////////////////////////////////////////////////////////////
// function

// get name

chaos_string chaos_func_if::get_name() const {
	return "if";
}

// execute

chaos_value chaos_func_if::execute(
	chaos_system* /*system*/,
	chaos_interpreter* interpreter,
	params_type* params) const
{
	// format:
	//   #If(expression)
	//   {
	//     [statement1 [statement2 [...]]]
	//   }
	// params:
	//   expression : if expression(boolean)
	// example:
	//   #If(@1023 == 100)
	//   {
	//     #PrintTextf(3);
	//     #Goto("_Start", "./title/cg.txt");
	//   }
	if (params->size() < 1) {
		LOG_ERR(
			"error : func if::execute - "
			"too few params\n");
		throw chaos_exception(chaos_exception::bad_format);
	}
	params_type::iterator it = params->begin();
	bool flag = (*(it++)).get_boolean();
	chaos_token token = interpreter->get_token();
	if (token.get_type() != chaos_token::tkn_braceleft) {
		LOG_ERR(
			"error : func if::execute - "
			"bad token(%s)\n",
			token.get_literal().c_str());
		throw chaos_exception(chaos_exception::bad_format);
	}
	if (!flag) {
		interpreter->skip_to_endblock();
	}
	return chaos_value();
}
