////////////////////////////////////////////////////////////////////////////////
// chaos_func_loadimage.cpp : load image function

////////////////////////////////////////////////////////////////////////////////
// include

#include "stdafx.h"

#include "chaos_func_loadimage.h"
#include "chaos_image_realizer.h"
#include "chaos_exception.h"
#include "chaos_utility.h"

////////////////////////////////////////////////////////////////////////////////
// chaos_func_loadimage : load image function class

////////////////////////////////////////////////////////////////////////////////
// create & destroy

// default constructor

chaos_func_loadimage::chaos_func_loadimage() :
	chaos_func()
{
}

// destructor

chaos_func_loadimage::~chaos_func_loadimage() {
}

////////////////////////////////////////////////////////////////////////////////
// function

// get name

chaos_string chaos_func_loadimage::get_name() const {
	return "loadimage";
}

// execute

chaos_value chaos_func_loadimage::execute(
	chaos_system* system,
	chaos_interpreter* /*interpreter*/,
	params_type* params) const
{
	// format:
	//   #LoadImage(image1, filename)
	//   #LoadImage(image1, filename, image2)
	// params:
	//   image1   : rgb target image number(integer)
	//   image2   : alpha target image number(integer)
	//   filename : image filename(string)
	// example:
	//   #LoadImage(3, "./marron.png");
	//   #LoadImage(11, "./bg/w33.png", 12);
	if (params->size() < 2) {
		LOG_ERR(
			"error : func loadimage::execute - "
			"too few params\n");
		throw chaos_exception(chaos_exception::bad_format);
	}
	chaos_func::params_type::iterator it = params->begin();
	int image1 = (int)(*(it++)).get_integer();
	chaos_string filename = (*it).get_string();
	if ((chaos_utility::str_find_str(filename, "/./") >= 0) &&
		(chaos_utility::str_find_str(
			chaos_utility::to_lower_str(filename), ".png.png") >= 0))
	{
		LOG_WRN(
			"warning : func loadimage::execute - "
			"bad filename(%s)\n",
			filename.c_str());
		filename = "";
	}
	if (!filename.empty()) {
		chaos_string info =
			chaos_image_realizer::encode_load_info(
				filename);
		system->set_image_info(image1, info);
		if (params->size() >= 3) {
			it++;
			int image2 = (int)(*(it++)).get_integer();
			system->set_image_info(image2, info);
		}
	}
	return chaos_value();
}
