////////////////////////////////////////////////////////////////////////////////
// chaos_func_locatef.cpp : locate message function

////////////////////////////////////////////////////////////////////////////////
// include

#include "stdafx.h"

#include "chaos_func_locatef.h"
#include "chaos_exception.h"

////////////////////////////////////////////////////////////////////////////////
// chaos_func_locatef : locate message function class

////////////////////////////////////////////////////////////////////////////////
// create & destroy

// default constructor

chaos_func_locatef::chaos_func_locatef() :
	chaos_func()
{
}

// destructor

chaos_func_locatef::~chaos_func_locatef() {
}

////////////////////////////////////////////////////////////////////////////////
// function

// get name

chaos_string chaos_func_locatef::get_name() const {
	return "locatef";
}

// execute

chaos_value chaos_func_locatef::execute(
	chaos_system* system,
	chaos_interpreter* /*interpreter*/,
	params_type* params) const
{
	// format:
	//   #Locatef(frame, x, y)
	// params:
	//   frame : message frame number(integer)
	//   x     : x position(integer)
	//   y     : y position(integer)
	// example:
	//   #Locatef(0, 35, 392);
	if (params->size() < 3) {
		LOG_ERR(
			"error : func locatef::execute - "
			"too few params\n");
		throw chaos_exception(chaos_exception::bad_format);
	}
	params_type::iterator it = params->begin();
	int frame = (*(it++)).get_integer();
	int x = (*(it++)).get_integer();
	int y = (*it).get_integer();
	int old_x, old_y;
	int length;
	chaos_color color;
	chaos_string info = system->get_message_info(frame);
	chaos_system::decode_message_info(
		info,
		old_x, old_y,
		length,
		color);
	info = chaos_system::encode_message_info(
		x, y,
		length,
		color);
	system->set_message_info(frame, info);
	return chaos_value();
}
