////////////////////////////////////////////////////////////////////////////////
// chaos_func_monofill.cpp : mono fill function

////////////////////////////////////////////////////////////////////////////////
// include

#include "stdafx.h"

#include "chaos_func_monofill.h"
#include "chaos_image_realizer.h"
#include "chaos_exception.h"

////////////////////////////////////////////////////////////////////////////////
// chaos_func_monofill : mono fill function class

////////////////////////////////////////////////////////////////////////////////
// create & destroy

// default constructor

chaos_func_monofill::chaos_func_monofill() :
	chaos_func()
{
}

// destructor

chaos_func_monofill::~chaos_func_monofill() {
}

////////////////////////////////////////////////////////////////////////////////
// function

// get name

chaos_string chaos_func_monofill::get_name() const {
	return "monofill";
}

// execute

chaos_value chaos_func_monofill::execute(
	chaos_system* system,
	chaos_interpreter* /*interpreter*/,
	params_type* params) const
{
	// format:
	//   #MonoFill(image1, x, y, width, height, color)
	// params:
	//   image  : target image number(integer)
	//   x      : fill x position(integer)
	//   y      : fill y position(integer)
	//   width  : fill width(integer)
	//   height : fill heght(integer)
	//   color  : fill color(color)
	// example:
	//   #MonoFill(1, 0, 0, 800, 600, RGB(0, 0, 0));
	if (params->size() < 6) {
		LOG_ERR(
			"error : func mono fill::execute - "
			"too few params\n");
		throw chaos_exception(chaos_exception::bad_format);
	}
	chaos_func::params_type::iterator it = params->begin();
	int image = (int)(*(it++)).get_integer();
	int x = (int)(*(it++)).get_integer();
	int y = (int)(*(it++)).get_integer();
	int width = (int)(*(it++)).get_integer();
	int height = (int)(*(it++)).get_integer();
	chaos_color color = (*it).get_color();
	chaos_string info =
		chaos_image_realizer::encode_mono_fill_info(
			x, y,
			width, height,
			color,
			system->get_image_info(image));
	system->set_image_info(image, info);
	return chaos_value();
}
