////////////////////////////////////////////////////////////////////////////////
// chaos_func_mouseclickeventex.cpp : mouse click event function

////////////////////////////////////////////////////////////////////////////////
// include

#include "stdafx.h"

#include "chaos_func_mouseclickeventex.h"
#include "chaos_exception.h"
#include "chaos_utility.h"

////////////////////////////////////////////////////////////////////////////////
// chaos_func_mouseclickeventex : mouse click event function class

////////////////////////////////////////////////////////////////////////////////
// create & destroy

// default constructor

chaos_func_mouseclickeventex::chaos_func_mouseclickeventex() :
	chaos_func()
{
}

// destructor

chaos_func_mouseclickeventex::~chaos_func_mouseclickeventex() {
}

////////////////////////////////////////////////////////////////////////////////
// function

// get name

chaos_string chaos_func_mouseclickeventex::get_name() const {
	return "mouseclickeventex";
}

// execute

chaos_value chaos_func_mouseclickeventex::execute(
	chaos_system* system,
	chaos_interpreter* interpreter,
	params_type* /*params*/) const
{
	// format:
	//   #MouseClickEventEX()
	//   {
	//     param-block1 [param-block2 [...]]
	//   }
	// param-block:
	//   x1, y1, x2, y2, label, filename, unk1, unk2, unk3, color
	// params:
	//   x1       : serection rect left position(integer)
	//   y1       : serection rect top position(integer)
	//   x2       : serection rect right position(integer)
	//   y2       : serection rect bottom position(integer)
	//   label    : jump script label(string)
	//   filename : jump script filename(string)
	//   color    : selection color(color)
	//   unk1     : unknown(integer)
	//   unk2     : unknown(integer)
	//   unk3     : unknown(integer)
	// example:
	//   #MouseClickEventEX()
	//   {
	//     40, 220, 190, 270, "_Name", "./script.txt", 1, 0, 3, RGB(48, 48, 48)
	//     190, 270, 340, 320, "_Load", "./menu.txt", 1, 0, 3, RGB(48, 48. 48)
	//     190, 370, 340, 420, "_End", "./menu.txt", 1, 0, 3, RGB(48, 48. 48)
	//   }

	chaos_token token = interpreter->get_token();
	if (token.get_type() != chaos_token::tkn_braceleft) {
		LOG_ERR(
			"error : func mouseclickeventex::execute - "
			"bad token(%s)\n",
			token.get_literal().c_str());
		throw chaos_exception(chaos_exception::bad_format);
	}
	int count = 0;
	try {
		while (true) {
			token = interpreter->get_token();
			if (token.get_type() == chaos_token::tkn_braceright) {
				break;
			}
			interpreter->unget_token(token);
			if (count >= 100) {
				LOG_ERR(
				"error : func mouseclickeventex::execute - "
					"params overflow(%s)\n",
					token.get_literal().c_str());
				throw chaos_exception(chaos_exception::bad_format);
			}
			int x1 = 0, y1 = 0, x2 = 0, y2 = 0,
				unk1 = 0, unk2 = 0, unk3 = 0;
			chaos_string label, filename;
			chaos_color color;
			for (int i = 0; i < 10; i++) {
				token = interpreter->let_expression();
				switch (i) {
				case 0:
					x1 = (int)token.get_value().get_integer();
					break;
				case 1:
					y1 = (int)token.get_value().get_integer();
					break;
				case 2:
					x2 = (int)token.get_value().get_integer();
					break;
				case 3:
					y2 = (int)token.get_value().get_integer();
					break;
				case 4:
					label = token.get_value().get_string();
					break;
				case 5:
					filename = token.get_value().get_string();
					break;
				case 6:
					unk1 = (int)token.get_value().get_integer();
					break;
				case 7:
					unk2 = (int)token.get_value().get_integer();
					break;
				case 8:
					unk3 = (int)token.get_value().get_integer();
					break;
				case 9:
					color = token.get_value().get_color();
					break;
				}
				if (i < 9) {
					token = interpreter->get_token();
					if (token.get_type() != chaos_token::tkn_comma) {
						LOG_ERR(
							"error : func mouseclickeventex::execute - "
							"bad token(%s)\n",
							token.get_literal().c_str());
						throw chaos_exception(chaos_exception::bad_format);
					}
				}
			}
			int types = 1;
			chaos_string info = chaos_system::encode_selection_info(
				types,
				x1, y1, x2, y2,
				unk1, unk2, unk3,
				color,
				filename,
				label);
			system->set_selection_info(count, info);
			count++;
		}
	} catch (...) {
		system->clear_selection();
		throw;
	}
	if (count > 0) {
		system->set_selection_count(count);
		system->change_skip_mode(chaos_system::skip_none);
		system->change_system_mode(chaos_system::sysmode_selevent);
	}
	return chaos_value();
}
