////////////////////////////////////////////////////////////////////////////////
// chaos_func_playwave.cpp : play pcm audio function

////////////////////////////////////////////////////////////////////////////////
// include

#include "stdafx.h"

#include "chaos_func_playwave.h"
#include "chaos_exception.h"

////////////////////////////////////////////////////////////////////////////////
// chaos_func_playwave : play pcm audio function class

////////////////////////////////////////////////////////////////////////////////
// create & destroy

// default constructor

chaos_func_playwave::chaos_func_playwave() :
	chaos_func()
{
}

// destructor

chaos_func_playwave::~chaos_func_playwave() {
}

////////////////////////////////////////////////////////////////////////////////
// function

// get name

chaos_string chaos_func_playwave::get_name() const {
	return "playwave";
}

// execute

chaos_value chaos_func_playwave::execute(
	chaos_system* system,
	chaos_interpreter* /*interpreter*/,
	params_type* params) const
{
	// format:
	//   #PlayWave(filename)
	//   #PlayWave(filename, unk)
	//   #PlayWave(filename, unk, repeat)
	// params:
	//   filename : pcm audio filename(string)
	//   unk      : unknown(integer)
	//   repeat   : repeat(integer)
	// example:
	//   #PlayWave("./se/folkdance.wav");
	//   #PlayWave("./se/asa.wav", 0);
	//   #PlayWave("./se/folkdance.wav", 0, 1);
	if (params->size() < 1) {
		LOG_ERR(
			"error : func playwave::execute - "
			"too few params\n");
		throw chaos_exception(chaos_exception::bad_format);
	}
	chaos_func::params_type::iterator it = params->begin();
	chaos_string filename = (*(it++)).get_string();
	int unk = 0, repeat = 0;
	if (params->size() >= 2) {
		unk = (int)(*(it++)).get_integer();
	}
	if (params->size() >= 3) {
		repeat = (int)(*it).get_integer();
	}
	chaos_string info =
		chaos_system::encode_wave_audio_info(
			unk, repeat,
			filename);
	system->play_wave_audio(info);
	return chaos_value();
}
