////////////////////////////////////////////////////////////////////////////////
// chaos_func_printtextf.cpp : print message function

////////////////////////////////////////////////////////////////////////////////
// include

#include "stdafx.h"

#include "chaos_func_printtextf.h"
#include "chaos_exception.h"
#include "chaos_utility.h"

////////////////////////////////////////////////////////////////////////////////
// chaos_func_printtextf : print message function class

////////////////////////////////////////////////////////////////////////////////
// create & destroy

// default constructor

chaos_func_printtextf::chaos_func_printtextf() :
	chaos_func()
{
}

// destructor

chaos_func_printtextf::~chaos_func_printtextf() {
}

////////////////////////////////////////////////////////////////////////////////
// function

// get name

chaos_string chaos_func_printtextf::get_name() const {
	return "printtextf";
}

// execute

chaos_value chaos_func_printtextf::execute(
	chaos_system* system,
	chaos_interpreter* /*interpreter*/,
	params_type* params) const
{
	// format:
	//   #PrintTextf(script_id)
	//   #PrintTextf(frame, script_id, label, filename)
	//   #PrintTextf(script_id, subcolor)
	//   #PrintTextf(unk, script_id)
	// params:
	//   script_id : message id number(integer)
	//   frame     : message frame number(integer)
	//   label     : jump script label(string)
	//   filename  : jump script filename(string)
	//   subcolor  : message substractive mixture color(color)
	//   unk       : unknown(integer)
	// example:
	//   #PrintTextf(1);
	//   #PrintTextf(1, 473, "_Start", "./1day/scr2ex1.txt");
	//   #PrintTextf(10, RGB(128, 128, 256));
	//   #PrintTextf(0, 77);
	if (params->size() < 1) {
		LOG_ERR(
			"error : func printtextf::execute - "
			"too few params\n");
		throw chaos_exception(chaos_exception::bad_format);
	}
	chaos_func::params_type::iterator it = params->begin();
	int script_id = 0;
	if (params->size() <= 1) {
		// type 1
		script_id = (int)(*it).get_integer();
	} else if (params->size() >= 4) {
		// type 2
		int frame = (int)(*(it++)).get_integer();
		script_id = (int)(*(it++)).get_integer();
		chaos_string label = (*(it++)).get_string();
		chaos_string filename = (*it).get_string();
		int x, y;
		int length;
		chaos_color mcolor;
		chaos_string minfo = system->get_message_info(frame);
		chaos_system::decode_message_info(
			minfo,
			x, y,
			length,
			mcolor);
		chaos_color scolor = system->get_selection_default_color();
		int font_size = system->get_font_size();
		int count = system->get_selection_count();
		int types = 1;
		int x1 = x+font_size;
		int y1 = y+font_size*count;
		int x2 = x1+font_size*length+font_size/2;
		int y2 = y1+font_size;
		int unk1 = frame;
		int unk2 = 0;
		int unk3 = 0;
		chaos_string sinfo = chaos_system::encode_selection_info(
			types,
			x1, y1, x2, y2,
			unk1, unk2, unk3,
			scolor,
			filename,
			label);
		system->set_selection_info(count, sinfo);
	} else {
		chaos_value val1 = *(it++);
		chaos_value val2 = *it;
		if (val2.get_type() == chaos_value::color) {
			// type 3
			script_id = (int)val1.get_integer();
			chaos_color subcolor = val2.get_color();
			system->set_message_subcolor(subcolor);
		} else {
			// type 4
		}
	}
	if (script_id > 0) {
		system->request_jump(
			chaos_system::jmp_gosub,
			system->get_cur_message_filename(),
			script_id,
			1, 1,
			"");
	}
	return chaos_value();
}
