////////////////////////////////////////////////////////////////////////////////
// chaos_func_setdefaultcolor.cpp : set message default color function

////////////////////////////////////////////////////////////////////////////////
// include

#include "stdafx.h"

#include "chaos_func_setdefaultcolor.h"
#include "chaos_exception.h"

////////////////////////////////////////////////////////////////////////////////
// chaos_func_setdefaultcolor : set message default color function class

////////////////////////////////////////////////////////////////////////////////
// create & destroy

// default constructor

chaos_func_setdefaultcolor::chaos_func_setdefaultcolor() :
	chaos_func()
{
}

// destructor

chaos_func_setdefaultcolor::~chaos_func_setdefaultcolor() {
}

////////////////////////////////////////////////////////////////////////////////
// function

// get name

chaos_string chaos_func_setdefaultcolor::get_name() const {
	return "setdefaultcolor";
}

// execute

chaos_value chaos_func_setdefaultcolor::execute(
	chaos_system* system,
	chaos_interpreter* /*interpreter*/,
	params_type* params) const
{
	// format:
	//   #SetDefaultColor(frame, color)
	// params:
	//   frame : frame number(integer)
	//   color : message default color(integer)
	// example:
	//   #SetDefaultColor(0, RGB(255, 255, 255));
	if (params->size() < 2) {
		LOG_ERR(
			"error : func setdefaultcolor::execute - "
			"too few params\n");
		throw chaos_exception(chaos_exception::bad_format);
	}
	params_type::iterator it = params->begin();
	int frame = (*(it++)).get_integer();
	chaos_color color = (*it).get_color();
	int x, y;
	int length;
	chaos_color old_color;
	chaos_string info = system->get_message_info(frame);
	chaos_system::decode_message_info(
		info,
		x, y,
		length,
		old_color);
	info = chaos_system::encode_message_info(
		x, y,
		length,
		color);
	system->set_message_info(frame, info);
	return chaos_value();
}
