////////////////////////////////////////////////////////////////////////////////
// chaos_func_transfade.cpp : trans fade function

////////////////////////////////////////////////////////////////////////////////
// include

#include "stdafx.h"

#include "chaos_func_transfade.h"
#include "chaos_image_realizer.h"
#include "chaos_exception.h"

////////////////////////////////////////////////////////////////////////////////
// chaos_func_transfade : trans fade function class

////////////////////////////////////////////////////////////////////////////////
// create & destroy

// default constructor

chaos_func_transfade::chaos_func_transfade() :
	chaos_func()
{
}

// destructor

chaos_func_transfade::~chaos_func_transfade() {
}

////////////////////////////////////////////////////////////////////////////////
// function

// get name

chaos_string chaos_func_transfade::get_name() const {
	return "transfade";
}

// execute

chaos_value chaos_func_transfade::execute(
	chaos_system* system,
	chaos_interpreter* /*interpreter*/,
	params_type* params) const
{
	// format:
	//   #TransFade(unk1, unit, unk2, unk3, x, y, width, height, image)
	// params:
	//   unit   : fade unit(integer)
	//   x      : fill x position(integer)
	//   y      : fill y position(integer)
	//   width  : fill width(integer)
	//   height : fill heght(integer)
	//   image  : source image number(integer)
	//   unk1   : unknown(integer)
	//   unk2   : unknown(integer)
	//   unk3   : unknown(integer)
	// example:
	//   #TransFade(-1, 64, 255, 0, 0, 0, 800, 600, 3);
	if (params->size() < 9) {
		LOG_ERR(
			"error : func transfade::execute - "
			"too few params\n");
		throw chaos_exception(chaos_exception::bad_format);
	}
	chaos_func::params_type::iterator it = params->begin();
	int unk1 = (int)(*(it++)).get_integer();
	int unit = (int)(*(it++)).get_integer();
	int unk2 = (int)(*(it++)).get_integer();
	int unk3 = (int)(*(it++)).get_integer();
	int x = (int)(*(it++)).get_integer();
	int y = (int)(*(it++)).get_integer();
	int width = (int)(*(it++)).get_integer();
	int height = (int)(*(it++)).get_integer();
	int image = (int)(*it).get_integer();
	chaos_string prev_info, now_info, next_info;
	prev_info = system->get_image_info(0);
	next_info = system->get_image_info(image);
	now_info = chaos_image_realizer::encode_trans_fade_info(
		0,
		unk1, unit, unk2, unk3,
		x, y,
		width, height,
		prev_info, next_info);
	int effect_step = system->get_effect_step();
	if (effect_step < 1) {
		effect_step = 1;
	}
	if (system->is_effect_cut()) {
		now_info = "";
	}
	bool dirty_image = true, dirty_message = false;
	if (system->get_skip_mode() != chaos_system::skip_none) {
		switch (system->get_skip_speed()) {
		case chaos_system::sspeed_fast:
			now_info = "";
			dirty_message = true;
			break;
		case chaos_system::sspeed_jump:
			now_info = "";
			dirty_image = false;
			break;
		}
	}
	system->set_image_info(1, next_info);
	if (now_info.empty()) {
		system->set_image_info(0, next_info);
	} else {
		if (effect_step > 1) {
			if (!chaos_image_realizer::is_effect_spippable(
					now_info))
			{
				effect_step = 1;
			}
		}
		system->set_image_info(0, now_info);
		system->change_system_mode(chaos_system::sysmode_effect);
		system->set_wait_time(
			(int)chaos_system::effect_unit_time*
				effect_step*
				chaos_image_realizer::get_effect_time_scale(
					now_info));
	}
	if (dirty_image) {
		system->set_dirty_image(true);
	}
	if (dirty_message) {
		system->set_dirty_message(true);
	}
	return chaos_value();
}
