////////////////////////////////////////////////////////////////////////////////
// chaos_func_transfill.cpp : trans fill function

////////////////////////////////////////////////////////////////////////////////
// include

#include "stdafx.h"

#include "chaos_func_transfill.h"
#include "chaos_image_realizer.h"
#include "chaos_exception.h"

////////////////////////////////////////////////////////////////////////////////
// chaos_func_transfill : trans fill function class

////////////////////////////////////////////////////////////////////////////////
// create & destroy

// default constructor

chaos_func_transfill::chaos_func_transfill() :
	chaos_func()
{
}

// destructor

chaos_func_transfill::~chaos_func_transfill() {
}

////////////////////////////////////////////////////////////////////////////////
// function

// get name

chaos_string chaos_func_transfill::get_name() const {
	return "transfill";
}

// execute

chaos_value chaos_func_transfill::execute(
	chaos_system* system,
	chaos_interpreter* /*interpreter*/,
	params_type* params) const
{
	// format:
	//   #TransFill(image1, x, y, width, height, color)
	//   #TransFill(unk1, unk2, unk3, ink4, x, y, width, height, image2)
	// params:
	//   image1 : target image number(integer)
	//   image2 : source image number(integer)
	//   x      : fill x position(integer)
	//   y      : fill y position(integer)
	//   width  : fill width(integer)
	//   height : fill heght(integer)
	//   color  : fill color(color)
	//   unk1   : unknown(integer)
	//   unk2   : unknown(integer)
	//   unk3   : unknown(integer)
	//   unk4   : unknown(integer)
	// example:
	//   #TransFill(1, 39, 387, 211, 40, RGB(-48, -48, -48));
	//   #TransFill(-1, 40, 255, 0, 0, 0, 800, 600, 3);
	if (params->size() < 6) {
		LOG_ERR(
			"error : func trans fill::execute - "
			"too few params\n");
		throw chaos_exception(chaos_exception::bad_format);
	}
	chaos_func::params_type::iterator it = params->begin();
	if (params->size() < 8) {
		// type 1
		int image1 = (int)(*(it++)).get_integer();
		int x = (int)(*(it++)).get_integer();
		int y = (int)(*(it++)).get_integer();
		int width = (int)(*(it++)).get_integer();
		int height = (int)(*(it++)).get_integer();
		chaos_color color = (*it).get_color();
		chaos_string info =
			chaos_image_realizer::encode_trans_fill_info(
				x, y,
				width, height,
				color,
				system->get_image_info(image1));
		system->set_image_info(image1, info);
	} else {
		// type 2
	}
	return chaos_value();
}
