////////////////////////////////////////////////////////////////////////////////
// chaos_func_wait.cpp : wait function

////////////////////////////////////////////////////////////////////////////////
// include

#include "stdafx.h"

#include "chaos_func_wait.h"
#include "chaos_exception.h"

////////////////////////////////////////////////////////////////////////////////
// chaos_func_wait : wait function class

////////////////////////////////////////////////////////////////////////////////
// create & destroy

// default constructor

chaos_func_wait::chaos_func_wait() :
	chaos_func()
{
}

// destructor

chaos_func_wait::~chaos_func_wait() {
}

////////////////////////////////////////////////////////////////////////////////
// function

// get name

chaos_string chaos_func_wait::get_name() const {
	return "wait";
}

// execute

chaos_value chaos_func_wait::execute(
	chaos_system* system,
	chaos_interpreter* /*interpreter*/,
	params_type* params) const
{
	// format:
	//   #Wait(time)
	// params:	
	//   time : wait time(integer)
	// example:
	//   #Wait(1500);
	if (params->size() < 1) {
		LOG_ERR(
			"error : func wait::execute - "
			"too few params\n");
		throw chaos_exception(chaos_exception::bad_format);
	}
	chaos_func::params_type::iterator it = params->begin();
	int time = (int)(*it).get_integer();
	if (system->get_skip_mode() != chaos_system::skip_none) {
		switch (system->get_skip_speed()) {
		case chaos_system::sspeed_fast:
		case chaos_system::sspeed_jump:
			time = 0;
			break;
		}
	}
	if (time > 0) {
		system->change_system_mode(chaos_system::sysmode_wait);
		system->set_wait_time(time);
	}
	return chaos_value();
}
