////////////////////////////////////////////////////////////////////////////////
// chaos_message_manager.cpp : message manager

////////////////////////////////////////////////////////////////////////////////
// include

#include "stdafx.h"

#include "chaos_message_manager.h"
#include "chaos_utility.h"

////////////////////////////////////////////////////////////////////////////////
// message manager class

////////////////////////////////////////////////////////////////////////////////
// create & destroy

// default constructor

chaos_message_manager::chaos_message_manager() :
	max_id(0)
{
}

// destructor

chaos_message_manager::~chaos_message_manager() {
}

// clear

void chaos_message_manager::clear() {
	messages.clear();
}

// load

bool chaos_message_manager::load(chaos_file_data* filedata) {
	messages.clear();
	int ch;
	bool result = true;
	int cur_max_id = 0;
	do {
		while ((ch = filedata->get()) != -1) {
			if (!chaos_utility::is_space(ch)) {
				break;
			}
		}
		if (ch == -1) {
			break;
		}
		if (chaos_utility::is_digit(ch)) {
			int id = ch-'0';
			while ((ch = filedata->get()) != -1) {
				if (!chaos_utility::is_digit(ch)) {
					break;
				}
				id = id*10+ch-'0';
			}
			while (chaos_utility::is_space(ch)) {
				ch = filedata->get();
			}
			chaos_string message;
			bool sjislead = false;
			enum {
				mm_normal,
				mm_dquote,
				mm_dquote2,
				mm_escape,
				mm_parenleft,
				mm_parenright,
				mm_finish
			} mmode = mm_normal;
			if (ch != ',') {
				message = (char)ch;
				if (chaos_utility::is_sjislead(ch)) {
					sjislead = true;
				} else if (ch == '"') {
					mmode = mm_dquote;
				}
			}
			while (
				(mmode != mm_finish) &&
				((ch = filedata->get()) != -1))
			{
				if (ch == '\n') {
					break;
				} else if (ch == '\r') {
					// nothing
				} else if (!sjislead) {
					message += (char)ch;
					sjislead = chaos_utility::is_sjislead(ch);
					switch (mmode) {
					case mm_normal:
						switch (ch) {
						case ';':
							mmode = mm_finish;
							break;
						case '"':
							mmode = mm_dquote;
							break;
						}
						break;
					case mm_dquote:
						switch (ch) {
						case '"':
							mmode = mm_normal;
							break;
						case '\\':
							mmode = mm_escape;
							break;
						}
						break;
					case mm_dquote2:
						switch (ch) {
						case '"':
							mmode = mm_parenright;
							break;
						}
						break;
					case mm_escape:
						switch (ch) {
						case '\\':
							mmode = mm_dquote;
							break;
						case '#':
						case 'c':
						case 'C':
							mmode = mm_parenleft;
							break;
						default:
							mmode = mm_dquote;
							break;
						}
						break;
					case mm_parenleft:
						switch (ch) {
						case '(':
							mmode = mm_parenright;
							break;
						}
						break;
					case mm_parenright:
						switch (ch) {
						case ')':
							mmode = mm_dquote;
							break;
						case '"':
							mmode = mm_dquote2;
							break;
						}
						break;
					}
				} else {
					message += (char)ch;
					sjislead = false;
				}
			}
			if (mmode == mm_finish) {
				ch = filedata->get();
				if (ch == '\r') {
					ch = filedata->get();
				}
				if ((ch != -1) && (ch != '\n')) {
					filedata->seek(-1, chaos_file_data::seek_cur);
				}
			}
			if (messages.find(id) != messages.end()) {
				id = cur_max_id+1;
			}
			messages[id] = message;
			if (id > max_id) {
				max_id = id;
			}
			if (id > cur_max_id) {
				cur_max_id = id;
			}
		} else if (ch == '\\') {
			ch = filedata->get();
			if (chaos_utility::to_lower((char)ch) == 'e') {
				break;
			}
			result = false;
		} else {
			break;
		}
	} while (result);
	ASSERT(result);
	return result;
}

////////////////////////////////////////////////////////////////////////////////
// get & set

// get message

chaos_string chaos_message_manager::get_message(int id) const {
	chaos_string message;
	message_map::const_iterator it = messages.find(id);
	if (it != messages.end()) {
		message = (*it).second;
	}
	return message;
}
