////////////////////////////////////////////////////////////////////////////////
// chaos_pos.h : 2d position

#ifndef CHAOS_POS_H_INCLUDED
#define CHAOS_POS_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// declare

class chaos_pos;

////////////////////////////////////////////////////////////////////////////////
// include

#ifndef CHAOS_DIM_H_INCLUDED
#include "chaos_dim.h"
#endif // CHAOS_DIM_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// chaos_pos : 2d position class

class chaos_pos {
// attribute
public:
	// x position
	int x;
	// y position
	int y;

// create & destroy
public:
	// default constructor
	chaos_pos() :
		x(0), y(0)
	{
	}
	// normal constructor
	chaos_pos(int new_x, int new_y) :
		x(new_x), y(new_y)
	{
	}
	// copy constructor
	chaos_pos(const chaos_pos& org) :
		x(org.x), y(org.y)
	{
	}
	// 2d dimension constructor
	chaos_pos(const chaos_dim& org);

// operation
public:
	// clear
	void clear() {
		x = y = 0;
	}

// operator
public:
	// copy
	chaos_pos& operator=(const chaos_pos& org) {
		x = org.x;
		y = org.y;
		return *this;
	}
	// add let
	chaos_pos& operator+=(const chaos_pos& org) {
		x += org.x;
		y += org.y;
		return *this;
	}
	// sub let
	chaos_pos& operator-=(const chaos_pos& org) {
		x -= org.x;
		y -= org.y;
		return *this;
	}
	// add
	chaos_pos operator+(const chaos_pos& org) const {
		chaos_pos tmp = *this;
		tmp += org;
		return tmp;
	}
	// sub
	chaos_pos operator-(const chaos_pos& org) const {
		chaos_pos tmp = *this;
		tmp -= org;
		return tmp;
	}
	// equal
	bool operator==(const chaos_pos& org) const {
		return (x == org.x) && (y == org.y);
	}
	// not equal
	bool operator!=(const chaos_pos& org) const {
		return (x != org.x) || (y != org.y);
	}
};

#endif // CHAOS_POS_H_INCLUDED
