////////////////////////////////////////////////////////////////////////////////
// chaos_readflag_manager.h : readflag manager

#ifndef CHAOS_READFLAG_MANAGER_H_INCLUDED
#define CHAOS_READFLAG_MANAGER_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// declare

class chaos_readflag_manager;

////////////////////////////////////////////////////////////////////////////////
// include

#ifndef CHAOS_STRING_H_INCLUDED
#include "chaos_string.h"
#endif // CHAOS_STRING_H_INCLUDED

#ifndef CHAOS_BUFFER_H_INCLUDED
#include "chaos_buffer.h"
#endif // CHAOS_BUFFER_H_INCLUDED

#ifndef CHAOS_XML_H_INCLUDED
#include "chaos_xml.h"
#endif // CHAOS_XML_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// chaos_readflag_manager : readflag manager class

class chaos_readflag_manager {
// enum
private:
	// limit
	enum {
		id_limit = 9999
	};

// typedef
private:
	// key map
	typedef std::map<chaos_string, int> key_map;
	// readflag map
	typedef std::map<int, chaos_buffer*> readflag_map;

// attribute
private:
	// key map
	key_map keys;
	// readflags
	readflag_map readflags;
	// next key
	int next_key;

// create & destroy
public:
	// default constructor
	chaos_readflag_manager();
	// destructor
	~chaos_readflag_manager();
	// clear
	void clear();

// operation
public:
	// get key
	int get_key(const chaos_string& filename) const;
	// regist
	int regist(const chaos_string& filename);
	// reserve
	void reserve(
		int key,
		size_t capacity);
	void reserve(
		const chaos_string& filename,
		size_t capacity);
	// is read
	bool is_read(
		int key,
		int id) const;
	bool is_read(
		const chaos_string& filename,
		int id) const;
	// set read
	bool set_read(
		int key,
		int id,
		bool read);
	bool set_read(
		const chaos_string& filename,
		int id,
		bool read);

// xml
public:
	// load
	bool load(const chaos_xml* readf_xml);
	// save
	bool save(chaos_xml* readf_xml) const;
};

#endif // CHAOS_READFLAG_MANAGER_H_INCLUDED
