////////////////////////////////////////////////////////////////////////////////
// chaos_resource.h : resource

#ifndef CHAOS_RESOURCE_H_INCLUDED
#define CHAOS_RESOURCE_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// declare

class chaos_resource;

////////////////////////////////////////////////////////////////////////////////
// include

#ifndef CHAOS_STRING_H_INCLUDED
#include "chaos_string.h"
#endif // CHAOS_STRING_H_INCLUDED

#ifndef CHAOS_BUFFER_H_INCLUDED
#include "chaos_buffer.h"
#endif // CHAOS_BUFFER_H_INCLUDED

#ifndef CHAOS_VALUE_MANAGER_H_INCLUDED
#include "chaos_value_manager.h"
#endif // CHAOS_VALUE_MANAGER_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// chaos_resource : resource class

class chaos_resource {
// enum
public:
	// resource item type
	enum resource_item_type {
		item_none,
		app_name,
		app_version,
		main_menu_skip_read,
		main_menu_appreciate,
		main_menu_save,
		main_menu_load,
		main_menu_recollect,
		main_menu_config,
		main_menu_restart,
		main_menu_exit,
		recollect_pageup,
		recollect_pagedown,
		def_lastname,
		def_firstname,
		def_nickname,
		def_postfix,
		def_font_name,
		def_font_xlfd,
		def_font_pango,
		tspeed_superslow,
		tspeed_slow,
		tspeed_normal,
		tspeed_fast,
		tspeed_superfast,
		tspeed_instant,
		sspeed_normal,
		sspeed_fast,
		sspeed_jump,
		logmode_file,
		logmode_window,
		loglv_none,
		loglv_error,
		loglv_warning,
		loglv_report,
		cd_drive_auto,
		font_sample_text,
		start_text,
		dlg_std_action_area,
		dlg_message_box,
		dlg_start,
		dlg_start_action_area,
		dlg_config,
		caption_font_select,
		err_cant_find_script,
		err_cant_find_cd_rom
	};

// attribute
private:
	// main menu item
	std::vector<resource_item_type> main_menu_items;
	// resource text
	std::map<resource_item_type, chaos_string> resource_texts;

	// suzu picture
	static chaos_uint8 suzu_picture[];

// create & destroy
private:
	// default constructor
	chaos_resource();

public:
	// destructor
	~chaos_resource();

	// get singleton instance
	static chaos_resource* instance();

// operation
public:
	// get main menu count
	int get_main_menu_count() const {
		return (int)main_menu_items.size();
	}
	// get main menu item
	resource_item_type get_main_menu_item(int menuno) const {
		return
			((menuno >= 0) && (menuno < get_main_menu_count()))?
			main_menu_items[menuno]:
			item_none;
	}
	// get main menu text
	chaos_string get_main_menu_text(int menuno) const {
		return get_resource_text(
			get_main_menu_item(menuno));
	}
	// bind resource text
	void bind_resource_text(
		resource_item_type item,
		const chaos_string& text);
	// bind resource text
	chaos_string get_resource_text(
		resource_item_type item) const;
	// get fix values
	void get_fixes(
		chaos_value_manager<chaos_string>* fixes) const;
	// get default system properties
	void get_def_sysproperties(
		chaos_value_manager<chaos_string>* sysproperties) const;

// picture
public:
	// get suzu picture
	void get_suzu_picture(chaos_buffer* buffer);
};

#endif // CHAOS_RESOURCE_H_INCLUDED
