////////////////////////////////////////////////////////////////////////////////
// chaos_start_dlg.cpp : start dialog

////////////////////////////////////////////////////////////////////////////////
// include

#include "stdafx.h"

#include "chaos_start_dlg.h"
#include "chaos_config_dlg.h"
#include "chaos_file_data.h"
#include "chaos_png_image_loader.h"
#include "chaos_resource.h"
#include "chaos_utility.h"

#include "resource.h"

////////////////////////////////////////////////////////////////////////////////
// chaos_start_dlg: start dialog class

////////////////////////////////////////////////////////////////////////////////
// create & destroy

// default constructor

chaos_start_dlg::chaos_start_dlg() :
	chaos_dlg(),
	dirty(false),
	suzu_image(NULL)
{

#ifdef CHAOS_GUI_GTK

	suzu_pixmap = NULL;

#endif // CHAOS_GUI

}

// destructor

chaos_start_dlg::~chaos_start_dlg() {
}

// get singleton instance

chaos_start_dlg* chaos_start_dlg::instance() {
	static chaos_start_dlg start_dlg;
	return &start_dlg;
}

////////////////////////////////////////////////////////////////////////////////
// dialog

#ifdef CHAOS_GUI_WIN32

// start dialog procedure

BOOL CALLBACK chaos_start_dlg::start_dlgproc(
	HWND dlg, UINT message, WPARAM wparam, LPARAM lparam)
{
	BOOL result = FALSE;
	switch (message) {
	case WM_INITDIALOG:
		{ // dummy block
			std::vector<char> org_caption(256+1);
			org_caption.back() = '\0';
			GetWindowText(dlg, &org_caption[0], org_caption.size());
			chaos_string new_caption =
				chaos_string(&org_caption[0])+
				chaos_string(" Ver. ")+
			chaos_resource::instance()->get_resource_text(
				chaos_resource::app_version);
			SetWindowText(dlg, new_caption.c_str());
			chaos_string start_text =
				chaos_resource::instance()->get_resource_text(
					chaos_resource::start_text);
			for (
				chaos_string::iterator it = start_text.begin();
				it != start_text.end();
				it++)
			{
				if (*it == '\n') {
					it = start_text.insert(it, '\r');
					ASSERT(*it == '\r');
					it++;
				}
			}
			SetDlgItemText(
				dlg, IDC_START_TEXT,
				start_text.c_str());
			SendDlgItemMessage(
				dlg, IDC_START_TEXT,
				EM_SETSEL,
				0,
				0);
			HWND suzu_static = GetDlgItem(dlg, IDC_SUZU);
			if (suzu_static != NULL) {
				SetWindowLong(
					suzu_static, GWL_STYLE,
					(GetWindowLong(suzu_static, GWL_STYLE) &
						~SS_TYPEMASK) | SS_OWNERDRAW);
			}
			chaos_config_dlg* config_dlg =
				chaos_config_dlg::instance();
			SendDlgItemMessage(
				dlg, IDC_NOT_SHOW_STARTUP,
				BM_SETCHECK,
				!config_dlg->get_bool_profile(
					"show_startup")?
						BST_CHECKED: BST_UNCHECKED,
				0);
			result = FALSE;
		}
		break;
	case WM_COMMAND:
		switch (LOWORD(wparam)) {
		case IDOK:
			{ // dummy block
				chaos_config_dlg* config_dlg =
					chaos_config_dlg::instance();
				bool show_startup =
						(SendDlgItemMessage(
							dlg, IDC_NOT_SHOW_STARTUP,
							BM_GETCHECK,
							0, 0) != BST_CHECKED),
					dirty = config_dlg->sync_bool_profile(
						"show_startup",
						show_startup);
				if (dirty) {
					instance()->set_dirty(true);
				}
			}
			EndDialog(dlg, LOWORD(wparam));
			break;
		case IDCANCEL:
			EndDialog(dlg, LOWORD(wparam));
			break;
		case IDC_CONFIG:
			{ // dummy block
				chaos_config_dlg* config_dlg =
					chaos_config_dlg::instance();
				bool show_startup =
						(SendDlgItemMessage(
							dlg, IDC_NOT_SHOW_STARTUP,
							BM_GETCHECK,
							0, 0) != BST_CHECKED),
					dirty = config_dlg->sync_bool_profile(
						"show_startup",
						show_startup);
				if (dirty) {
					instance()->set_dirty(true);
				}
				if (config_dlg->do_modal(dlg) &&
					config_dlg->is_dirty())
				{
					instance()->set_dirty(true);
					SendDlgItemMessage(
						dlg, IDC_NOT_SHOW_STARTUP,
						BM_SETCHECK,
						!config_dlg->get_bool_profile(
							"show_startup")?
								BST_CHECKED: BST_UNCHECKED,
						0);
				}
			}
			break;
		}
		break;
	case WM_DRAWITEM:
		if (wparam == IDC_SUZU) {
			DRAWITEMSTRUCT* dis = (DRAWITEMSTRUCT*)lparam;
			chaos_image* image = instance()->get_suzu_image();
			if (!image->is_valid()) {
				chaos_dim suzu_dim(
					dis->rcItem.right-dis->rcItem.left,
					dis->rcItem.bottom-dis->rcItem.top);
				instance()->build_suzu_image(suzu_dim);
			}
			if (image->is_valid()) {
				BITMAPINFO bmi;
				memset(&bmi, 0, sizeof(bmi));
				bmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
				bmi.bmiHeader.biWidth = image->get_width();
				bmi.bmiHeader.biHeight = -image->get_height();
				bmi.bmiHeader.biBitCount = 32;
				bmi.bmiHeader.biPlanes = 1;
				bmi.bmiHeader.biCompression = BI_RGB;
				SetDIBitsToDevice(
					dis->hDC,
					0, 0,
					image->get_width(), image->get_height(),
					0, 0,
					0, image->get_height(),
					image->get_bits(),
					&bmi, DIB_RGB_COLORS);
			}
		}
		break;
	default:
		result = FALSE;
		break;
	}
	return result;
}

#elif defined(CHAOS_GUI_GTK)

// suzu expose event

gboolean chaos_start_dlg::suzu_expose_event(
	GtkWidget* suzu_area,
	GdkEventExpose* /*event*/,
	gpointer data)
{
	chaos_start_dlg* dlg = (chaos_start_dlg*)data;
	chaos_dim area_dim(
		suzu_area->allocation.width,
		suzu_area->allocation.height);
	chaos_image* image = dlg->get_suzu_image();
	GdkPixmap* pixmap = dlg->get_suzu_pixmap();
	if (!image->is_valid()) {
		chaos_dim suzu_dim = area_dim;
		dlg->build_suzu_image(suzu_dim);
	}
	GdkGC* gc = gdk_gc_new(suzu_area->window);
	if ((pixmap == NULL) && image->is_valid()) {
		pixmap = gdk_pixmap_new(
			suzu_area->window,
			image->get_width(), image->get_height(), -1);
		if (pixmap != NULL) {
			GdkImage* img = gdk_image_new(
				GDK_IMAGE_NORMAL,
				gdk_window_get_visual(suzu_area->window),
				image->get_width(), image->get_height());
			if (img != NULL) {
				for (int y = 0; y < image->get_height(); y++) {
					for (int x = 0; x < image->get_width(); x++) {
						chaos_uint32 col = image->get_pixel(x, y);
						GdkColor color;
						color.red =
							(gushort)(CHAOS_RGBA_RED(col)*0x0101);
						color.green =
							(gushort)(CHAOS_RGBA_GREEN(col)*0x0101);
						color.blue =
							(gushort)(CHAOS_RGBA_BLUE(col)*0x0101);
						gdk_color_alloc(
							gdk_colormap_get_system(),
							&color);
						gdk_image_put_pixel(
							img,
							x, y,
							color.pixel);
					}
				}
				gdk_draw_image(
					pixmap,
					gc,
					img,
					0, 0,
					0, 0,
					image->get_width(), image->get_height());
				gdk_image_destroy(img);
			}
			dlg->set_suzu_pixmap(pixmap);
		}
	}
	GdkColor white;
	white.red =  white.green =  white.blue = 0xFFFF;
	gdk_color_alloc(gdk_colormap_get_system(), &white);
	gdk_gc_set_foreground(gc, &white);
	gdk_draw_rectangle(
		suzu_area->window,
		gc,
		TRUE,
		0, 0,
		area_dim.width, area_dim.height);
	if ((pixmap != NULL) && image->is_valid()) {
		gdk_draw_pixmap(
			suzu_area->window,
			gc,
			pixmap,
			0, 0,
			0, 0,
			image->get_width(), image->get_height());
	}
	return FALSE;
}

// config button clicked

void chaos_start_dlg::config_clicked(
	GtkWidget* /*config_button*/,
	gpointer data)
{
	chaos_config_dlg* config_dlg =
		chaos_config_dlg::instance();
	GtkWidget* not_show_startup_check =
		((chaos_start_dlg*)data)->get_widget("not_show_startup");
	if (not_show_startup_check != NULL) {
		bool show_startup = (gtk_toggle_button_get_active(
				GTK_TOGGLE_BUTTON(not_show_startup_check)) == FALSE),
			dirty = config_dlg->sync_bool_profile(
				"show_startup",
				show_startup);
		if (dirty) {
			((chaos_start_dlg*)data)->set_dirty(true);
		}
	}
	if (config_dlg->do_modal(NULL) &&
		config_dlg->is_dirty())
	{
		((chaos_start_dlg*)data)->set_dirty(true);
		if (not_show_startup_check != NULL) {
			gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(not_show_startup_check),
				!config_dlg->get_bool_profile("show_startup")?
					TRUE: FALSE);
		}
	}
}

#endif // CHAOS_GUI

////////////////////////////////////////////////////////////////////////////////
// suzu

// build suzu image

bool chaos_start_dlg::build_suzu_image(chaos_dim suzu_dim) {
	if (suzu_dim.empty()) {
		return false;
	}
	if (suzu_image == NULL) {
		return false;
	}
	if (!suzu_image->create(
			suzu_dim,
			true,
			CHAOS_RGBA(255, 255, 255, 0)))
	{
		return false;
	}
	chaos_image org_image;
	{ // dummy block
		chaos_file_data filedata;
		{ // dummy block
			chaos_buffer buffer;
			chaos_resource::instance()->get_suzu_picture(&buffer);
			filedata.write(buffer.data(), buffer.size());
		}
		filedata.seek(0, chaos_file_data::seek_set);
		chaos_png_image_loader loader(
			&filedata,
			&org_image);
		if (!loader.load()) {
			return false;
		}
	}
	if (!org_image.get_dim().empty()) {
		chaos_dim blt_dim(
			 (suzu_dim.height*org_image.get_width())/org_image.get_height(),
			 (suzu_dim.width*org_image.get_height())/org_image.get_width());
		if (blt_dim.width > suzu_dim.width) {
			blt_dim.width = suzu_dim.width;
		} else {
			blt_dim.height = suzu_dim.height;
		}
		chaos_pos blt_pos(
			(suzu_dim.width-blt_dim.width)/2,
			(suzu_dim.height-blt_dim.height)/2);
		suzu_image->smooth_stretch_blt(
			&org_image,
			blt_pos, blt_dim);
	}
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// operation

// modal

bool chaos_start_dlg::do_modal() {
	set_dirty(false);
	suzu_image = new chaos_image;
	bool result;

#ifdef CHAOS_GUI_WIN32

	HINSTANCE process_instance = (HINSTANCE)GetModuleHandle(NULL);
	result = (DialogBox(
		process_instance,
		MAKEINTRESOURCE(IDD_START),
		NULL,
		start_dlgproc) == IDOK);

#elif defined(CHAOS_GUI_GTK)

	suzu_pixmap = NULL;
	widgets.clear();
	dialog = gtk_dialog_new();
	gtk_widget_ref(dialog);
	build_widgets(
		GTK_DIALOG(dialog)->action_area,
		chaos_resource::instance()->get_resource_text(
			chaos_resource::dlg_std_action_area));
	build_widgets(
		GTK_DIALOG(dialog)->vbox,
		chaos_resource::instance()->get_resource_text(
			chaos_resource::dlg_start));
	build_widgets(
		GTK_DIALOG(dialog)->action_area,
		chaos_resource::instance()->get_resource_text(
			chaos_resource::dlg_start_action_area));
	init_dialog_default(
		chaos_resource::instance()->get_resource_text(
			chaos_resource::app_name)+
		chaos_string(" Ver. ")+
		chaos_resource::instance()->get_resource_text(
			chaos_resource::app_version));
	initialize_widgets();
	gtk_widget_show(dialog);
	gtk_grab_add(dialog);
	gtk_window_present(GTK_WINDOW(dialog));
	push_current_modaling(dialog);
	gtk_main();
	pop_current_modaling();
	result = dialog_result;
	destroy_widgets();
	gtk_grab_remove(dialog);
	gtk_widget_unref(dialog);
	dialog = NULL;
	if (suzu_pixmap != NULL) {
		gdk_pixmap_unref(suzu_pixmap);
		suzu_pixmap = NULL;
	}

#endif // CHAOS_GUI

	delete suzu_image;
	suzu_image = NULL;
	return result;
}

#ifdef CHAOS_GUI_GTK

// initialize widgets

void chaos_start_dlg::initialize_widgets() {
	GtkWidget* start_text = get_widget("start_text");
	if (start_text != NULL) {
		chaos_string text =
			chaos_utility::sjis_to_gui(
				chaos_resource::instance()->get_resource_text(
					chaos_resource::start_text));
#if !GTK_CHECK_VERSION(2, 0, 0)
		int position = 0;
		gtk_editable_insert_text(
			GTK_EDITABLE(start_text),
			text.c_str(),
			text.length(),
			&position);
#else // GTK_CHECK_VERSION(2, 0, 0)
		GtkTextBuffer* text_buffer =
			gtk_text_view_get_buffer(
				GTK_TEXT_VIEW(start_text));
		if (text_buffer != NULL) {
			gtk_text_buffer_set_text(
				text_buffer,
				text.c_str(),
				text.length());
		}
#endif // GTK_CHECK_VERSION
	}
	GtkWidget* suzu_area = get_widget("suzu");
	if (suzu_area != NULL) {
		gtk_signal_connect(
			GTK_OBJECT(suzu_area), "expose_event",
			GTK_SIGNAL_FUNC(suzu_expose_event), this);
	}
	GtkWidget* config_button = get_widget("config");
	if (config_button != NULL) {
		gtk_signal_connect(
			GTK_OBJECT(config_button), "clicked",
			GTK_SIGNAL_FUNC(config_clicked), this);
	}
	GtkWidget* not_show_startup_check = get_widget("not_show_startup");
	if (not_show_startup_check != NULL) {
		chaos_config_dlg* config_dlg =
			chaos_config_dlg::instance();
		gtk_toggle_button_set_active(
			GTK_TOGGLE_BUTTON(not_show_startup_check),
			!config_dlg->get_bool_profile("show_startup")?
				TRUE: FALSE);
	}
}

// on ok clicked

void chaos_start_dlg::on_ok_clicked() {
	GtkWidget* not_show_startup_check = get_widget("not_show_startup");
	if (not_show_startup_check != NULL) {
		chaos_config_dlg* config_dlg =
			chaos_config_dlg::instance();
		bool show_startup = (gtk_toggle_button_get_active(
				GTK_TOGGLE_BUTTON(not_show_startup_check)) == FALSE),
			dirty = config_dlg->sync_bool_profile(
				"show_startup",
				show_startup);
		if (dirty) {
			set_dirty(true);
		}
	}
}

#endif // CHAOS_GUI
