////////////////////////////////////////////////////////////////////////////////
// chaos_start_dlg.h : start dialog

#ifndef CHAOS_START_DLG_H_INCLUDED
#define CHAOS_START_DLG_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// declare

class chaos_start_dlg;

////////////////////////////////////////////////////////////////////////////////
// include

#ifndef CHAOS_IMAGE_H_INCLUDED
#include "chaos_image.h"
#endif // CHAOS_IMAGE_H_INCLUDED

#ifndef CHAOS_DLG_H_INCLUDED
#include "chaos_dlg.h"
#endif // CHAOS_DLG_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// chaos_start_dlg: start dialog class

class chaos_start_dlg :
	public chaos_dlg
{
// attribute
private:
	// dirty
	bool dirty;
	// suzu image
	chaos_image* suzu_image;

#ifdef CHAOS_GUI_GTK

	// suzu pixmap
	GdkPixmap* suzu_pixmap;

#endif // CHAOS_GUI

// create & destroy
private:
	// default constructor
	chaos_start_dlg();

public:
	// destructor
	~chaos_start_dlg();

	// get singleton instance
	static chaos_start_dlg* instance();

// get & set
public:
	// is dirty
	bool is_dirty() const {
		return dirty;
	}
	// set dirty
	void set_dirty(bool new_dirty) {
		dirty = new_dirty;
	}
	// get suzu image
	const chaos_image* get_suzu_image() const {
		return suzu_image;
	}
	chaos_image* get_suzu_image() {
		return suzu_image;
	}

#ifdef CHAOS_GUI_GTK

	// get suzu pixmap
	const GdkPixmap* get_suzu_pixmap() const {
		return suzu_pixmap;
	}
	GdkPixmap* get_suzu_pixmap() {
		return suzu_pixmap;
	}
	// set suzu pixmap
	void set_suzu_pixmap(GdkPixmap* new_suzu_pixmap) {
		suzu_pixmap = new_suzu_pixmap;
	}

#endif // CHAOS_GUI

// dialog
private:

#ifdef CHAOS_GUI_WIN32

	// start dialog procedure
	static BOOL CALLBACK start_dlgproc(
		HWND dlg, UINT message, WPARAM wparam, LPARAM lparam);

#elif defined(CHAOS_GUI_GTK)

	// suzu expose event
	static gboolean suzu_expose_event(
		GtkWidget* suzu_area,
		GdkEventExpose* event,
		gpointer data);
	// config button clicked
	static void config_clicked(
		GtkWidget* config_button,
		gpointer data);

#endif // CHAOS_GUI

// suzu
private:
	// build suzu image
	bool build_suzu_image(chaos_dim suzu_dim);

// operation
public:
	// modal
	bool do_modal();

#ifdef CHAOS_GUI_GTK

	// initialize widgets
	void initialize_widgets();
	// on ok clicked
	virtual void on_ok_clicked();

#endif // CHAOS_GUI

};

#endif // CHAOS_START_DLG_H_INCLUDED
