////////////////////////////////////////////////////////////////////////////////
// chaos_string.cpp : string

////////////////////////////////////////////////////////////////////////////////
// include

#include "stdafx.h"

#include "chaos_string.h"

////////////////////////////////////////////////////////////////////////////////
// chaos_string : string class

////////////////////////////////////////////////////////////////////////////////
// attribute

// nul string

const chaos_string::value_type chaos_string::null_str[1] = "";

// maximum position

const chaos_string::size_type chaos_string::npos = UINT_MAX;

////////////////////////////////////////////////////////////////////////////////
// operator

////////////////////////////////////////////////////////////////////////////////
// buffer

// allocate data buffer

void chaos_string::alloc_data(size_type new_capacity) {
	if ((data_ptr == NULL) ||
		(data_ptr->ref_count > 1) ||
		(data_ptr->capacity < new_capacity))
	{
		size_type new_capacity2 = new_capacity;
		if (new_capacity2 < (size_t)capacity_min) {
			new_capacity2 = (size_t)capacity_min;
		}
		bool reallocate = ((data_ptr != NULL) &&
				(data_ptr->ref_count == 1));
		size_type old_size = (data_ptr == NULL)?
				0:
				sizeof(chaos_string_data)+data_ptr->capacity,
			new_size = sizeof(chaos_string_data)+new_capacity2;
		chaos_string_data* new_data_ptr;
		if (reallocate) {
			ASSERT(data_ptr->ref_count == 1);
			new_data_ptr = (chaos_string_data*)realloc(data_ptr, new_size);
			if (new_data_ptr != NULL) {
				data_ptr = NULL;
			}
		} else {
			new_data_ptr = (chaos_string_data*)malloc(new_size);
		}
		if (new_data_ptr == NULL) {
			throw std::bad_alloc();
		}
		if (data_ptr != NULL) {
			memcpy(
				new_data_ptr, data_ptr,
				(old_size < new_size)? old_size: new_size);
		}
		if (new_size > old_size) {
			memset((char*)new_data_ptr+old_size, 0, new_size-old_size);
		}
		new_data_ptr->ref_count = 1;
		new_data_ptr->capacity = new_capacity2;
		free_data();
		data_ptr = new_data_ptr;
	}
}

// free data buffer

void chaos_string::free_data() {
	if (data_ptr != NULL) {
		if (--data_ptr->ref_count <= 0) {
			ASSERT(data_ptr->ref_count == 0);
			free(data_ptr);
		}
		data_ptr = NULL;
	}
}

////////////////////////////////////////////////////////////////////////////////
// operation

// erase

chaos_string& chaos_string::erase(size_type pos, size_type count) {
	size_type first = pos;
	if (first > length()) {
		first = length();
	}
	size_type dist = count;
	if (dist > length()-first) {
		dist = length()-first;
	}
	size_type last = first+dist;
	alloc_data(length());
	ASSERT(data_ptr != NULL);
	ASSERT(data_ptr->ref_count == 1);
	if (dist > 0) {
		if (last < data_ptr->length) {
			memmove(
				data_ptr->string+first,
				data_ptr->string+last,
				data_ptr->length-last);
		}
		memset(data_ptr->string+data_ptr->length-dist, 0, dist);
		data_ptr->length -= dist;
	}
	return *this;
}

chaos_string::iterator chaos_string::erase(iterator first, iterator last) {
	ASSERT(first <= last);
	ASSERT(first >= ((const chaos_string&)*this).begin());
	ASSERT(last <= ((const chaos_string&)*this).end());
	size_type pos = std::distance(
		((const chaos_string&)*this).begin(),
		(const_iterator)first);
	size_type count = std::distance(first, last);
	erase(pos, count);
	return begin()+pos;
}

chaos_string::iterator chaos_string::erase(iterator target) {
	return erase(target, target+1);
}

// insert

chaos_string& chaos_string::insert(size_type pos, const chaos_string& org) {
	if (&org == this) {
		chaos_string tmp = org;
		insert(pos, tmp);
	} else {
		alloc_data(length()+org.length());
		ASSERT(data_ptr != NULL);
		if (org.length() > 0) {
			if (pos < data_ptr->length) {
				memmove(
					data_ptr->string+pos+org.length(),
					data_ptr->string+pos,
					data_ptr->length-pos);
			}
			strncpy(
				data_ptr->string+pos, org.data_ptr->string,
				org.length());
			data_ptr->length += org.length();
			data_ptr->string[data_ptr->length] = '\0';
		}
	}
	return *this;
}

void chaos_string::insert(
	iterator target,
	size_type fill,
	const value_type& value)
{
	ASSERT(target >= ((const chaos_string&)*this).begin());
	ASSERT(target <= ((const chaos_string&)*this).end());
	if (fill > 0) {
		size_type pos = std::distance(
			((const chaos_string&)*this).begin(),
			(const_iterator)target);
		chaos_string tmp(fill, value);
		insert(pos, tmp);
	}
}

chaos_string::iterator chaos_string::insert(
	iterator target,
	const value_type& value)
{
	ASSERT(target >= ((const chaos_string&)*this).begin());
	ASSERT(target <= ((const chaos_string&)*this).end());
	size_type pos = std::distance(
		((const chaos_string&)*this).begin(),
		(const_iterator)target);
	chaos_string tmp(value);
	insert(pos, tmp);
	return begin()+pos;
}

// get substring

chaos_string chaos_string::substr(size_type pos, size_type count) const {
	size_type first = pos;
	if (first > length()) {
		first = length();
	}
	size_type dist = count;
	if (dist > length()-first) {
		dist = length()-first;
	}
	size_type last = first+dist;
	return chaos_string(begin()+first, begin()+last);
}

// push back

void chaos_string::push_back(const value_type& value) {
	insert(end(), value);
}

// pop back

void chaos_string::pop_back() {
	erase(end()-1);
}

// compare string

int chaos_string::compare(const chaos_string& org) const {
	return strcmp(c_str(), org.c_str());
}

////////////////////////////////////////////////////////////////////////////////
// operator

// copy

chaos_string& chaos_string::operator=(const chaos_string& org) {
	if (org.data_ptr != data_ptr) {
		free_data();
		data_ptr = org.data_ptr;
		if (data_ptr != NULL) {
			data_ptr->ref_count++;
		}
	}
	return *this;
}

// append

chaos_string& chaos_string::operator+=(const chaos_string& org) {
	if (org.length() == 0) {
		// nothing
	} else if (length() == 0) {
		*this = org;
	} else if (&org == this) {
		chaos_string tmp = org;
		*this += tmp;
	} else {
		ASSERT(data_ptr != NULL);
		ASSERT(org.data_ptr != NULL);
		size_type new_length = length()+org.length();
		alloc_data(
			new_length+
				((data_ptr->capacity < new_length)?
					(size_t)capacity_step: 0));
		ASSERT(data_ptr != NULL);
		ASSERT(data_ptr->capacity >= new_length);
		ASSERT(data_ptr->ref_count == 1);
		strcat(
			data_ptr->string,
			org.data_ptr->string);
		data_ptr->length = new_length;
	}
	return *this;
}

chaos_string& chaos_string::operator+=(const value_type& org) {
	size_type new_length = length()+1;
	alloc_data(
		new_length+
			((capacity() < new_length)?
				(size_t)capacity_step: 0));
	ASSERT(data_ptr != NULL);
	ASSERT(data_ptr->capacity >= data_ptr->length+1);
	ASSERT(data_ptr->ref_count == 1);
	data_ptr->string[data_ptr->length++] = org;
	data_ptr->string[data_ptr->length] = '\0';
	return *this;
}

// add

chaos_string chaos_string::operator+(const chaos_string& org) const {
	chaos_string tmp = *this;
	tmp += org;
	return tmp;
}
