////////////////////////////////////////////////////////////////////////////////
// chaos_system.cpp : system

////////////////////////////////////////////////////////////////////////////////
// include

#include "stdafx.h"

#include "chaos_system.h"
#include "chaos_file_serializer.h"
#include "chaos_image_realizer.h"
#include "chaos_cd_player.h"
#include "chaos_wave_player.h"
#include "chaos_utility.h"

////////////////////////////////////////////////////////////////////////////////
// chaos_system : system class

////////////////////////////////////////////////////////////////////////////////
// create & destroy

// default constructor

chaos_system::chaos_system() :
	variables(NULL),
	properties(NULL),
	properties_ln(NULL),
	sysvariables(NULL),
	sysproperties(NULL),
	sysproperties_ln(NULL),
	fixes(NULL),
	messages(NULL),
	cur_message_key(-1),
	readflags(NULL),
	dirty_profile(false),
	dirty_readflag(false),
	dirty_image(false),
	dirty_message(false),
	skip_mode(skip_none),
	history_requested(false),
	requested_jump_mode(jmp_none),
	requested_jump_id(-1),
	requested_jump_line(1),
	requested_jump_column(1)
{
	variables = new chaos_value_manager<int>;
	properties = new chaos_value_manager<int>;
	properties_ln = new chaos_value_manager<chaos_string>;
	sysvariables = new chaos_value_manager<int>;
	sysproperties = new chaos_value_manager<int>;
	sysproperties_ln = new chaos_value_manager<chaos_string>;
	fixes = new chaos_value_manager<chaos_string>;
	messages = new chaos_message_manager;
	readflags = new chaos_readflag_manager;

#define ADD_VAR_NAME(n) { \
	var_id2name.insert( \
		std::map<int, chaos_string>::value_type((int)n, #n)); \
	var_name2id.insert( \
		std::map<chaos_string, int>::value_type(#n, (int)n)); \
}

	// property
	ADD_VAR_NAME(system_mode);
	ADD_VAR_NAME(wait_time);
	ADD_VAR_NAME(enable_load);
	ADD_VAR_NAME(enable_save);
	ADD_VAR_NAME(cur_message_filename);
	ADD_VAR_NAME(message_frame_count);
	ADD_VAR_NAME(message_text0);
	ADD_VAR_NAME(message_text1);
	ADD_VAR_NAME(message_info0);
	ADD_VAR_NAME(message_info1);
	ADD_VAR_NAME(message_speed0);
	ADD_VAR_NAME(message_speed1);
	ADD_VAR_NAME(reserved_message_text);
	ADD_VAR_NAME(next_message_text);
	ADD_VAR_NAME(message_subcolor);
	ADD_VAR_NAME(message_shape);
	ADD_VAR_NAME(message_clear);
	ADD_VAR_NAME(prompt_info);
	ADD_VAR_NAME(prompt_visible);
	ADD_VAR_NAME(sel_defcolor);
	ADD_VAR_NAME(sel_count);
	ADD_VAR_NAME(sel_current);
	ADD_VAR_NAME(sel_preparation);
	ADD_VAR_NAME(sel_frame_count);
	ADD_VAR_NAME(sel_info0);
	ADD_VAR_NAME(sel_info1);
	ADD_VAR_NAME(sel_info2);
	ADD_VAR_NAME(sel_info3);
	ADD_VAR_NAME(sel_info4);
	ADD_VAR_NAME(sel_info5);
	ADD_VAR_NAME(sel_info6);
	ADD_VAR_NAME(sel_info7);
	ADD_VAR_NAME(sel_info8);
	ADD_VAR_NAME(sel_info9);
	ADD_VAR_NAME(image_info0);
	ADD_VAR_NAME(image_info1);
	ADD_VAR_NAME(image_info2);
	ADD_VAR_NAME(image_info3);
	ADD_VAR_NAME(image_info4);
	ADD_VAR_NAME(image_info5);
	ADD_VAR_NAME(image_info6);
	ADD_VAR_NAME(image_info7);
	ADD_VAR_NAME(image_info8);
	ADD_VAR_NAME(image_info9);
	ADD_VAR_NAME(image_info10);
	ADD_VAR_NAME(image_info11);
	ADD_VAR_NAME(image_info12);
	ADD_VAR_NAME(image_info13);
	ADD_VAR_NAME(image_info14);
	ADD_VAR_NAME(image_info15);
	ADD_VAR_NAME(image_info16);
	ADD_VAR_NAME(image_info17);
	ADD_VAR_NAME(image_info18);
	ADD_VAR_NAME(image_info19);
	ADD_VAR_NAME(image_info20);
	ADD_VAR_NAME(cd_audio_info);
	ADD_VAR_NAME(wave_audio_info);
	ADD_VAR_NAME(stack_count);
	ADD_VAR_NAME(stack_info0);
	ADD_VAR_NAME(stack_info1);
	ADD_VAR_NAME(stack_info2);
	ADD_VAR_NAME(stack_info3);
	ADD_VAR_NAME(stack_info4);
	ADD_VAR_NAME(stack_info5);
	ADD_VAR_NAME(stack_info6);
	ADD_VAR_NAME(stack_info7);
	ADD_VAR_NAME(stack_info8);
	ADD_VAR_NAME(stack_info9);
	ADD_VAR_NAME(stack_info10);
	ADD_VAR_NAME(stack_info11);
	ADD_VAR_NAME(stack_info12);
	ADD_VAR_NAME(stack_info13);
	ADD_VAR_NAME(stack_info14);
	ADD_VAR_NAME(stack_info15);
	ADD_VAR_NAME(stack_info16);
	ADD_VAR_NAME(stack_info17);
	ADD_VAR_NAME(stack_info18);
	ADD_VAR_NAME(stack_info19);
	// system property
	ADD_VAR_NAME(start_script);
	ADD_VAR_NAME(start_label);
	ADD_VAR_NAME(sys_width);
	ADD_VAR_NAME(sys_height);
	ADD_VAR_NAME(sys_font_name);
	ADD_VAR_NAME(sys_font_xlfd);
	ADD_VAR_NAME(sys_font_pango);
	ADD_VAR_NAME(sys_small_font_size);
	ADD_VAR_NAME(sys_medium_font_size);
	ADD_VAR_NAME(sys_large_font_size);
	ADD_VAR_NAME(sys_font_bold);
	ADD_VAR_NAME(sys_font_anti_alias);
	ADD_VAR_NAME(sys_image_buffer_count);
	ADD_VAR_NAME(effect_step);
	ADD_VAR_NAME(effect_cut);
	ADD_VAR_NAME(meswin_info_count);
	ADD_VAR_NAME(meswin_info0);
	ADD_VAR_NAME(meswin_info1);
	ADD_VAR_NAME(meswin_info2);
	ADD_VAR_NAME(meswin_info3);
	ADD_VAR_NAME(meswin_info4);
	ADD_VAR_NAME(meswin_info5);
	ADD_VAR_NAME(meswin_info6);
	ADD_VAR_NAME(meswin_info7);
	ADD_VAR_NAME(meswin_info8);
	ADD_VAR_NAME(meswin_info9);
	ADD_VAR_NAME(log_mode);
	ADD_VAR_NAME(log_level);
	ADD_VAR_NAME(full_screen);
	ADD_VAR_NAME(window_center);
	ADD_VAR_NAME(font_name);
	ADD_VAR_NAME(font_xlfd);
	ADD_VAR_NAME(font_pango);
	ADD_VAR_NAME(font_size);
	ADD_VAR_NAME(font_bold);
	ADD_VAR_NAME(font_anti_alias);
	ADD_VAR_NAME(text_speed);
	ADD_VAR_NAME(skip_speed);
	ADD_VAR_NAME(show_startup)
	ADD_VAR_NAME(cd_info);
	ADD_VAR_NAME(cd_drive);
	ADD_VAR_NAME(bgm_sound_type);
	ADD_VAR_NAME(bgm_sound_volume);
	ADD_VAR_NAME(effect_sound_volume);

#undef ADD_VAR_NAME

#define ADD_VAR_NAME_IDX(n) { \
	var_name2id_indexed.insert( \
		std::map<chaos_string, int>::value_type(#n, (int)(n##0))); \
}

	// property(indexed)
	ADD_VAR_NAME_IDX(message_text);
	ADD_VAR_NAME_IDX(message_info);
	ADD_VAR_NAME_IDX(message_speed);
	ADD_VAR_NAME_IDX(sel_info);
	ADD_VAR_NAME_IDX(image_info);
	ADD_VAR_NAME_IDX(stack_info);
	// system property(indexed)
	ADD_VAR_NAME_IDX(meswin_info);

#undef ADD_VAR_NAME_IDX

}

// destructor

chaos_system::~chaos_system() {
	clear();
	delete variables;
	delete properties;
	delete properties_ln;
	delete sysvariables;
	delete sysproperties;
	delete sysproperties_ln;
	delete fixes;
	delete messages;
	delete readflags;
}

// clear

void chaos_system::clear() {
	variables->clear();
	properties->clear();
	properties_ln->clear();
	sysvariables->clear();
	sysproperties->clear();
	sysproperties_ln->clear();
	fixes->clear();
	messages->clear();
	readflags->clear();
	cur_message_key = -1;
	set_dirty_profile(false);
	set_dirty_readflag(false);
	set_dirty_image(false);
	set_dirty_message(false);
	set_skip_mode(skip_none);
	set_history_requested(false);
	request_jump(jmp_none, "", -1, 1, 1, "");
	while (histories.size() > 0) {
		delete histories.back();
		histories.pop_back();
	}
	while (prop_histories.size() > 0) {
		delete prop_histories.back();
		prop_histories.pop_back();
	}
	while (interpreters.size() > 0) {
		delete interpreters.back();
		interpreters.pop_back();
	}
}

// reset

bool chaos_system::reset() {
	variables->merge(sysvariables);
	variables->clear_nul();
	properties->clear_nul();
	properties_ln->clear_nul();
	reflect_stack_info();
	set_dirty_image(true);
	set_dirty_message(true);
	set_skip_mode(skip_none);
	set_history_requested(false);
	request_jump(jmp_none, "", -1, 1, 1, "");
	play_cd_audio(get_cd_audio_info());
	play_wave_audio(get_wave_audio_info());
	return true;
}

// restart

bool chaos_system::restart() {
	variables->clear();
	properties->clear();
	properties_ln->clear();
	clear_histories();
	gosub_script(get_start_script_filename(), -1, 1, 1, "");
	reset();
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// variable

// get variable id

int chaos_system::get_variable_id(const chaos_string& variable_name) const {
	int variable_id = (int)invalid_variable;
	const char* str = variable_name.c_str();
	if (str[0] == '@') {
		if (chaos_utility::is_digit(str[1])) {
			char* endptr = NULL;
			variable_id = (int)chaos_system::num_variable_base+
					(int)strtol(str+1, &endptr, 10);
			ASSERT(*endptr == '\0');
			ASSERT(
				(variable_id >= (int)chaos_system::num_variable_base+1) &&
				(variable_id <= (int)chaos_system::num_variable_base+9999));
		} else if (chaos_utility::is_alpha(str[1])) {
			ASSERT(str[2] == '\0');
			variable_id = (int)chaos_system::str_variable_base+
				+(toupper(str[1])-'A'+1);
			ASSERT(
				(variable_id >= (int)chaos_system::str_variable_base+1) &&
				(variable_id <= (int)chaos_system::str_variable_base+26));
		} else {
			ASSERT(false);
		}
	} else {
		std::map<chaos_string, int>::const_iterator it =
			var_name2id.find(variable_name);
		if (it != var_name2id.end()) {
			variable_id = (*it).second;
		} else {
			const char* first = variable_name.begin();
			const char* last = first;
			char ch;
			while (
				((ch = *last) != '\0') &&
				!chaos_utility::is_digit(ch))
			{
				last++;
			}
			int id_base = (int)invalid_variable;
			int ofs_max = -1, ofs = 0;
			if ((first != last) && (ch != '\0')) {
				it = var_name2id_indexed.find(
					chaos_string(first, last));
				if (it != var_name2id_indexed.end()) {
					id_base = (*it).second;
					switch (id_base) {
					case (int)message_text0:
						ofs_max = (int)frame_max;
						break;
					case (int)message_info0:
						ofs_max = (int)frame_max;
						break;
					case (int)message_speed0:
						ofs_max = (int)frame_max;
						break;
					case (int)sel_info0:
						ofs_max = (int)selection_max;
						break;
					case (int)image_info0:
						ofs_max = (int)image_max;
						break;
					case (int)stack_info0:
						ofs_max = (int)stack_max;
						break;
					case (int)meswin_info0:
						ofs_max = (int)meswin_max;
						break;
					}
					ofs = atoi(last);
				}
			}
			if ((id_base != (int)invalid_variable) &&
				(ofs < ofs_max))
			{
				variable_id = id_base+ofs;
			} else {
				// lost number or invalid name
			}
		}
	}
	return variable_id;
}

// get variable name

chaos_string chaos_system::get_variable_name(int variable_id) const {
	chaos_string variable_name;
	if ((variable_id >= (int)num_variable_base+1) &&
		(variable_id <= (int)num_variable_base+9999))
	{
		variable_name = chaos_utility::format_str(
			"@%d", variable_id);
	} else if (
		(variable_id >= (int)str_variable_base+1) &&
		(variable_id <= (int)str_variable_base+26))
	{
		variable_name = chaos_utility::format_str(
			"@%c", 'A'+variable_id-((int)str_variable_base+1));
	} else {
		std::map<int, chaos_string>::const_iterator it =
			var_id2name.find(variable_id);
		if (it != var_id2name.end()) {
			variable_name = (*it).second;
		} else if (
			(variable_id >= (int)message_text0) &&
			(variable_id < (int)message_text0+(int)frame_max))
		{
			variable_name = chaos_utility::format_str(
				"message_text%d",
				variable_id-(int)message_text0);
		} else if (
			(variable_id >= (int)message_info0) &&
			(variable_id < (int)message_info0+(int)frame_max))
		{
			variable_name = chaos_utility::format_str(
				"message_info%d",
				variable_id-(int)message_info0);
		} else if (
			(variable_id >= (int)message_speed0) &&
			(variable_id < (int)message_speed0+(int)frame_max))
		{
			variable_name = chaos_utility::format_str(
				"message_speed%d",
				variable_id-(int)message_speed0);
		} else if (
			(variable_id >= (int)sel_info0) &&
			(variable_id < (int)sel_info0+(int)selection_max))
		{
			variable_name = chaos_utility::format_str(
				"sel_info%d",
				variable_id-(int)sel_info0);
		} else if (
			(variable_id >= (int)image_info0) &&
			(variable_id < (int)image_info0+(int)image_max))
		{
			variable_name = chaos_utility::format_str(
				"image_info%d",
				variable_id-(int)image_info0);
		} else if (
			(variable_id >= (int)stack_info0) &&
			(variable_id < (int)stack_info0+(int)stack_max))
		{
			variable_name = chaos_utility::format_str(
				"stack_info%d",
				variable_id-(int)stack_info0);
		} else if (
			(variable_id >= (int)meswin_info0) &&
			(variable_id < (int)meswin_info0+(int)meswin_max))
		{
			variable_name = chaos_utility::format_str(
				"meswin_info%d",
				variable_id-(int)meswin_info0);
		} else {
			ASSERT(false);
		}
	}
	return variable_name;
}

// set variable value

void chaos_system::set_variable(int id, const chaos_value& value) {
	ASSERT(id < (int)property_base);
	if (histories.size() > 0) {
		chaos_value_manager<int>* history = histories.front();
		if (!history->exist_value(id)) {
			history->set_value(id, variables->get_value(id));
		}
	}
	variables->set_value(id, value);
}

// clear variable

void chaos_system::clear_variable(int id) {
	ASSERT(id < (int)property_base);
	if (histories.size() > 0) {
		chaos_value_manager<int>* history = histories.front();
		if (!history->exist_value(id)) {
			history->set_value(id, variables->get_value(id));
		}
	}
	variables->clear_value(id);
}

// get variable value(priority system variable)

const chaos_value& chaos_system::get_variable_ex(int id) const {
	ASSERT(id < (int)property_base);
	return (sysvariables->exist_value(id))?
		sysvariables->get_value(id):
		get_variable(id);
}

// set variable value(priority system variable)

void chaos_system::set_variable_ex(int id, const chaos_value& value) {
	ASSERT(id < (int)property_base);
	if (sysvariables->exist_value(id)) {
		sysvariables->set_value(id, value);
		dirty_profile = true;
	}
	set_variable(id,value);
}

////////////////////////////////////////////////////////////////////////////////
// property

// set property value

void chaos_system::set_property(var_id_type id, const chaos_value& value) {
	ASSERT((id >= property_base) && (id < sysproperty_base));
	if (prop_histories.size() > 0) {
		chaos_value_manager<int>* history = prop_histories.front();
		if (!history->exist_value((int)id)) {
			history->set_value(
				(int)id, properties->get_value((int)id));
		}
	}
	properties->set_value((int)id, value);
}

// clear property value

void chaos_system::clear_property(var_id_type id) {
	ASSERT((id >= property_base) && (id < sysproperty_base));
	if (prop_histories.size() > 0) {
		chaos_value_manager<int>* history = prop_histories.front();
		if (!history->exist_value((int)id)) {
			history->set_value(
				(int)id, properties->get_value((int)id));
		}
	}
	properties->clear_value((int)id);
}

////////////////////////////////////////////////////////////////////////////////
// history

// clear histories

void chaos_system::clear_histories() {
	ASSERT(histories.size() == prop_histories.size());
	while (histories.size() > 0) {
		delete histories.back();
		histories.pop_back();
	}
	while (prop_histories.size() > 0) {
		delete prop_histories.back();
		prop_histories.pop_back();
	}
	ASSERT(histories.size() == prop_histories.size());
}

// add new history

void chaos_system::add_new_history() {
	ASSERT(histories.size() == prop_histories.size());
	store_stack_info();
	histories.push_front(new chaos_value_manager<int>());
	while (histories.size() > (size_t)history_max) {
		delete histories.back();
		histories.pop_back();
	}
	prop_histories.push_front(new chaos_value_manager<int>());
	while (prop_histories.size() > (size_t)history_max) {
		delete prop_histories.back();
		prop_histories.pop_back();
	}
	ASSERT(histories.size() == prop_histories.size());
}

// rewind history

void chaos_system::rewind_history(int step) {
	ASSERT(histories.size() == prop_histories.size());
	for (int i = 0; i <= step; i++) {
		if ((histories.size() <= 0) ||
			(prop_histories.size() <= 0))
		{
			break;
		}
		variables->merge(histories.front());
		delete histories.front();
		histories.pop_front();
		properties->merge(prop_histories.front());
		delete prop_histories.front();
		prop_histories.pop_front();
	}
	histories.push_front(new chaos_value_manager<int>());
	prop_histories.push_front(new chaos_value_manager<int>());
	ASSERT(histories.size() == prop_histories.size());
	reset();
}

// get history message text

chaos_string chaos_system::get_history_message_text(int historyno) const {
	ASSERT(histories.size() == prop_histories.size());
	chaos_string text;
	if ((historyno >= 0) &&
		(historyno < (int)prop_histories.size()))
	{
		std::list<chaos_value_manager<int>*>::const_iterator it =
			prop_histories.begin();
		std::advance(it, historyno);
		text = (*it)->get_value((int)next_message_text).get_string();
	}
	return text;
}

////////////////////////////////////////////////////////////////////////////////
// snapshot

// load snapshot

bool chaos_system::load_snapshot(const chaos_xml* snapshot) {
	if (snapshot->get_kind() != "snapshot") {
		return false;
	}
	if (snapshot->get_values_version() != "1.0") {
		// nothing
	}
	const chaos_value_manager<chaos_string>* snap_values =
		snapshot->get_values();
	variables->clear();
	properties->clear();
	properties_ln->clear();
	chaos_value_manager<chaos_string>::const_iterator it;
	for (it = snap_values->begin(); it != snap_values->end(); it++) {
		int id = get_variable_id((*it).first);
		if (id == (int)invalid_variable) {
			properties_ln->set_value((*it).first, (*it).second);
		} else if (id < (int)property_base) {
			variables->set_value(id, (*it).second);
		} else {
			properties->set_value(id, (*it).second);
		}
	}
	clear_histories();
	histories.push_front(new chaos_value_manager<int>());
	prop_histories.push_front(new chaos_value_manager<int>());
	ASSERT(histories.size() == prop_histories.size());
	reset();
	return true;
}

// save snapshot

bool chaos_system::save_snapshot(chaos_xml* snapshot) const {
	snapshot->clear();
	snapshot->set_kind("snapshot");
	snapshot->set_values_version("1.0");
	chaos_value_manager<chaos_string>* snap_values =
		snapshot->get_values();
	chaos_value_manager<int>::const_iterator it;
	for (it = variables->begin(); it != variables->end(); it++) {
		snap_values->set_value(
			get_variable_name((*it).first),
			(*it).second);
	}
	if (histories.size() > 0) {
		chaos_value_manager<int>* history = histories.front();
		for (it = history->begin(); it != history->end(); it++) {
			snap_values->set_value(
				get_variable_name((*it).first),
				(*it).second);
		}
	}
	for (it = properties->begin(); it != properties->end(); it++) {
		snap_values->set_value(
			get_variable_name((*it).first),
			(*it).second);
	}
	if (prop_histories.size() > 0) {
		chaos_value_manager<int>* history = prop_histories.front();
		for (it = history->begin(); it != history->end(); it++) {
			snap_values->set_value(
				get_variable_name((*it).first),
				(*it).second);
		}
	}
	chaos_value_manager<chaos_string>::const_iterator it2;
	for (
		it2 = properties_ln->begin();
		it2 != properties_ln->end();
		it2++)
		{
		snap_values->set_value(
			(*it2).first,
			(*it2).second);
	}
	snap_values->clear_nul();
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// profile

// load profile

bool chaos_system::load_profile(const chaos_xml* profile) {
	if (profile->get_kind() != "profile") {
		return false;
	}
	if (profile->get_values_version() != "1.0") {
		// nothing
	}
	const chaos_value_manager<chaos_string>* prof_values =
		profile->get_values();
	sysvariables->clear();
	sysproperties->clear();
	sysproperties_ln->clear();
	chaos_value_manager<chaos_string>::const_iterator it;
	for (it = prof_values->begin(); it != prof_values->end(); it++) {
		int id = get_variable_id((*it).first);
		if (id == (int)invalid_variable) {
			sysproperties_ln->set_value((*it).first, (*it).second);
		} else if (id < (int)property_base) {
			sysvariables->set_value(id, (*it).second);
		} else {
			sysproperties->set_value(id, (*it).second);
		}
	}
	return true;
}

// save profile

bool chaos_system::save_profile(chaos_xml* profile) const {
	profile->clear();
	profile->set_kind("profile");
	profile->set_values_version("1.0");
	chaos_value_manager<chaos_string>* prof_values =
		profile->get_values();
	chaos_value_manager<int>::const_iterator it;
	for (it = sysvariables->begin(); it != sysvariables->end(); it++) {
		prof_values->set_value(
			get_variable_name((*it).first),
			(*it).second);
	}
	for (it = sysproperties->begin(); it != sysproperties->end(); it++) {
		prof_values->set_value(
			get_variable_name((*it).first),
			(*it).second);
	}
	chaos_value_manager<chaos_string>::const_iterator it2;
	for (
		it2 = sysproperties_ln->begin();
		it2 != sysproperties_ln->end();
		it2++)
		{
		prof_values->set_value(
			(*it2).first,
			(*it2).second);
	}
	prof_values->clear_nul();
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// readflag

// load readflag

bool chaos_system::load_readflag(const chaos_xml* readf_xml) {
	return readflags->load(readf_xml);
}

// save readflag

bool chaos_system::save_readflag(chaos_xml* readf_xml) const {
	return readflags->save(readf_xml);
}

// is current message read

bool chaos_system::is_current_message_read() const {
	bool read = false;
	if (interpreters.size() > 0) {
		int id = interpreters.back()->get_script_id();
		if (id >= 0) {
			const chaos_string& filename =
				interpreters.back()->get_script_filename();
			int key;
			if (filename == messages->get_filename()) {
				key = cur_message_key;
			} else {
				key = readflags->get_key(filename);
			}
			read = readflags->is_read(key, id);
		}
	}
	return read;
}

////////////////////////////////////////////////////////////////////////////////
// system mode

// change system mode

void chaos_system::change_system_mode(system_mode_type new_mode) {
	system_mode_type old_mode = get_system_mode();
	set_system_mode(new_mode);
	switch (old_mode) {
	case sysmode_userprompt:
		set_wait_time(0);
		if (is_prompt_visible()) {
			set_prompt_visible(false);
			set_dirty_image(true);
		}
		break;
	case sysmode_selmessage:
	case sysmode_selevent:
		{ // dummy block
			int selno = get_current_selection();
			chaos_string info = get_selection_info(selno);
			int types,
				x1, y1, x2, y2,
				unk1, unk2, unk3;
			chaos_color color;
			chaos_string filename, label;
			decode_selection_info(
				info,
				types,
				x1, y1, x2, y2,
				unk1, unk2, unk3,
				color,
				filename, label);
			clear_selection();
			goto_script(filename, -1, 1, 1, label);
			set_wait_time(0);
		}
		break;
	case sysmode_effect:
		{ // dummy block
			chaos_string info1 = get_image_info(1);
			set_image_info(0, info1);
			set_dirty_image(true);
			set_wait_time(0);
		}
		break;
	}
	switch (new_mode) {
	case sysmode_message:
		set_dirty_message(true);
		break;
	case sysmode_selmessage:
	case sysmode_selevent:
		set_current_selection(
			(new_mode == sysmode_selmessage)?
				(int)select_invalid:
				-1);
		break;
	case sysmode_userprompt:
		set_prompt_visible(true);
		set_dirty_image(true);
		set_dirty_message(true);
		break;
	}
}

////////////////////////////////////////////////////////////////////////////////
// skip

// is enable skip mode

bool chaos_system::is_enable_skip_mode(skip_mode_type check_mode) const {
	bool enable = false;
	switch (check_mode) {
	case skip_none:
		enable = true;
		break;
	case skip_cur_message:
		switch (get_system_mode()) {
		case sysmode_message:
			enable = true;
			break;
		}
		break;
	case skip_read:
		switch (get_system_mode()) {
		case sysmode_selmessage:
		case sysmode_selevent:
			break;
		default:
			if (is_current_message_read()) {
				enable = true;
			}
			break;
		}
		break;
	case skip_force:
		switch (get_system_mode()) {
		case sysmode_selmessage:
		case sysmode_selevent:
			break;
		default:
			enable = true;
			break;
		}
		break;
	}
	return enable;
}

// change skip mode

void chaos_system::change_skip_mode(skip_mode_type new_mode) {
	skip_mode_type old_mode = get_skip_mode();
	set_skip_mode(new_mode);
	if ((new_mode == old_mode) ||
		!is_enable_skip_mode(new_mode))
	{
		return;
	}
	switch (old_mode) {
	case skip_none:
		break;
	default:
		switch (get_skip_speed()) {
		case sspeed_jump:
			play_cd_audio(get_cd_audio_info());
			play_wave_audio(get_wave_audio_info());
			break;
		}
		break;
	}
	switch (new_mode) {
	case skip_none:
		set_dirty_image(true);
		set_dirty_message(true);
		break;
	default:
		switch (get_system_mode()) {
		case sysmode_message:
			{ // dummy block
				chaos_string reserved_text = get_reserved_message_text();
				if (!reserved_text.empty()) {
					clear_reserved_message_text();
					add_message(reserved_text, false);
				}
				change_system_mode(sysmode_interpret);
			}
			break;
		case sysmode_userprompt:
		case sysmode_userclick:
			change_system_mode(sysmode_interpret);
			break;
		case sysmode_wait:
		case sysmode_effect:
			switch (get_skip_speed()) {
			case sspeed_fast:
			case sspeed_jump:
				change_system_mode(sysmode_interpret);
				break;
			}
			break;
		}
		break;
	}
}

////////////////////////////////////////////////////////////////////////////////
// message

// load current messages

bool chaos_system::load_cur_messages(
	const chaos_string& message_filename)
{
	chaos_string message_real_filename =
		chaos_file_serializer::get_pure_filename(message_filename);
	set_str_property(cur_message_filename, message_real_filename);
	if (message_real_filename != messages->get_filename()) {
		chaos_file_data filedata;
		if (!chaos_file_serializer::instance()->load_file_data(
				&filedata,
				message_real_filename,
				chaos_file_serializer::fpos_packed))
		{
			return false;
		}
		if (!messages->load(&filedata)) {
			return false;
		}
		messages->set_filename(message_real_filename);
		cur_message_key = readflags->regist(message_real_filename);
		readflags->reserve(cur_message_key, messages->get_max_id()+1);
	}
	return true;
}

// add message frame

void chaos_system::add_message_frame() {
	int frame_count = get_message_frame_count();
	if (frame_count < 0) {
		frame_count = 0;
	}
	frame_count++;
	set_message_frame_count(frame_count);
	if (is_selection_preparation()) {
		chaos_string text = get_message_text(frame_count-1);
		if (!text.empty()) {
			text += '\n';
			set_message_text(frame_count-1, text);
		}
	}
}

// add message

void chaos_system::add_message(
	const chaos_string& add_text, bool wait)
{
	int frame_count = get_message_frame_count();
	if (frame_count <= 0) {
		return;
	}
	int wait_time = 0;
	chaos_string old_text = get_message_text(frame_count-1),
		new_text = old_text+add_text;
	if (!is_selection_preparation()) {
		if (wait && (get_skip_mode() == skip_none)) {
			int speed = get_message_speed(frame_count-1);
			if (speed > 0) {
				switch (get_text_speed()) {
				case tspeed_superslow:
					wait_time = speed*8;
					break;
				case tspeed_slow:
					wait_time = speed*4;
					break;
				case tspeed_normal:
					wait_time = speed*2;
					break;
				case tspeed_fast:
					wait_time = speed;
					break;
				case tspeed_superfast:
					wait_time = speed/2;
					break;
				case tspeed_instant:
					break;
				default:
					wait_time = speed*2;
					break;
				}
			}
		}
	} else {
		int sel_count = get_selection_count();
		chaos_string info = get_selection_info(sel_count);
		int types,
			x1, y1, x2, y2,
			unk1, unk2, unk3;
		chaos_color color;
		if (decode_selection_info(
				info,
				types,
				x1, y1, x2, y2,
				unk1, unk2, unk3,
				color))
		{
			if (unk1 != frame_count-1) {
				new_text = old_text;
			}
		}
	}
	if (wait_time > 0) {
		change_system_mode(sysmode_message);
		set_wait_time(wait_time);
		ASSERT(new_text.length() >= old_text.length());
		const char* str = new_text.c_str();
		size_t length = old_text.length();
		if (chaos_utility::is_sjislead(str[length])) {
			length++;
		}
		if (str[length] != '\0') {
			length++;
		}
		if (length < new_text.length()) {
			set_reserved_message_text(new_text.substr(length));
			new_text = new_text.substr(0, length);
		}
	}
	set_message_text(frame_count-1, new_text);
}

// clear message

void chaos_system::clear_message() {
	int count = get_message_frame_count();
	if (!is_selection_preparation()) {
		if ((prop_histories.size() > 0) &&
			!prop_histories.front()->exist_value((int)next_message_text))
		{
			LOG_WRN(
				"warning : system::clear message - "
				"not exist message wait\n");
			copy_next_message_text();
		}
		bool dirty = true;
		if (get_skip_mode() != skip_none) {
			switch (get_skip_speed()) {
			case sspeed_fast:
			case sspeed_jump:
				dirty = false;
				break;
			}
		}
		if (dirty) {
			set_dirty_message(true);
		}
		for (int frame = 0; frame < count; frame++) {
			clear_property((var_id_type)(message_text0+frame));
		}
	} else {
		int sel_count = get_selection_count();
		set_selection_count(sel_count+1);
		int sel_frame_count = get_selection_frame_count();
		if (count > sel_frame_count) {
			set_selection_frame_count(count);
		}
	}
	clear_property(message_frame_count);
	clear_property(message_subcolor);
}

// wait message

void chaos_system::wait_message() {
	if (!is_selection_preparation()) {
		copy_next_message_text();
		bool prompt = true, dirty = true;
		switch (get_skip_mode()) {
		case skip_cur_message:
			set_skip_mode(skip_none);
			break;
		case skip_read:
		case skip_force:
			prompt = false;
			switch (get_skip_speed()) {
			case sspeed_jump:
				dirty = false;
				break;
			}
			break;
		}
		if (prompt) {
			change_system_mode(sysmode_userprompt);
			set_wait_time((int)prompt_blink_time);
		} else {
			if (dirty) {
				set_dirty_message(true);
			}
		}
	}
}

// copy next message text

void chaos_system::copy_next_message_text() {
	if (prop_histories.size() > 0) {
		chaos_value_manager<int>* history = prop_histories.front();
		chaos_string next_text;
		int count = get_message_frame_count();
		for (int frame = 0; frame < count; frame++) {
			if (!next_text.empty()) {
				next_text += " : ";
			}
			chaos_string message_text = get_message_text(frame);
			const char* first = message_text.c_str();
			const char* last = first;
			bool sjislead = false, color = false;
			char ch;
			do {
				ch = *last;
				bool flush = false, skip = false;
				const char* special = NULL;
				if (ch == '\0') {
					flush = true;
				} else if (sjislead) {
					sjislead = false;
				} else if (color) {
					skip = true;
					if (ch == ')') {
						color = false;
					}
				} else {
					if (chaos_utility::is_sjislead(ch)) {
						sjislead = true;
					} else if (ch == '\\') {
						color = true;
						flush = true;
						skip = true;
					} else if (ch == '\n') {
						special = " / ";
						flush = true;
						skip = true;
					}
				}
				if (flush) {
					next_text += chaos_string(first, last);
					first = last;
				}
				last++;
				if (skip) {
					first = last;
				}
				if ((special != NULL) && !next_text.empty()) {
					next_text += special;
				}
			} while (ch != '\0');
		}
		history->set_value(
			(int)next_message_text,
			chaos_value(next_text));
	}
}

// flush next message text

void chaos_system::flush_next_message_text() {
	switch (get_system_mode()) {
	case sysmode_message:
		copy_next_message_text();
		break;
	}
}

// encode message info

chaos_string chaos_system::encode_message_info(
	int x, int y,
	int length,
	chaos_color color)
{
	chaos_string info;
	info = chaos_utility::format_str(
		"%d,%d,%d,RGB(%d,%d,%d)",
		x, y,
		length,
		color.get_red(), color.get_green(), color.get_blue());
	return info;
}

// decode message info

bool chaos_system::decode_message_info(
	const chaos_string& info,
	int& x, int& y,
	int& length,
	chaos_color& color)
{
	if (info.empty()) {
		x = 0;
		y = 0;
		length = 0;
		color = chaos_color(255, 255, 255);
		return true;
	}
	const char* infostr = info.c_str();
	int red, green, blue;
	if (sscanf(
			infostr,
			"%d,%d,%d,RGB(%d,%d,%d)",
			&x, &y,
			&length,
			&red, &green, &blue) != 6)
	{
		x = 0;
		y = 0;
		length = 0;
		color = chaos_color(255, 255, 255);
		return false;
	}
	color = chaos_color(red, green, blue);
	return true;
}

// encode prompt info

chaos_string chaos_system::encode_prompt_info(
	bool place,
	int x, int y)
{
	chaos_string info;
	info = chaos_utility::format_str(
		"%d,%d,%d",
		place? 1: 0,
		x, y);
	return info;
}

// decode prompt info

bool chaos_system::decode_prompt_info(
	const chaos_string& info,
	bool& place,
	int& x, int& y)
{
	if (info.empty()) {
		place = false;
		x = y = 0;
		return true;
	}
	const char* infostr = info.c_str();
	int place2 = 0;
	if (sscanf(
			infostr,
			"%d,%d,%d",
			&place2,
			&x, &y) != 3)
	{
		return false;
	}
	place = (place2 != 0);
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// selection

// start selection preparation

void chaos_system::start_selection_preparation() {
	set_selection_preparation(true);
	set_history_requested(true);
}

// finish selection preparation

void chaos_system::finish_selection_preparation() {
	set_selection_preparation(false);
	set_dirty_image(true);
	set_dirty_message(true);
	if (get_selection_count() > 0) {
		int count = get_selection_frame_count();
		clear_property(sel_frame_count);
		set_message_frame_count(count);
		copy_next_message_text();
		change_skip_mode(skip_none);
		change_system_mode(sysmode_selmessage);
	} else {
		clear_selection();
	}
}

// clear selection

void chaos_system::clear_selection() {
	int count = get_selection_count();
	for (int i = 0; i <= count; i++) {
		clear_property((var_id_type)(sel_info0+i));
	}
	clear_property(sel_preparation);
	clear_property(sel_count);
	clear_property(sel_current);
	clear_property(sel_frame_count);
	clear_message();
	set_dirty_image(true);
}

// encode selection info

chaos_string chaos_system::encode_selection_info(
	int types,
	int x1, int y1, int x2, int y2,
	int unk1, int unk2, int unk3,
	chaos_color color,
	const chaos_string& filename,
	const chaos_string& label)
{
	chaos_string info;
	info = chaos_utility::format_str(
		"%d,%d,%d,%d,%d,%d,%d,%d,RGB(%d,%d,%d),%s,%s",
		types,
		x1, y1, x2, y2,
		unk1, unk2, unk3,
		color.get_red(), color.get_green(), color.get_blue(),
		chaos_utility::str_to_str(
			chaos_file_serializer::get_pure_filename(filename)).c_str(),
		chaos_utility::str_to_str(
			chaos_utility::to_lower_str(label)).c_str());
	return info;
}

// decode selection info

bool chaos_system::decode_selection_info(
	const chaos_string& info,
	int& types,
	int& x1, int& y1, int& x2, int& y2,
	int& unk1, int& unk2, int& unk3,
	chaos_color& color,
	chaos_string& filename,
	chaos_string& label)
{
	const char* infostr = info.c_str();
	int next = 0,
		red, green, blue;
	if (sscanf(
			infostr,
			"%d,%d,%d,%d,%d,%d,%d,%d,RGB(%d,%d,%d),%n",
			&types,
			&x1, &y1, &x2, &y2,
			&unk1, &unk2, &unk3,
			&red, &green, &blue,
			&next) != 11)
	{
		return false;
	}
	color = chaos_color(red, green, blue);
	infostr += next;
	filename = chaos_utility::str_from_str(infostr);
	if (*infostr != ',') {
		return false;
	}
	infostr++;
	label = chaos_utility::str_from_str(infostr);
	return true;
}

// decode selection info(omit string info)

bool chaos_system::decode_selection_info(
	const chaos_string& info,
	int& types,
	int& x1, int& y1, int& x2, int& y2,
	int& unk1, int& unk2, int& unk3,
	chaos_color& color)
{
	const char* infostr = info.c_str();
	int red, green, blue;
	if (sscanf(
			infostr,
			"%d,%d,%d,%d,%d,%d,%d,%d,RGB(%d,%d,%d)",
			&types,
			&x1, &y1, &x2, &y2,
			&unk1, &unk2, &unk3,
			&red, &green, &blue) != 11)
	{
		return false;
	}
	color = chaos_color(red, green, blue);
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// cd audio

// play cd audio

bool chaos_system::play_cd_audio(const chaos_string& info) {
	int track1 = -1, track2 = -1,
		repeat = 0;
	if (!decode_cd_audio_info(
			info,
			track1, track2,
			repeat))
	{
		track1 = -1;
		track2 = -1;
		repeat = 0;
	}
	chaos_cd_player* cd_player = chaos_cd_player::instance();
	bool result = true;
	if (track1 > 0) {
		if (repeat) {
			set_cd_audio_info(info);
		} else {
			clear_cd_audio_info();
		}
		if ((get_skip_mode() == skip_none) ||
			(get_skip_speed() != sspeed_jump))
		{
			result = cd_player->play(
				track1, track2,
				repeat != 0);
		}
	} else {
		clear_cd_audio_info();
		if ((get_skip_mode() == skip_none) ||
			(get_skip_speed() != sspeed_jump))
		{
			result = cd_player->stop();
		}
	}
	return result;
}

// encode cd audio info

chaos_string chaos_system::encode_cd_audio_info(
	int track1, int track2,
	int repeat)
{
	chaos_string info;
	if (track1 > 0) {
		info = chaos_utility::format_str(
			"%d,%d,%d",
			track1, track2,
			repeat);
	}
	return info;
}

// decode cd audio info

bool chaos_system::decode_cd_audio_info(
	const chaos_string& info,
	int& track1, int& track2,
	int& repeat)
{
	if (info.empty()) {
		track1 = -1;
		track2 = -1;
		repeat = 0;
		return true;
	}
	const char* infostr = info.c_str();
	if (sscanf(
			infostr,
			"%d,%d,%d",
			&track1, &track2,
			&repeat) != 3)
	{
		return false;
	}
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// wave audio

// play wave audio

bool chaos_system::play_wave_audio(const chaos_string& info) {
	int unk = 0, repeat = 0;
	chaos_string filename;
	if (!decode_wave_audio_info(
			info,
			unk, repeat,
			filename))
	{
		unk = 0;
		repeat = 0;
		filename = "";
	}
	chaos_wave_player* wave_player = chaos_wave_player::instance();
	bool result = true;
	if (!filename.empty()) {
		if (repeat) {
			set_wave_audio_info(info);
		} else {
			clear_wave_audio_info();
		}
		if ((get_skip_mode() == skip_none) ||
			(get_skip_speed() != sspeed_jump))
		{
			result = wave_player->play(
				filename,
				repeat != 0);
		}
	} else {
		clear_wave_audio_info();
		if ((get_skip_mode() == skip_none) ||
			(get_skip_speed() != sspeed_jump))
		{
			result = wave_player->stop();
		}
	}
	return result;
}

// encode wave audio info

chaos_string chaos_system::encode_wave_audio_info(
	int unk, int repeat,
	const chaos_string& filename)
{
	chaos_string info;
	info = chaos_utility::format_str(
		"%d,%d,%s",
		unk, repeat,
		chaos_utility::str_to_str(
			chaos_file_serializer::get_pure_filename(filename)).c_str());
	return info;
}

// decode wave audio info

bool chaos_system::decode_wave_audio_info(
	const chaos_string& info,
	int& unk, int& repeat,
	chaos_string& filename)
{
	if (info.empty()) {
		unk = 0;
		repeat = 0;
		filename = "";
		return true;
	}
	const char* infostr = info.c_str();
	int next = 0;
	if (sscanf(
			infostr,
			"%d,%d,%n",
			&unk, &repeat,
			&next) != 2)
	{
		return false;
	}
	infostr += next;
	filename = chaos_utility::str_from_str(infostr);
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// stack

// store stack info

void chaos_system::store_stack_info() {
	int count = (int)interpreters.size();
	set_stack_count(count);
	if (count > 0) {
		chaos_string filename = interpreters.back()->get_script_filename();
		int id = interpreters.back()->get_script_id(),
			line = interpreters.back()->get_script_line(),
			column = interpreters.back()->get_script_column();
		chaos_string info = encode_stack_info(
			filename,
			id,
			line, column);
		set_stack_info(count-1, info);
	}
}

// reflect stack info

void chaos_system::reflect_stack_info() {
	while (interpreters.size() > 0) {
		delete interpreters.back();
		interpreters.pop_back();
	}
	int count = get_stack_count();
	for (int stackno = 0; stackno < count; stackno++) {
		chaos_string info = get_stack_info(stackno);
		chaos_string filename;
		int id,
			line, column;
		if (decode_stack_info(
				info,
				filename,
				id,
				line, column))
		{
			chaos_interpreter* routine =
				new chaos_interpreter(
					this,
					filename,
					id,
					line, column);
			interpreters.push_back(routine);
		}
	}
}

// encode stack info

chaos_string chaos_system::encode_stack_info(
	const chaos_string& filename,
	int id,
	int line, int column)
{
	chaos_string info;
	info = chaos_utility::format_str(
		"%s,%d,%d,%d",
		chaos_utility::str_to_str(
			chaos_file_serializer::get_pure_filename(filename)).c_str(),
		id,
		line, column);
	return info;
}

// decode stack info

bool chaos_system::decode_stack_info(
	const chaos_string& info,
	chaos_string& filename,
	int& id,
	int& line, int& column)
{
	const char* infostr = info.c_str();
	filename = chaos_utility::str_from_str(infostr);
	if (*infostr != ',') {
		return false;
	}
	infostr++;
	if (sscanf(
			infostr,
			"%d,%d,%d",
			&id,
			&line, &column) != 3)
	{
		return false;
	}
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// script

// load script

bool chaos_system::load_script(
	chaos_file_data* script,
	const chaos_string& script_filename,
	int script_id)
{
	bool result;
	if (script_id < 0) {
		result = chaos_file_serializer::instance()->load_file_data(
			script,
			script_filename,
			chaos_file_serializer::fpos_packed);
	} else {
		result = load_cur_messages(script_filename);
		if (result) {
			result = script->load_from_string(
				messages->get_message(script_id));
		}
	}
	return result;
}

// goto script

bool chaos_system::goto_script(
	const chaos_string& script_filename,
	int script_id,
	int script_line, int script_column,
	const chaos_string& script_label)
{
	int count = (int)interpreters.size();
	if (count <= 0) {
		return gosub_script(
			script_filename,
			script_id,
			script_line, script_column,
			script_label);
	}
	interpreters.back()->goto_script(
		script_filename,
		script_id,
		script_line, script_column);
	if (!script_label.empty()) {
		interpreters.back()->jump_to_label(script_label);
	}
	store_stack_info();
	return true;
}

// gosub script

bool chaos_system::gosub_script(
	const chaos_string& script_filename,
	int script_id,
	int script_line, int script_column,
	const chaos_string& script_label)
{
	store_stack_info();
	int count = (int)interpreters.size();
	if (count >= (int)stack_max) {
		return false;
	}
	chaos_interpreter* subroutine =
		new chaos_interpreter(
			this,
			script_filename,
			script_id,
			script_line, script_column);
	interpreters.push_back(subroutine);
	if (!script_label.empty()) {
		subroutine->jump_to_label(script_label);
	}
	ASSERT(count+1 == (int)interpreters.size());
	store_stack_info();
	if (script_id >= 0) {
		if (get_message_clear_mode() != 0) {
			clear_message();
		}
		add_message_frame();
		if (!is_selection_preparation()) {
			chaos_string info = get_prompt_info();
			bool place;
			int x, y;
			if (decode_prompt_info(
					info,
					place,
					x, y) &&
				place)
			{
				set_history_requested(true);
			}
		}
		switch (get_skip_mode()) {
		case skip_read:
			if (!is_current_message_read()) {
				change_skip_mode(skip_none);
			}
			break;
		}
	}
	return true;
}

// return script

bool chaos_system::return_script() {
	int count = (int)interpreters.size();
	if (count <= 0) {
		return false;
	}
	chaos_string script_filename =
		interpreters.back()->get_script_filename();
	int script_id = interpreters.back()->get_script_id();
	delete interpreters.back();
	interpreters.pop_back();
	count--;
	ASSERT(count == (int)interpreters.size());
	ASSERT(count >= 0);
	set_stack_count(count);
	clear_stack_info(count);
	if (script_id >= 0) {
		if (get_message_clear_mode() == 0) {
			clear_message();
		}
		int key;
		if (script_filename == messages->get_filename()) {
			key = cur_message_key;
		} else {
			key = readflags->regist(script_filename);
		}
		if (!readflags->set_read(key, script_id, true)) {
			dirty_readflag = true;
		}
	}
	return true;
}

// request jump

void chaos_system::request_jump(
	jump_mode_type  new_jump_mode,
	const chaos_string& new_jump_filename,
	int new_jump_id,
	int new_jump_line, int new_jump_column,
	const chaos_string& new_jump_label)
{
	requested_jump_mode = new_jump_mode;
	requested_jump_filename =
		chaos_file_serializer::get_pure_filename(new_jump_filename);
	requested_jump_id = new_jump_id;
	requested_jump_line = new_jump_line;
	requested_jump_column = new_jump_column;
	requested_jump_label =
		chaos_utility::to_lower_str(new_jump_label);
}

////////////////////////////////////////////////////////////////////////////////
// operation

// execute chaos system

bool chaos_system::execute(int pass_time) {
	int wait_time = get_wait_time()-pass_time;
	if (wait_time < 0) {
		wait_time = 0;
	}
	set_wait_time(wait_time);
	int exec;
	bool work = false;
	switch (get_system_mode()) {
	case sysmode_interpret:
		for (exec = pass_time*10; exec > 0; exec--) {
			if (interpreters.size() > 0) {
				work = true;
				interpreters.back()->interpret();
				if (requested_jump_mode != jmp_none) {
					switch (requested_jump_mode) {
					case jmp_goto:
						goto_script(
							requested_jump_filename,
							requested_jump_id,
							requested_jump_line,
							requested_jump_column,
							requested_jump_label);
						break;
					case jmp_gosub:
						gosub_script(
							requested_jump_filename,
							requested_jump_id,
							requested_jump_line,
							requested_jump_column,
							requested_jump_label);
						break;
					case jmp_return:
						return_script();
						break;
					}
					request_jump(jmp_none, "", -1, 1, 1, "");
				}
				if (is_history_requested()) {
					set_history_requested(false);
					add_new_history();
				}
				if ((get_system_mode() != sysmode_interpret) ||
					is_dirty_image() ||
					is_dirty_message())
				{
					break;
				}
			} else {
				break;
			}
		}
		break;
	case sysmode_wait:
		if (wait_time <= 0) {
			change_system_mode(sysmode_interpret);
		}
		break;
	case sysmode_message:
		if (wait_time <= 0) {
			chaos_string reserved_text = get_reserved_message_text();
			if (!reserved_text.empty()) {
				clear_reserved_message_text();
				add_message(reserved_text, true);
			} else {
				change_system_mode(sysmode_interpret);
			}
		}
		break;
	case sysmode_effect:
		work = true;
		if (wait_time <= 0) {
			int effect_step = get_effect_step();
			if (effect_step < 1) {
				effect_step = 1;
			}
			chaos_string info =
				chaos_image_realizer::get_next_step_effect_info(
					get_image_info(0),
					effect_step);
			if (!info.empty()) {
				set_image_info(0, info);
				set_wait_time(
					(int)effect_unit_time*
						effect_step*
						chaos_image_realizer::get_effect_time_scale(
							info));
				set_dirty_image(true);
			} else {
				change_system_mode(sysmode_interpret);
			}
		}
		break;
	case sysmode_userprompt:
		if (wait_time <= 0) {
			set_wait_time((int)prompt_blink_time);
			set_prompt_visible(!is_prompt_visible());
			set_dirty_image(true);
		}
		break;
	case sysmode_userclick:
	case sysmode_selmessage:
	case sysmode_selevent:
		break;
	default:
		change_system_mode(sysmode_interpret);
		break;
	}
	return work;
}
