////////////////////////////////////////////////////////////////////////////////
// chaos_system.h : system

#ifndef CHAOS_SYSTEM_H_INCLUDED
#define CHAOS_SYSTEM_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// declare

class chaos_system;

////////////////////////////////////////////////////////////////////////////////
// include

#ifndef CHAOS_COLOR_H_INCLUDED
#include "chaos_color.h"
#endif // CHAOS_COLOR_H_INCLUDED

#ifndef CHAOS_STRING_H_INCLUDED
#include "chaos_string.h"
#endif // CHAOS_STRING_H_INCLUDED

#ifndef CHAOS_FILE_DATA_H_INCLUDED
#include "chaos_file_data.h"
#endif // CHAOS_FILE_DATA_H_INCLUDED

#ifndef CHAOS_VALUE_H_INCLUDED
#include "chaos_value.h"
#endif // CHAOS_VALUE_H_INCLUDED

#ifndef CHAOS_VALUE_MANAGER_H_INCLUDED
#include "chaos_value_manager.h"
#endif // CHAOS_VALUE_MANAGER_H_INCLUDED

#ifndef CHAOS_READFLAG_MANAGER_H_INCLUDED
#include "chaos_readflag_manager.h"
#endif // CHAOS_READFLAG_MANAGER_H_INCLUDED

#ifndef CHAOS_XML_H_INCLUDED
#include "chaos_xml.h"
#endif // CHAOS_XML_H_INCLUDED

#ifndef CHAOS_MESSAGE_MANAGER_H_INCLUDED
#include "chaos_message_manager.h"
#endif // CHAOS_MESSAGE_MANAGER_H_INCLUDED

#ifndef CHAOS_INTERPRETER_H_INCLUDED
#include "chaos_interpreter.h"
#endif // CHAOS_INTERPRETER_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// chaos_system : system class

class chaos_system {
// enum
public:
	// limit
	enum {
		frame_max		= 100,
		selection_max	= 100,
		image_max		= 100,
		stack_max		= 1000,
		history_max		= 100,
		meswin_max		= 100
	};
	// select
	enum {
		select_invalid	= -9999
	};
	// time
	enum {
		effect_unit_time  = 50,
		prompt_blink_time = 500
	};
	// jump mode type
	enum jump_mode_type {
		jmp_none,
		jmp_goto,
		jmp_gosub,
		jmp_return
	};
	// variable / property id type
	enum var_id_type {
		invalid_variable		= 0,
		num_variable_base		= 0,
		str_variable_base		= 10000,
		property_base			= 20000,
		sysproperty_base		= 30000,
		system_mode				= property_base+0,
		wait_time				= property_base+1,
		enable_load				= property_base+10,
		enable_save				= property_base+11,
		cur_message_filename	= property_base+1000,
		message_frame_count		= property_base+1100,
		message_text0			= property_base+1200,
		message_text1			= property_base+1201,
		message_info0			= property_base+1300,
		message_info1			= property_base+1301,
		message_speed0			= property_base+1400,
		message_speed1			= property_base+1401,
		reserved_message_text	= property_base+1500,
		next_message_text		= property_base+1510,
		message_subcolor		= property_base+1520,
		message_shape			= property_base+1521,
		message_clear			= property_base+1522,
		prompt_info				= property_base+1600,
		prompt_visible			= property_base+1601,
		sel_defcolor			= property_base+2000,
		sel_count				= property_base+2010,
		sel_current				= property_base+2011,
		sel_preparation			= property_base+2020,
		sel_frame_count			= property_base+2030,
		sel_info0				= property_base+2100,
		sel_info1				= property_base+2101,
		sel_info2				= property_base+2102,
		sel_info3				= property_base+2103,
		sel_info4				= property_base+2104,
		sel_info5				= property_base+2105,
		sel_info6				= property_base+2106,
		sel_info7				= property_base+2107,
		sel_info8				= property_base+2108,
		sel_info9				= property_base+2109,
		image_info0				= property_base+3000,
		image_info1				= property_base+3001,
		image_info2				= property_base+3002,
		image_info3				= property_base+3003,
		image_info4				= property_base+3004,
		image_info5				= property_base+3005,
		image_info6				= property_base+3006,
		image_info7				= property_base+3007,
		image_info8				= property_base+3008,
		image_info9				= property_base+3009,
		image_info10			= property_base+3010,
		image_info11			= property_base+3011,
		image_info12			= property_base+3012,
		image_info13			= property_base+3013,
		image_info14			= property_base+3014,
		image_info15			= property_base+3015,
		image_info16			= property_base+3016,
		image_info17			= property_base+3017,
		image_info18			= property_base+3018,
		image_info19			= property_base+3019,
		image_info20			= property_base+3020,
		cd_audio_info			= property_base+4000,
		wave_audio_info			= property_base+4100,
		stack_count				= property_base+5000,
		stack_info0				= property_base+6000,
		stack_info1				= property_base+6001,
		stack_info2				= property_base+6002,
		stack_info3				= property_base+6003,
		stack_info4				= property_base+6004,
		stack_info5				= property_base+6005,
		stack_info6				= property_base+6006,
		stack_info7				= property_base+6007,
		stack_info8				= property_base+6008,
		stack_info9				= property_base+6009,
		stack_info10			= property_base+6010,
		stack_info11			= property_base+6011,
		stack_info12			= property_base+6012,
		stack_info13			= property_base+6013,
		stack_info14			= property_base+6014,
		stack_info15			= property_base+6015,
		stack_info16			= property_base+6016,
		stack_info17			= property_base+6017,
		stack_info18			= property_base+6018,
		stack_info19			= property_base+6019,
		start_script			= sysproperty_base+0,
		start_label				= sysproperty_base+1,
		sys_width				= sysproperty_base+100,
		sys_height				= sysproperty_base+101,
		sys_font_name			= sysproperty_base+200,
		sys_font_xlfd			= sysproperty_base+201,
		sys_font_pango			= sysproperty_base+202,
		sys_small_font_size		= sysproperty_base+210,
		sys_medium_font_size	= sysproperty_base+211,
		sys_large_font_size		= sysproperty_base+212,
		sys_font_bold			= sysproperty_base+220,
		sys_font_anti_alias		= sysproperty_base+230,
		sys_image_buffer_count	= sysproperty_base+300,
		effect_step				= sysproperty_base+310,
		effect_cut				= sysproperty_base+311,
		meswin_info_count		= sysproperty_base+1000,
		meswin_info0			= sysproperty_base+1100,
		meswin_info1			= sysproperty_base+1101,
		meswin_info2			= sysproperty_base+1102,
		meswin_info3			= sysproperty_base+1103,
		meswin_info4			= sysproperty_base+1104,
		meswin_info5			= sysproperty_base+1105,
		meswin_info6			= sysproperty_base+1106,
		meswin_info7			= sysproperty_base+1107,
		meswin_info8			= sysproperty_base+1108,
		meswin_info9			= sysproperty_base+1109,
		log_mode				= sysproperty_base+2000,
		log_level				= sysproperty_base+2001,
		full_screen				= sysproperty_base+2010,
		window_center			= sysproperty_base+2011,
		font_name				= sysproperty_base+2100,
		font_xlfd				= sysproperty_base+2101,
		font_pango				= sysproperty_base+2102,
		font_size				= sysproperty_base+2110,
		font_bold				= sysproperty_base+2120,
		font_anti_alias			= sysproperty_base+2130,
		text_speed				= sysproperty_base+2200,
		skip_speed				= sysproperty_base+2201,
		show_startup			= sysproperty_base+2300,
		cd_info					= sysproperty_base+3000,
		cd_drive				= sysproperty_base+3010,
		bgm_sound_type			= sysproperty_base+3020,
		bgm_sound_volume		= sysproperty_base+3021,
		effect_sound_volume		= sysproperty_base+3022
	};
	// system mode type
	enum system_mode_type {
		sysmode_interpret	= 0,
		sysmode_wait		= 1,
		sysmode_message		= 2,
		sysmode_effect		= 3,
		sysmode_userprompt	= 4,
		sysmode_userclick	= 5,
		sysmode_selmessage	= 6,
		sysmode_selevent	= 7,
		sysmode_breakapp	= 100,
		sysmode_load		= 101
	};
	// log mode type
	enum log_mode_type {
		logmode_file	= 1,
		logmode_window	= 2
	};
	// log level
	enum log_level_type {
		loglv_none		= 0,
		loglv_error		= 1,
		loglv_warning	= 2,
		loglv_report	= 3
	};
	// text speed type
	enum text_speed_type {
		tspeed_superslow	= 1,
		tspeed_slow			= 2,
		tspeed_normal		= 3,
		tspeed_fast			= 4,
		tspeed_superfast	= 5,
		tspeed_instant		= 6
	};
	// skip speed type
	enum skip_speed_type {
		sspeed_normal		= 1,
		sspeed_fast			= 2,
		sspeed_jump			= 3
	};
	// skip mode type
	enum skip_mode_type {
		skip_none			= 0,
		skip_cur_message	= 1,
		skip_read			= 2,
		skip_force			= 3
	};

// attribute
private:
	// variables
	chaos_value_manager<int>* variables;
	// properties
	chaos_value_manager<int>* properties;
	// properties(lost number)
	chaos_value_manager<chaos_string>* properties_ln;
	// system variables
	chaos_value_manager<int>* sysvariables;
	// system properties
	chaos_value_manager<int>* sysproperties;
	// system properties(lost number)
	chaos_value_manager<chaos_string>* sysproperties_ln;
	// histories
	std::list<chaos_value_manager<int>*> histories;
	// property histories
	std::list<chaos_value_manager<int>*> prop_histories;
	// fixes
	chaos_value_manager<chaos_string>* fixes;
	// messages
	chaos_message_manager* messages;
	// current message key
	int cur_message_key;
	// script interpreter list
	std::list<chaos_interpreter*> interpreters;
	// readflags
	chaos_readflag_manager* readflags;
	// dirty profile(profile is updated)
	bool dirty_profile;
	// dirty readflag(readflag is updated)
	bool dirty_readflag;
	// dirty image(image is updated)
	bool dirty_image;
	// dirty message(message is updated)
	bool dirty_message;
	// skip mode
	skip_mode_type skip_mode;
	// history requested
	bool history_requested;
	// requested jump mode
	jump_mode_type requested_jump_mode;
	// requested jump filename
	chaos_string requested_jump_filename;
	// requested jump id
	int requested_jump_id;
	// requested jump line
	int requested_jump_line;
	// requested jump column
	int requested_jump_column;
	// requested jump label
	chaos_string requested_jump_label;
	// variable id to name map
	std::map<int, chaos_string> var_id2name;
	// variable name to id map
	std::map<chaos_string, int> var_name2id;
	// indexed variable name to id map
	std::map<chaos_string, int> var_name2id_indexed;

// create & destroy
public:
	// default constructor
	chaos_system();
	// destructor
	~chaos_system();
	// clear
	void clear();
	// reset
	bool reset();
	// restart
	bool restart();

// variable
public:
	// get variable id
	int get_variable_id(const chaos_string& variable_name) const;
	// get variable name
	chaos_string get_variable_name(int variable_id)  const;
	// variable exist check
	bool exist_variable(int id) const {
		ASSERT(id < (int)property_base);
		return variables->exist_value(id);
	}
	// get variable value
	const chaos_value& get_variable(int id) const {
		ASSERT(id < (int)property_base);
		return variables->get_value(id);
	}
	// set variable value
	void set_variable(int id, const chaos_value& value);
	// clear variable value
	void clear_variable(int id);
	// get variable value(priority system variable)
	const chaos_value& get_variable_ex(int id) const;
	chaos_value get_variable_ex(const chaos_string& name) const {
		return get_variable_ex(get_variable_id(name));
	}
	// set variable value(priority system variable)
	void set_variable_ex(int id, const chaos_value& value);
	void set_variable_ex(const chaos_string& name, const chaos_value& value) {
		set_variable_ex(get_variable_id(name), value);
	}

// property
private:
	// property exist check
	bool exist_property(var_id_type id) const {
		ASSERT((id >= property_base) && (id < sysproperty_base));
		return properties->exist_value((int)id);
	}
	// get property value
	const chaos_value& get_property(var_id_type id) const {
		ASSERT((id >= property_base) && (id < sysproperty_base));
		return properties->get_value((int)id);
	}
	// set property value
	void set_property(var_id_type id, const chaos_value& value);
	// clear property value
	void clear_property(var_id_type id);
	// get integer property
	int get_int_property(var_id_type id) const {
		return (int)get_property(id).get_integer();
	}
	// set integer property
	void set_int_property(var_id_type id, int value) {
		set_property(id, chaos_value((chaos_int32)value));
	}
	// get boolean property
	bool get_bool_property(var_id_type id) const {
		return get_property(id).get_boolean();
	}
	// set boolean property
	void set_bool_property(var_id_type id, bool value) {
		set_property(id, chaos_value(value));
	}
	// get color property
	chaos_color get_col_property(var_id_type id) const {
		return get_property(id).get_color();
	}
	// set color property
	void set_col_property(var_id_type id, chaos_color value) {
		set_property(id, chaos_value(value));
	}
	// get string property
	chaos_string get_str_property(var_id_type id) const {
		return get_property(id).get_string();
	}
	// set string property
	void set_str_property(var_id_type id, const chaos_string& value) {
		set_property(id, chaos_value(value));
	}

// system property
private:
	// system property exist check(numbered)
	bool exist_sysproperty(var_id_type id) const {
		ASSERT((id >= sysproperty_base));
		return sysproperties->exist_value((int)id);
	}
	// get system property value(numbered)
	const chaos_value& get_sysproperty(var_id_type id) const {
		ASSERT((id >= sysproperty_base));
		return sysproperties->get_value((int)id);
	}
	// set system property value(numbered)
	void set_sysproperty(var_id_type id, const chaos_value& value) {
		ASSERT((id >= sysproperty_base));
		sysproperties->set_value((int)id, value);
		dirty_profile = true;
	}
	// clear system property(numbered)
	void clear_sysproperty(var_id_type id) {
		ASSERT((id >= sysproperty_base));
		sysproperties->clear_value((int)id);
		dirty_profile = true;
	}
	// get integer system property(numbered)
	int get_int_sysproperty(var_id_type id) const {
		return (int)get_sysproperty(id).get_integer();
	}
	// set integer system property(numbered)
	void set_int_sysproperty(var_id_type id, int value) {
		set_sysproperty(id, chaos_value((chaos_int32)value));
	}
	// get boolean system property(numbered)
	bool get_bool_sysproperty(var_id_type id) const {
		return get_sysproperty(id).get_boolean();
	}
	// set boolean system property(numbered)
	void set_bool_sysproperty(var_id_type id, bool value) {
		set_sysproperty(id, chaos_value(value));
	}
	// get color system property(numbered)
	chaos_color get_col_sysproperty(var_id_type id) const {
		return get_sysproperty(id).get_color();
	}
	// set color system property(numbered)
	void set_col_sysproperty(var_id_type id, chaos_color value) {
		set_sysproperty(id, chaos_value(value));
	}
	// get string system property(numbered)
	chaos_string get_str_sysproperty(var_id_type id) const {
		return get_sysproperty(id).get_string();
	}
	// set string system property(numbered)
	void set_str_sysproperty(var_id_type id, const chaos_string& value) {
		set_sysproperty(id, chaos_value(value));
	}

public:
	// system property exist check
	bool exist_sysproperty(const chaos_string& name) const {
		var_id_type id = (var_id_type)get_variable_id(name);
		if (id != invalid_variable) {
			return exist_sysproperty(id);
		}
		return sysproperties_ln->exist_value(name);
	}
	// get system property value
	const chaos_value& get_sysproperty(const chaos_string& name) const {
		var_id_type id = (var_id_type)get_variable_id(name);
		if (id != invalid_variable) {
			return get_sysproperty(id);
		}
		return sysproperties_ln->get_value(name);
	}
	// set system property value
	void set_sysproperty(const chaos_string& name, const chaos_value& value) {
		var_id_type id = (var_id_type)get_variable_id(name);
		if (id != invalid_variable) {
			set_sysproperty(id, value);
			return;
		}
		sysproperties_ln->set_value(name, value);
		dirty_profile = true;
	}
	// clear system property
	void clear_sysproperty(const chaos_string& name) {
		var_id_type id = (var_id_type)get_variable_id(name);
		if (id != invalid_variable) {
			clear_sysproperty(id);
			return;
		}
		sysproperties_ln->clear_value(name);
		dirty_profile = true;
	}
	// get integer system property
	int get_int_sysproperty(const chaos_string& name) const {
		return (int)get_sysproperty(name).get_integer();
	}
	// set integer system property
	void set_int_sysproperty(const chaos_string& name, int value) {
		set_sysproperty(name, chaos_value((chaos_int32)value));
	}
	// get boolean system property
	bool get_bool_sysproperty(const chaos_string& name) const {
		return get_sysproperty(name).get_boolean();
	}
	// set boolean system property
	void set_bool_sysproperty(const chaos_string& name, bool value) {
		set_sysproperty(name, chaos_value(value));
	}
	// get color system property
	chaos_color get_col_sysproperty(const chaos_string& name) const {
		return get_sysproperty(name).get_color();
	}
	// set color system property
	void set_col_sysproperty(const chaos_string& name, chaos_color value) {
		set_sysproperty(name, chaos_value(value));
	}
	// get string system property
	chaos_string get_str_sysproperty(const chaos_string& name) const {
		return get_sysproperty(name).get_string();
	}
	// set string system property
	void set_str_sysproperty(
		const chaos_string& name, const chaos_string& value)
	{
		set_sysproperty(name, chaos_value(value));
	}
	// get start script filename
	chaos_string get_start_script_filename() const {
		return get_str_sysproperty(start_script);
	}
	// get start script label
	chaos_string get_start_script_label() const {
		return get_str_sysproperty(start_label);
	}
	// get system width
	int get_sys_width() const {
		return get_int_sysproperty(sys_width);
	}
	// get system height
	int get_sys_height() const {
		return get_int_sysproperty(sys_height);
	}
	// get system font name
	chaos_string get_sys_font_name() const {
		return get_str_sysproperty(sys_font_name);
	}
	// get system font xlfd
	chaos_string get_sys_font_xlfd() const {
		return get_str_sysproperty(sys_font_xlfd);
	}
	// get system font pango
	chaos_string get_sys_font_pango() const {
		return get_str_sysproperty(sys_font_pango);
	}
	// get small font size
	int get_sys_small_font_size() const {
		return get_int_sysproperty(sys_small_font_size);
	}
	// get system medium font size
	int get_sys_medium_font_size() const {
		return get_int_sysproperty(sys_medium_font_size);
	}
	// get large font size
	int get_sys_large_font_size() const {
		return get_int_sysproperty(sys_large_font_size);
	}
	// is system font bold
	bool is_sys_font_bold() const {
		return get_bool_sysproperty(sys_font_bold);
	}
	// is system font anti alias
	bool is_sys_font_anti_alias() const {
		return get_bool_sysproperty(sys_font_anti_alias);
	}
	// get sys image buffer count
	int get_sys_image_buffer_count() const {
		return get_int_sysproperty(sys_image_buffer_count);
	}
	// get effect step
	int get_effect_step() const {
		return get_int_sysproperty(effect_step);
	}
	// is effect cut
	bool is_effect_cut() const {
		return get_bool_sysproperty(effect_cut);
	}
	// get message window info count
	int get_message_window_info_count() const {
		return get_int_sysproperty(meswin_info_count);
	}
	// get message window info
	chaos_string get_message_window_info(int mwno) const {
		if ((mwno < 0) || (mwno >= (int)meswin_max)) {
			return chaos_string();
		}
		return get_str_sysproperty((var_id_type)(meswin_info0+mwno));
	}
	// get log mode
	log_mode_type get_log_mode() const {
		return (log_mode_type)get_int_sysproperty(log_mode);
	}
	// set log mode
	void set_log_mode(log_mode_type new_mode) {
		set_int_sysproperty(log_mode, (int)new_mode);
	}
	// get log level
	log_level_type get_log_level() const {
		return (log_level_type)get_int_sysproperty(log_level);
	}
	// set log level
	void set_log_level(log_level_type new_level) {
		set_int_sysproperty(log_level, (int)new_level);
	}
	// is full screen
	bool is_full_screen() const {
		return get_bool_sysproperty(full_screen);
	}
	// set full screen
	void set_full_screen(bool new_full_screen) {
		set_bool_sysproperty(full_screen, new_full_screen);
	}
	// is window center
	bool is_window_center() const {
		return get_bool_sysproperty(window_center);
	}
	// set window center
	void set_window_center(bool new_window_center) {
		set_bool_sysproperty(window_center, new_window_center);
	}
	// get font name
	chaos_string get_font_name() const {
		return get_str_sysproperty(font_name);
	}
	// set font name
	void set_font_name(const chaos_string& new_font_name) {
		set_str_sysproperty(font_name, new_font_name);
	}
	// get font xlfd
	chaos_string get_font_xlfd() const {
		return get_str_sysproperty(font_xlfd);
	}
	// set font xlfd
	void set_font_xlfd(const chaos_string& new_font_xlfd) {
		set_str_sysproperty(font_xlfd, new_font_xlfd);
	}
	// get font pango
	chaos_string get_font_pango() const {
		return get_str_sysproperty(font_pango);
	}
	// set font pango
	void set_font_pango(const chaos_string& new_font_pango) {
		set_str_sysproperty(font_pango, new_font_pango);
	}
	// get font size
	int get_font_size() const {
		return get_int_sysproperty(font_size);
	}
	// set font size
	void set_font_size(int new_font_size) {
		set_int_sysproperty(font_size, new_font_size);
	}
	// is font bold
	bool is_font_bold() const {
		return get_bool_sysproperty(font_bold);
	}
	// set font bold
	void set_font_bold(bool new_bold) {
		set_bool_sysproperty(font_bold, new_bold);
	}
	// is font anti alias
	bool is_font_anti_alias() const {
		return get_bool_sysproperty(font_anti_alias);
	}
	// set font anti alias
	void set_font_anti_alias(bool new_alias) {
		set_bool_sysproperty(font_anti_alias, new_alias);
	}
	// get text speed
	text_speed_type get_text_speed() const {
		return (text_speed_type)get_int_sysproperty(text_speed);
	}
	// set text speed
	void set_text_speed(text_speed_type new_speed) {
		set_int_sysproperty(text_speed, (int)new_speed);
	}
	// get skip speed
	skip_speed_type get_skip_speed() const {
		return (skip_speed_type)get_int_sysproperty(skip_speed);
	}
	// set skip speed
	void set_skip_speed(skip_speed_type new_speed) {
		set_int_sysproperty(skip_speed, (int)new_speed);
	}
	// is show startup
	bool is_show_startup() const {
		return get_bool_sysproperty(show_startup);
	}
	// set show startup
	void set_show_startup(bool new_show_startup) {
		set_bool_sysproperty(show_startup, new_show_startup);
	}

// history
public:
	// get history count
	int get_history_count() const {
		return (int)histories.size();
	}
	// clear histories
	void clear_histories();
	// add new history
	void add_new_history();
	// rewind history
	void rewind_history(int step);
	// get history message text
	chaos_string get_history_message_text(int historyno) const;
	// is history requested
	bool is_history_requested() const {
		return history_requested;
	}
	// set history requested
	void set_history_requested(bool new_requested) {
		history_requested = new_requested;
	}

// snapshot
public:
	// is enable snapshot load
	bool is_enable_load() const {
		return get_bool_property(enable_load);
	}
	// set enable snapshot load
	void set_enable_load(bool new_enable) {
		set_bool_property(enable_load, new_enable);
	}
	// is enable snapshot save
	bool is_enable_save() const {
		return get_bool_property(enable_save);
	}
	// set enable snapshot save
	void set_enable_save(bool new_enable) {
		set_bool_property(enable_save, new_enable);
	}
	// load snapshot
	bool load_snapshot(const chaos_xml* snapshot);
	// save snapshot
	bool save_snapshot(chaos_xml* snapshot) const;

// profile
public:
	// is dirty profile(profile is updated)
	bool is_dirty_profile() const {
		return dirty_profile;
	}
	// set dirty profile(profile is updated)
	void set_dirty_profile(bool new_dirty) {
		dirty_profile = new_dirty;
	}
	// load profile
	bool load_profile(const chaos_xml* profile);
	// save profile
	bool save_profile(chaos_xml* profile) const;

// readflag
public:
	// is dirty readflag(readflag is updated)
	bool is_dirty_readflag() const {
		return dirty_readflag;
	}
	// set dirty readflag(readflag is updated)
	void set_dirty_readflag(bool new_dirty) {
		dirty_readflag = new_dirty;
	}
	// load readflag
	bool load_readflag(const chaos_xml* readf_xml);
	// save readflag
	bool save_readflag(chaos_xml* readf_xml) const;
	// is current message read
	bool is_current_message_read() const;

// fix
public:
	// fix exist check
	bool exist_fix(const chaos_string& name) const {
		return fixes->exist_value(name);
	}
	// get fix value
	const chaos_value& get_fix(const chaos_string& name) const {
		return fixes->get_value(name);
	}
	// set fix value
	void set_fix(const chaos_string& name, const chaos_value& value) {
		fixes->set_value(name, value);
	}

// system mode
public:
	// get system mode
	system_mode_type get_system_mode() const {
		return (system_mode_type)get_int_property(system_mode);
	}
	// set system mode
	void set_system_mode(system_mode_type new_mode) {
		set_int_property(system_mode, (int)new_mode);
	}
	// change system mode
	void change_system_mode(system_mode_type new_mode);
	// get wait time
	int get_wait_time() const {
		return get_int_property(wait_time);
	}
	// set wait time
	void set_wait_time(int new_time) {
		set_int_property(wait_time, new_time);
	}

// skip
public:
	// get skip mode
	skip_mode_type get_skip_mode() const {
		return skip_mode;
	}
	// set skip mode
	void set_skip_mode(skip_mode_type new_mode) {
		skip_mode = new_mode;
	}
	// is enable skip mode
	bool is_enable_skip_mode(skip_mode_type check_mode) const;
	// change skip mode
	void change_skip_mode(skip_mode_type new_mode);

// message
public:
	// get current message filename
	chaos_string get_cur_message_filename() const {
		return get_str_property(cur_message_filename);
	}
	// load current messages
	bool load_cur_messages(const chaos_string& message_filename);
	// get message frame count
	int get_message_frame_count() const {
		return get_int_property(message_frame_count);
	}
	// set message frame count
	void set_message_frame_count(int new_count) {
		set_int_property(message_frame_count, new_count);
	}
	// get message text
	chaos_string get_message_text(int frame) const {
		if ((frame < 0) || (frame >= (int)frame_max)) {
			return chaos_string();
		}
		return get_str_property((var_id_type)(message_text0+frame));
	}
	// set message text
	void set_message_text(int frame, const chaos_string& new_text) {
		if ((frame < 0) || (frame >= (int)frame_max)) {
			return;
		}
		set_str_property((var_id_type)(message_text0+frame), new_text);
	}
	// get message info
	chaos_string get_message_info(int frame) const {
		if ((frame < 0) || (frame >= (int)frame_max)) {
			return chaos_string();
		}
		return get_str_property((var_id_type)(message_info0+frame));
	}
	// set message info
	void set_message_info(int frame, const chaos_string& new_info) {
		if ((frame < 0) || (frame >= (int)frame_max)) {
			return;
		}
		set_str_property((var_id_type)(message_info0+frame), new_info);
	}
	// get message speed
	int get_message_speed(int frame) const {
		if ((frame < 0) || (frame >= (int)frame_max)) {
			return 0;
		}
		return get_int_property((var_id_type)(message_speed0+frame));
	}
	// set message speed
	void set_message_speed(int frame, int speed) {
		if ((frame < 0) || (frame >= (int)frame_max)) {
			return;
		}
		set_int_property((var_id_type)(message_speed0+frame), speed);
	}
	// get reserved message text
	chaos_string get_reserved_message_text() const {
		return get_str_property(reserved_message_text);
	}
	// set reserved message text
	void set_reserved_message_text(const chaos_string& new_text) {
		set_str_property(reserved_message_text, new_text);
	}
	// clear reserved message text
	void clear_reserved_message_text() {
		clear_property(reserved_message_text);
	}
	// get next message text
	chaos_string get_next_message_text() const {
		return get_str_property(next_message_text);
	}
	// set next message text
	void set_next_message_text(const chaos_string& new_text) {
		set_str_property(next_message_text, new_text);
	}
	// add message frame
	void add_message_frame();
	// add message
	void add_message(const chaos_string& add_text, bool wait);
	// clear message
	void clear_message();
	// wait message
	void wait_message();
	// copy next message text
	void copy_next_message_text();
	// flush next message text
	void flush_next_message_text();
	// get message substractive mixture color
	chaos_color get_message_subcolor() const {
		return get_col_property(message_subcolor);
	}
	// set message substractive mixture color
	void set_message_subcolor(chaos_color new_color) {
		set_col_property(message_subcolor, new_color);
	}
	// get message shape
	int get_message_shape() const {
		return get_int_property(message_shape);
	}
	// set message shape
	void set_message_shape(int new_mshape) {
		set_int_property(message_shape, new_mshape);
	}
	// get message clear mode
	int get_message_clear_mode() const {
		return get_int_property(message_clear);
	}
	// set message clear mode
	void set_message_clear_mode(int new_clear_mode) {
		set_int_property(message_clear, new_clear_mode);
	}
	// get prompt info
	chaos_string get_prompt_info() const {
		return get_str_property(prompt_info);
	}
	// set prompt info
	void set_prompt_info(const chaos_string& new_info) {
		set_str_property(prompt_info, new_info);
	}
	// clear prompt info
	void clear_prompt_info() {
		clear_property(prompt_info);
	}
	// is prompt visible
	bool is_prompt_visible() const {
		return get_bool_property(prompt_visible);
	}
	// set prompt visible
	void set_prompt_visible(bool new_visible) {
		set_bool_property(prompt_visible, new_visible);
	}
	// encode message info
	static chaos_string encode_message_info(
		int x, int y,
		int length,
		chaos_color color);
	// decode message info
	static bool decode_message_info(
		const chaos_string& info,
		int& x, int& y,
		int& length,
		chaos_color& color);
	// encode prompt info
	static chaos_string encode_prompt_info(
		bool place,
		int x, int y);
	// decode prompt info
	static bool decode_prompt_info(
		const chaos_string& info,
		bool& place,
		int& x, int& y);

// selection
public:
	// get selection default color
	chaos_color get_selection_default_color() const {
		return get_col_property(sel_defcolor);
	}
	// set selection default color
	void set_selection_default_color(chaos_color new_color) {
		set_col_property(sel_defcolor, new_color);
	}
	// get selection count
	int get_selection_count() const {
		return get_int_property(sel_count);
	}
	// set selection count
	void set_selection_count(int new_count) {
		set_int_property(sel_count, new_count);
	}
	// get current selection
	int get_current_selection() const {
		return get_int_property(sel_current);
	}
	// set current selection
	void set_current_selection(int new_current) {
		set_int_property(sel_current, new_current);
	}
	// is selection preparation
	bool is_selection_preparation() const {
		return get_bool_property(sel_preparation);
	}
	// set selection preparation
	void set_selection_preparation(bool new_preparation) {
		set_bool_property(sel_preparation, new_preparation);
	}
	// get selection frame count
	int get_selection_frame_count() const {
		return get_int_property(sel_frame_count);
	}
	// set selection frame count
	void set_selection_frame_count(int new_count) {
		set_int_property(sel_frame_count, new_count);
	}
	// get selection info
	chaos_string get_selection_info(int selno) const {
		if ((selno < 0) || (selno >= (int)selection_max)) {
			return chaos_string();
		}
		return get_str_property((var_id_type)(sel_info0+selno));
	}
	// set selection info
	void set_selection_info(int selno, const chaos_string& new_info) {
		if ((selno < 0) || (selno >= (int)selection_max)) {
			return;
		}
		set_str_property((var_id_type)(sel_info0+selno), new_info);
	}
	// start selection preparation
	void start_selection_preparation();
	// finish selection preparation
	void finish_selection_preparation();
	// clear selection
	void clear_selection();
	// encode selection info
	static chaos_string encode_selection_info(
		int types,
		int x1, int y1, int x2, int y2,
		int unk1, int unk2, int unk3,
		chaos_color color,
		const chaos_string& filename,
		const chaos_string& label);
	// decode selection info
	static bool decode_selection_info(
		const chaos_string& info,
		int& types,
		int& x1, int& y1, int& x2, int& y2,
		int& unk1, int& unk2, int& unk3,
		chaos_color& color,
		chaos_string& filename,
		chaos_string& label);
	// decode selection info(omit string info)
	static bool decode_selection_info(
		const chaos_string& info,
		int& types,
		int& x1, int& y1, int& x2, int& y2,
		int& unk1, int& unk2, int& unk3,
		chaos_color& color);

// image
public:
	// is dirty main image
	bool is_dirty_image() const {
		return dirty_image;
	}
	// set dirty main image
	void set_dirty_image(bool new_dirty) {
		dirty_image = new_dirty;
	}
	// is dirty message
	bool is_dirty_message() const {
		return dirty_message;
	}
	// set dirty message
	void set_dirty_message(bool new_dirty) {
		dirty_message = new_dirty;
	}
	// get image build info
	chaos_string get_image_info(int imageno) const {
		if ((imageno < 0) || (imageno >= (int)image_max)) {
			return chaos_string();
		}
		return get_str_property((var_id_type)(image_info0+imageno));
	}
	// set image build info
	void set_image_info(int imageno, const chaos_string& new_info) {
		if ((imageno < 0) || (imageno >= (int)image_max)) {
			return;
		}
		set_str_property((var_id_type)(image_info0+imageno), new_info);
	}
	// clear image build info
	void clear_image_info(int imageno) {
		if ((imageno < 0) || (imageno >= (int)image_max)) {
			return;
		}
		clear_property((var_id_type)(image_info0+imageno));
	}

// cd audio
public:
	// get cd info
	chaos_string get_cd_info() const {
		return get_str_sysproperty(cd_info);
	}
	// set cd info
	void set_cd_info(const chaos_string& new_cd_info) {
		set_str_sysproperty(cd_info, new_cd_info);
	}
	// get cd drive
	chaos_string get_cd_drive() const {
		return get_str_sysproperty(cd_drive);
	}
	// set cd drive
	void set_cd_drive(const chaos_string& new_cd_drive) {
		set_str_sysproperty(cd_drive, new_cd_drive);
	}
	// get cd audio info
	chaos_string get_cd_audio_info() const {
		return get_str_property(cd_audio_info);
	}
	// set cd audio info
	void set_cd_audio_info(const chaos_string& new_info) {
		set_str_property(cd_audio_info, new_info);
	}
	// clear cd audio info
	void clear_cd_audio_info() {
		clear_property(cd_audio_info);
	}
	// play cd audio
	bool play_cd_audio(const chaos_string& info);
	// encode cd audio info
	static chaos_string encode_cd_audio_info(
		int track1, int track2,
		int repeat);
	// decode cd audio info
	static bool decode_cd_audio_info(
		const chaos_string& info,
		int& track1, int& track2,
		int& repeat);

// wave audio
public:
	// get wave audio info
	chaos_string get_wave_audio_info() const {
		return get_str_property(wave_audio_info);
	}
	// set wave audio info
	void set_wave_audio_info(const chaos_string& new_info) {
		set_str_property(wave_audio_info, new_info);
	}
	// clear wave audio info
	void clear_wave_audio_info() {
		clear_property(wave_audio_info);
	}
	// play wave audio
	bool play_wave_audio(const chaos_string& info);
	// encode wave audio info
	static chaos_string encode_wave_audio_info(
		int unk, int repeat,
		const chaos_string& filename);
	// decode wave audio info
	static bool decode_wave_audio_info(
		const chaos_string& info,
		int& unk, int& repeat,
		chaos_string& filename);

// sound
public:
	// get bgm sound type
	chaos_string get_bgm_sound_type() const {
		return get_str_sysproperty(bgm_sound_type);
	}
	// set bgm sound type
	void set_bgm_sound_type(const chaos_string& new_bgm_sound_type) {
		set_str_sysproperty(
			bgm_sound_type, new_bgm_sound_type);
	}
	// get bgm sound volume
	int get_bgm_sound_volume() const {
		return get_int_sysproperty(bgm_sound_volume);
	}
	// set bgm sound volume
	void set_bgm_sound_volume(int new_bgm_sound_volume) {
		set_int_sysproperty(
			bgm_sound_volume, new_bgm_sound_volume);
	}
	// get effect sound volume
	int get_effect_sound_volume() const {
		return get_int_sysproperty(effect_sound_volume);
	}
	// set effect sound volume
	void set_effect_sound_volume(int new_effect_sound_volume) {
		set_int_sysproperty(
			effect_sound_volume, new_effect_sound_volume);
	}

// stack
public:
	// get stack count
	int get_stack_count() const {
		return get_int_property(stack_count);
	}
	// set stack count
	void set_stack_count(int new_count) {
		set_int_property(stack_count, new_count);
	}
	// get stack build info
	chaos_string get_stack_info(int stackno) const {
		if ((stackno < 0) || (stackno >= (int)stack_max)) {
			return chaos_string();
		}
		return get_str_property((var_id_type)(stack_info0+stackno));
	}
	// set stack build info
	void set_stack_info(int stackno, const chaos_string& new_info) {
		if ((stackno < 0) || (stackno >= (int)stack_max)) {
			return;
		}
		set_str_property((var_id_type)(stack_info0+stackno), new_info);
	}
	// clear stack build info
	void clear_stack_info(int stackno) {
		if ((stackno < 0) || (stackno >= (int)stack_max)) {
			return;
		}
		clear_property((var_id_type)(stack_info0+stackno));
	}
	// store stack info
	void store_stack_info();
	// reflect stack info
	void reflect_stack_info();
	// encode stack info
	static chaos_string encode_stack_info(
		const chaos_string& filename,
		int id,
		int line, int column);
	// decode stack info
	static bool decode_stack_info(
		const chaos_string& info,
		chaos_string& filename,
		int& id,
		int& line, int& column);

// script
public:
	// load script
	bool load_script(
		chaos_file_data* script,
		const chaos_string& script_filename,
		int script_id);
	// goto script
	bool goto_script(
		const chaos_string& script_filename,
		int script_id,
		int script_line, int script_column,
		const chaos_string& script_label);
	// gosub script
	bool gosub_script(
		const chaos_string& script_filename,
		int script_id,
		int script_line, int script_column,
		const chaos_string& script_label);
	// return script
	bool return_script();
	// request jump
	void request_jump(
		jump_mode_type  new_jump_mode,
		const chaos_string& new_jump_filename,
		int new_jump_id,
		int new_jump_line, int new_jump_column,
		const chaos_string& new_jump_label);

// operation
public:
	// execute chaos system
	bool execute(int pass_time);
};

#endif // CHAOS_SYSTEM_H_INCLUDED
