////////////////////////////////////////////////////////////////////////////////
// chaos_token.h : script token

#ifndef CHAOS_TOKEN_H_INCLUDED
#define CHAOS_TOKEN_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// declare

class chaos_token;

////////////////////////////////////////////////////////////////////////////////
// include

#ifndef CHAOS_STRING_H_INCLUDED
#include "chaos_string.h"
#endif // CHAOS_STRING_H_INCLUDED

#ifndef CHAOS_VALUE_H_INCLUDED
#include "chaos_value.h"
#endif // CHAOS_VALUE_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// script token class

class chaos_token {
// enum
public:
	// type
	enum token_type {
		tkn_nul,
		tkn_unknown,
		tkn_value,
		tkn_variable,
		tkn_fix,
		tkn_rgb,
		tkn_func,
		tkn_label,
		tkn_equal,
		tkn_notequal,
		tkn_less,
		tkn_lessequal,
		tkn_greater,
		tkn_greaterequal,
		tkn_and,
		tkn_or,
		tkn_not,
		tkn_plus,
		tkn_minus,
		tkn_let,
		tkn_inc,
		tkn_dec,
		tkn_parenleft,
		tkn_parenright,
		tkn_braceleft,
		tkn_braceright,
		tkn_comma,
		tkn_period,
		tkn_semicolon,
		tkn_linecomment,
		tkn_startcomment,
		tkn_endcomment
	};

// attribute
private:
	// token type
	token_type types;
	// token literal
	chaos_string literals;
	// token value
	chaos_value values;

// create & destroy
public:
	// default constructor
	chaos_token() :
		types(tkn_nul),
		literals(),
		values()
	{
	}
	// copy constructor
	chaos_token(const chaos_token& org) :
		types(org.types),
		literals(org.literals),
		values(org.values)
	{
	}
	// normal constructor
	chaos_token(
			token_type new_types,
			const chaos_string& new_literals,
			const chaos_value& new_values = chaos_value()) :
		types(new_types),
		literals(new_literals),
		values(new_values)
	{
	}
	// destructor
	~chaos_token() {
	}

// get & set
public:
	// get token type
	token_type get_type() const {
		return types;
	}
	// set token type
	void set_type(token_type new_types) {
		types = new_types;
	}
	// get literal
	const chaos_string& get_literal() const {
		return literals;
	}
	// set literal
	void set_literal(const chaos_string& new_literals) {
		literals = new_literals;
	}
	// get value
	const chaos_value& get_value() const {
		return values;
	}
	// set value
	void set_value(const chaos_value& new_values) {
		values = new_values;
	}

// operator
public:
	// copy
	chaos_token& operator=(const chaos_token& org) {
		types = org.types;
		literals = org.literals;
		values = org.values;
		return *this;
	}
};

#endif // CHAOS_TOKEN_H_INCLUDED
