////////////////////////////////////////////////////////////////////////////////
// chaos_utility.h : utility

#ifndef CHAOS_UTILITY_H_INCLUDED
#define CHAOS_UTILITY_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// declare

class chaos_utility;

////////////////////////////////////////////////////////////////////////////////
// include

#ifndef CHAOS_STRING_H_INCLUDED
#include "chaos_string.h"
#endif // CHAOS_STRING_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// chaos_utility : utility class

class chaos_utility {
// char
public:
	// is space character
	static bool is_space(int ch) {
		return (ch > '\0') && (ch <= ' ');
	}
	static bool is_space(char ch) {
		return (ch > '\0') && (ch <= ' ');
	}
	// is digit character
	static bool is_digit(int ch) {
		return (ch >= '0') && (ch <= '9');
	}
	static bool is_digit(char ch) {
		return (ch >= '0') && (ch <= '9');
	}
	// is hex character
	static bool is_hex(int ch) {
		return ((ch >= '0') && (ch <= '9')) ||
			((ch >= 'A') && (ch <= 'F')) ||
			((ch >= 'a') && (ch <= 'f'));
	}
	static bool is_hex(char ch) {
		return ((ch >= '0') && (ch <= '9')) ||
			((ch >= 'A') && (ch <= 'F')) ||
			((ch >= 'a') && (ch <= 'f'));
	}
	// is alphabet character
	static bool is_alpha(int ch) {
		return ((ch >= 'A') && (ch <= 'Z')) ||
			((ch >= 'a') && (ch <= 'z'));
	}
	static bool is_alpha(char ch) {
		return ((ch >= 'A') && (ch <= 'Z')) ||
			((ch >= 'a') && (ch <= 'z'));
	}
	// is upper alphabet character
	static bool is_ualpha(int ch) {
		return (ch >= 'A') && (ch <= 'Z');
	}
	static bool is_ualpha(char ch) {
		return (ch >= 'A') && (ch <= 'Z');
	}
	// is lower alphabet character
	static bool is_lalpha(int ch) {
		return (ch >= 'a') && (ch <= 'z');
	}
	static bool is_lalpha(char ch) {
		return (ch >= 'a') && (ch <= 'z');
	}
	// to upper
	static char to_upper(char ch) {
		return ((ch >= 'a') && (ch <= 'z'))?
			(char)(ch-('a'-'A')): ch;
	}
	// to lowper
	static char to_lower(char ch) {
		return ((ch >= 'A') && (ch <= 'Z'))?
			(char)(ch+('a'-'A')): ch;
	}
	// is sjis lead character
	static bool is_sjislead(int ch) {
		return ((ch >= 0x81) && (ch <= 0x9F)) ||
			((ch >= 0xE0) && (ch <= 0xEF));
	}
	static bool is_sjislead(char ch) {
		return ((ch >= '\x81') && (ch <= '\x9F')) ||
			((ch >= '\xE0') && (ch <= '\xEF'));
	}
	// is euc lead character
	static bool is_euclead(int ch) {
		return (ch >= 0x8E) ||
			((ch >= 0xA1) && (ch <= 0xFF));
	}
	static bool is_euclead(char ch) {
		return (ch >= '\x8E') ||
			((ch >= '\xA1') && (ch <= '\xFF'));
	}
	// sjis to jis
	static int sjis_to_jis(int sjis);
	// jis to sjis
	static int jis_to_sjis(int jis);
	// euc to jis
	static int euc_to_jis(int euc);
	// jis to euc
	static int jis_to_euc(int jis);
	// sjis to euc
	static int sjis_to_euc(int sjis) {
		return jis_to_euc(sjis_to_jis(sjis));
	}
	// euc to sjis
	static int euc_to_sjis(int euc) {
		return jis_to_sjis(euc_to_jis(euc));
	}
	// jis hankaku to zenkaku
	static int jis_han_to_zen(int han);
	// jis zenkaku to hankaku
	static int jis_zen_to_han(
		int zen,
		bool* dakuten_ptr = NULL, bool* handakuten_ptr = NULL);

// string
public:
	// compare ignore case
	static int icompare(
		const chaos_string& str1, const chaos_string& str2);
	// is string sjis trail character
	static bool is_str_sjistrail(
		const chaos_string& str,
		chaos_string::size_type index);
	// to upper string
	static chaos_string to_upper_str(const chaos_string& str);
	// to lower string
	static chaos_string to_lower_str(const chaos_string& str);
	// format string
	static chaos_string format_str(const char* format, ...);
	// serialize string to string
	static chaos_string str_to_str(const char* target);
	static chaos_string str_to_str(const chaos_string& target) {
		return str_to_str(target.c_str());
	}
	// serialize string to string(designate quote character)
	static chaos_string str_to_str(const char* target, char quote);
	static chaos_string str_to_str(const chaos_string& target, char quote) {
		return str_to_str(target.c_str(), quote);
	}
	// serialize string from string
	static chaos_string str_from_str(const char*& serial);
	static chaos_string str_from_str(const chaos_string& org) {
		const char* serial = org.c_str();
		return str_from_str(serial);
	}
	// find string
	static int str_find_str(
		const char* target, const char* keyword,
		int startofs = 0);
	// find string
	static int str_find_str(
		const chaos_string& target, const chaos_string& keyword,
		int startofs = 0)
	{
		return str_find_str(target.c_str(), keyword.c_str(), startofs);
	}

#ifdef CHAOS_API_WIN32

	// utf-8 string to wchar_t string
	static void utf8_to_wstr(
		std::vector<wchar_t>& wstr,
		const chaos_string& utf8str);
	// wchar_t string to utf-8 string
	static void wstr_to_utf8(
		chaos_string& utf8str,
		const std::vector<wchar_t>& wstr);

#endif // CHAOS_API

	// sjis string to euc string
	static chaos_string sjis_to_euc(const chaos_string& sjisstr);
	// euc string to sjis string
	static chaos_string euc_to_sjis(const chaos_string& eucstr);
	// sjis string to utf-8 string
	static chaos_string sjis_to_utf8(const chaos_string& sjisstr);
	// utf-8 string to sjis string
	static chaos_string utf8_to_sjis(const chaos_string& utf8str);
	// utf-16le string to sjis string
	static chaos_string utf16le_to_sjis(const chaos_string& utf16lstr);
	// utf-16be string to sjis string
	static chaos_string utf16be_to_sjis(const chaos_string& utf16bstr);

	// source string to sjis string
	static chaos_string source_to_sjis(const chaos_string& srcstr);

	// sjis string to locale string
	static chaos_string sjis_to_locale(const chaos_string& sjisstr);
	// locale string to sjis string
	static chaos_string locale_to_sjis(const chaos_string& locstr);

	// sjis string to filename string
	static chaos_string sjis_to_filename(const chaos_string& sjisstr);
	// filename string to sjis string
	static chaos_string filename_to_sjis(const chaos_string& filestr);

	// sjis string to gui string
	static chaos_string sjis_to_gui(const chaos_string& sjisstr);
	// gui string to sjis string
	static chaos_string gui_to_sjis(const chaos_string& guistr);

// xml
public:
	// serialize string to xml string
	static chaos_string str_to_xmlstr(const char* target);
	static chaos_string str_to_xmlstr(const chaos_string& target) {
		return str_to_xmlstr(target.c_str());
	}
	// serialize string from xml string
	static chaos_string str_from_xmlstr(const char*& serial);
	static chaos_string str_from_xmlstr(const chaos_string& org) {
		const char* serial = org.c_str();
		return str_from_xmlstr(serial);
	}
	// get xml tag name
	static chaos_string get_xml_tag_name(
		const chaos_string& tag);
	// get xml tag attribute value
	static chaos_string get_xml_tag_attr_val(
		const chaos_string& tag,
		const chaos_string& attr_name);
};

#endif // CHAOS_UTILITY_H_INCLUDED
