////////////////////////////////////////////////////////////////////////////////
// chaos_value.cpp : value

////////////////////////////////////////////////////////////////////////////////
// include

#include "stdafx.h"

#include "chaos_value.h"
#include "chaos_utility.h"

////////////////////////////////////////////////////////////////////////////////
// chaos_value : value class

////////////////////////////////////////////////////////////////////////////////
// attribute

// null value

chaos_value chaos_value::null_value;

////////////////////////////////////////////////////////////////////////////////
// get & set

// get integer value

chaos_int32 chaos_value::get_integer() const {
	chaos_int32 result_int_value = 0;
	switch (types) {
	case integer:
	case boolean:
	case color:
		result_int_value = int_value;
		break;
	case string:
		result_int_value =
			(chaos_int32)strtol(str_value.c_str(), NULL, 0);
		break;
	}
	return result_int_value;
}

// get boolean value

bool chaos_value::get_boolean() const {
	bool result_bool_value = false;
	switch (types) {
	case integer:
	case boolean:
	case color:
		result_bool_value = (int_value != 0);
		break;
	case string:
		result_bool_value = (from_string(str_value).int_value != 0);
		break;
	}
	return result_bool_value;
}

// get color value

chaos_color chaos_value::get_color() const {
	chaos_color result_col_value;
	switch (types) {
	case integer:
	case boolean:
	case color:
		result_col_value.set_value(int_value);
		break;
	case string:
		result_col_value.set_value(from_string(str_value).int_value);
		break;
	}
	return result_col_value;
}

// get string value

chaos_string chaos_value::get_string() const {
	chaos_string result_str_value;
	switch (types) {
	case integer:
		result_str_value = chaos_utility::format_str(
			"%d",
			(int)int_value);
		break;
	case boolean:
		result_str_value = (int_value != 0)? "TRUE": "FALSE";
		break;
	case color:
		{ // dummy block
			chaos_color col;
			col.set_value(int_value);
			result_str_value = chaos_utility::format_str(
				"RGB(%d,%d,%d)",
				col.get_red(),
				col.get_green(),
				col.get_blue());
		}
		break;
	case string:
		result_str_value = str_value;
		break;
	}
	return result_str_value;
}

////////////////////////////////////////////////////////////////////////////////
// operation

// compare

int chaos_value::compare(const chaos_value& org) const {
	value_type compare_types;
	if (types == org.types) {
		compare_types = types;
	} else if (
		(types == integer) || (org.types == integer) ||
		(types == boolean) || (org.types == boolean))
	{
		compare_types = integer;
	} else if ((types == color) || (org.types == color)) {
		compare_types = color;
	} else {
		compare_types = string;
	}
	int result = 0;
	switch (compare_types) {
	case integer:
		{ // dummy block
			chaos_int32 left = get_integer(),
				right = org.get_integer();
			if (left < right) {
				result = -1;
			} else if (left > right) {
				result = 1;
			}
		}
		break;
	case color:
		{ // dummy block
			chaos_color left = get_color(),
				right = org.get_color();
			if (left < right) {
				result = -1;
			} else if (left > right) {
				result = 1;
			}
		}
		break;
	case string:
		{ // dummy block
			chaos_string left = get_string(),
				right = org.get_string();
			result = left.compare(right);
		}
		break;
	}
	return result;
}

////////////////////////////////////////////////////////////////////////////////
// operator

// copy

chaos_value& chaos_value::operator=(const chaos_value& org) {
	types = org.types;
	int_value = org.int_value;
	str_value = org.str_value;
	return *this;
}

// add let / append

chaos_value& chaos_value::operator+=(const chaos_value& org) {
	value_type compare_types;
	if (types == org.types) {
		compare_types = types;
	} else if ((types == string) || (org.types == string)) {
		compare_types = string;
	} else if (
		(types == integer) || (org.types == integer) ||
		(types == boolean) || (org.types == boolean))
	{
		compare_types = integer;
	} else {
		compare_types = color;
	}
	switch (compare_types) {
	case integer:
		{ // dummy block
			chaos_int32 left = get_integer(),
				right = org.get_integer();
			*this = chaos_value(left+right);
		}
		break;
	case color:
		{ // dummy block
			chaos_color left = get_color(),
				right = org.get_color();
			*this = chaos_value(
				chaos_color(
					left.get_red()+right.get_red(),
					left.get_green()+right.get_green(),
					left.get_blue()+right.get_blue()));
		}
		break;
	case string:
		{ // dummy block
			chaos_string left = get_string(),
				right = org.get_string();
			*this = chaos_value(left+right);
		}
		break;
	}
	return *this;
}

// sub let

chaos_value& chaos_value::operator-=(const chaos_value& org) {
	value_type compare_types;
	if (types == org.types) {
		compare_types = types;
		if ((compare_types == string) || (compare_types == boolean)) {
			compare_types = integer;
		}
	} else if (
		(types == integer) || (org.types == integer) ||
		(types == boolean) || (org.types == boolean))
	{
		compare_types = integer;
	} else {
		compare_types = color;
	}
	switch (compare_types) {
	case integer:
		{ // dummy block
			chaos_int32 left = get_integer(),
				right = org.get_integer();
			*this = chaos_value(left-right);
		}
		break;
	case color:
		{ // dummy block
			chaos_color left = get_color(),
				right = org.get_color();
			*this = chaos_value(
				chaos_color(
					left.get_red()-right.get_red(),
					left.get_green()-right.get_green(),
					left.get_blue()-right.get_blue()));
		}
		break;
	}
	return *this;
}

////////////////////////////////////////////////////////////////////////////////
// serialize

// to string

chaos_string chaos_value::to_string() const {
	chaos_string serial;
	switch (types) {
	case nul:
		serial = "NUL";
		break;
	case integer:
		serial = chaos_utility::format_str("%d", (int)int_value);
		break;
	case boolean:
		serial = (int_value != 0)? "TRUE": "FALSE";
		break;
	case color:
		{ // dummy block
			chaos_color color;
			color.set_value(int_value);
			serial = chaos_utility::format_str(
				"RGB(%d,%d,%d)",
				color.get_red(), color.get_green(), color.get_blue());
		}
		break;
	case string:
		serial = chaos_utility::str_to_str(str_value);
		break;
	}
	return serial;
}

// from string

chaos_value chaos_value::from_string(const char*& serial) {
	chaos_value value;
	char ch;
	while ((ch = *serial) != '\0') {
		if (!chaos_utility::is_space(ch)) {
			break;
		}
		serial++;
	}
	if (ch == '\0') {
		return chaos_value();
	}
	if (chaos_utility::is_digit(ch) || (ch == '-')) {
		int next = 0,
			num = 0;
		sscanf(serial, "%d%n",&num, &next);
		value = chaos_value((chaos_int32)num);
		serial += next;
	} else if ((ch == '"') || (ch == '\'')) {
		value = chaos_value(
			chaos_utility::str_from_str(serial));
	} else {
		chaos_string word;
		while ((ch = *serial) != '\0') {
			if (!chaos_utility::is_alpha(ch)) {
				break;
			}
			serial++;
			word += ch;
		}
		if (chaos_utility::icompare(word, "RGB") == 0) {
			int next = 0,
				red = 0, green = 0, blue = 0;
			sscanf(
				serial,
				"(%d,%d,%d)%n",
				&red, &green, &blue, &next);
			value = chaos_value(chaos_color(red, green, blue));
			serial += next;
		} else if (chaos_utility::icompare(word, "TRUE") == 0) {
			value = chaos_value(true);
		} else if (chaos_utility::icompare(word, "FALSE") == 0) {
			value = chaos_value(false);
		} else if (chaos_utility::icompare(word, "NUL") == 0) {
		} else {
			if (word.length() <= 0) {
				serial++;
			}
		}
	}
	return value;
}
