////////////////////////////////////////////////////////////////////////////////
// chaos_value.h : value

#ifndef CHAOS_VALUE_H_INCLUDED
#define CHAOS_VALUE_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// declare

class chaos_value;

////////////////////////////////////////////////////////////////////////////////
// include

#ifndef CHAOS_COLOR_H_INCLUDED
#include "chaos_color.h"
#endif // CHAOS_COLOR_H_INCLUDED

#ifndef CHAOS_STRING_H_INCLUDED
#include "chaos_string.h"
#endif // CHAOS_STRING_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// chaos_value : value class

class chaos_value {
// enum
public:
	// type
	enum value_type {
		nul,
		integer,
		boolean,
		color,
		string
	};

// attribute
private:
	// value type
	value_type types;
	// integer value
	chaos_int32 int_value;
	// string value
	chaos_string str_value;

public:
	// null value
	static chaos_value null_value;

// create & destroy
public:
	// default constructor
	chaos_value() :
		types(nul),
		int_value(0),
		str_value()
	{
	}
	// copy constructor
	chaos_value(const chaos_value& org) :
		types(org.types),
		int_value(org.int_value),
		str_value(org.str_value)
	{
	}
	// integer value constructor
	explicit chaos_value(chaos_int32 new_int_value) :
		types(integer),
		int_value(new_int_value),
		str_value()
	{
	}
	// boolean value constructor
	explicit chaos_value(bool new_bool_value) :
		types(boolean),
		int_value(new_bool_value? 1: 0),
		str_value()
	{
	}
	// color value constructor
	explicit chaos_value(chaos_color new_col_value) :
		types(color),
		int_value(new_col_value.get_value()),
		str_value()
	{
	}
	// string value constructor
	explicit chaos_value(const chaos_string& new_str_value) :
		types(string),
		int_value(0),
		str_value(new_str_value)
	{
	}
	// destructor
	~chaos_value() {
	}

// get & set
public:
	// get value type
	value_type get_type() const {
		return types;
	}
	// get integer value
	chaos_int32 get_integer() const;
	// get boolean value
	bool get_boolean() const;
	// get color value
	chaos_color get_color() const;
	// get string value
	chaos_string get_string() const;

// operation
public:
	// compare
	int compare(const chaos_value& org) const;

// operator
public:
	// copy
	chaos_value& operator=(const chaos_value& org);
	// add let / append
	chaos_value& operator+=(const chaos_value& org);
	// sub let
	chaos_value& operator-=(const chaos_value& org);
	// equal
	bool operator==(const chaos_value& org) const {
		return compare(org) == 0;
	}
	// not equal
	bool operator!=(const chaos_value& org) const {
		return compare(org) != 0;
	}
	// less
	bool operator<(const chaos_value& org) const {
		return compare(org) < 0;
	}
	// less equal
	bool operator<=(const chaos_value& org) const {
		return compare(org) <= 0;
	}
	// greater
	bool operator>(const chaos_value& org) const {
		return compare(org) > 0;
	}
	// greater equal
	bool operator>=(const chaos_value& org) const {
		return compare(org) >= 0;
	}
	// add
	chaos_value operator+(const chaos_value& org) const {
		chaos_value tmp = *this;
		tmp += org;
		return tmp;
	}
	// sub
	chaos_value operator-(const chaos_value& org) const {
		chaos_value tmp = *this;
		tmp -= org;
		return tmp;
	}

// serialize
public:
	// to string
	chaos_string to_string() const;
	// from string
	static chaos_value from_string(const char*& serial);
	static chaos_value from_string(const chaos_string& org) {
		const char* serial = org.c_str();
		return from_string(serial);
	}
};

#endif // CHAOS_VALUE_H_INCLUDED
