////////////////////////////////////////////////////////////////////////////////
// chaos_value_manager.h : value manager

#ifndef CHAOS_VALUE_MANAGER_H_INCLUDED
#define CHAOS_VALUE_MANAGER_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// declare

template <class id_type>
class chaos_value_manager;

////////////////////////////////////////////////////////////////////////////////
// include

#ifndef CHAOS_VALUE_H_INCLUDED
#include "chaos_value.h"
#endif // CHAOS_VALUE_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// value manager class template

template <class id_type>
class chaos_value_manager {
// typedef
private:
	// my type
	typedef chaos_value_manager<id_type> my_type;
	// value map type
	typedef std::map<id_type, chaos_value> value_map;

public:
	// value type
	typedef typename value_map::value_type value_type;
	// size type
	typedef typename value_map::size_type size_type;
	// iterator
	typedef typename value_map::iterator iterator;
	// const iterator
	typedef typename value_map::const_iterator const_iterator;

// attribute
private:
	// values
	value_map values;

// create & destroy
public:
	// default constructor
	chaos_value_manager() {
	}
	// copy constructor
	chaos_value_manager(const my_type& org) :
		values(org.values)
	{
	}
	// destructor
	~chaos_value_manager() {
	}

// operation
public:
	// clear all values
	void clear() {
		values.clear();
	}
	// clear nul values
	void clear_nul() {
		typename value_map::iterator it = values.begin();
		while(it != values.end()) {
			if ((*it).second.get_type() == chaos_value::nul) {
				typename value_map::iterator it2 = it++;
				values.erase(it2);
			} else {
				it++;
			}
		}
	}
	// exist check
	bool exist_value(const id_type& id) const {
		chaos_value value;
		typename value_map::const_iterator value_it = values.find(id);
		return value_it != values.end();
	}
	// get value
	const chaos_value& get_value(const id_type& id) const {
		typename value_map::const_iterator value_it = values.find(id);
		return (value_it != values.end())?
			(*value_it).second:
			chaos_value::null_value;
	}
	// set value
	void set_value(const id_type& id, const chaos_value& value) {
		values[id] = value;
	}
	// clear value
	void clear_value(const id_type& id) {
		values.erase(id);
	}
	// merge
	void merge(const chaos_value_manager* org) {
		for (
			typename value_map::const_iterator it = org->values.begin();
			it != org->values.end();
			it++)
		{
			values[(*it).first] = (*it).second;
		}
	}
	// get size
	size_type size() const {
		return values.size();
	}
	// get begin
	const_iterator begin() const {
		return values.begin();
	}
	iterator begin() {
		return values.begin();
	}
	// get end
	const_iterator end() const {
		return values.end();
	}
	iterator end() {
		return values.end();
	}
};

#endif // CHAOS_VALUE_MANAGER_H_INCLUDED
