////////////////////////////////////////////////////////////////////////////////
// chaos_wave_player.cpp : wave player

////////////////////////////////////////////////////////////////////////////////
// include

#include "stdafx.h"

#include "chaos_wave_player.h"
#include "chaos_file_data.h"
#include "chaos_file_serializer.h"

////////////////////////////////////////////////////////////////////////////////
// chaos_wave_player : wave player class

////////////////////////////////////////////////////////////////////////////////
// create & destroy

// default constructor

chaos_wave_player::chaos_wave_player() {
}

// destructor

chaos_wave_player::~chaos_wave_player() {
	dispose();
}

// dispose

void chaos_wave_player::dispose() {
	stop();
}

// get singleton instance

chaos_wave_player* chaos_wave_player::instance() {
	static chaos_wave_player wave_player;
	return &wave_player;
}

////////////////////////////////////////////////////////////////////////////////
// play

// play wave audio

bool chaos_wave_player::play(
	const chaos_string& new_wave_filename,
	bool new_repeat)
{
	if ((new_wave_filename == playing_wave_filename) &&
		(new_repeat == repeat) &&
		repeat)
	{
		return true;
	}
	if (is_playing()) {
		if (!stop()) {
			return false;
		}
	}
	playing_wave_filename = "";
	repeat = false;
	chaos_file_data filedata;
	if (!chaos_file_serializer::instance()->load_file_data(
			&filedata,
			new_wave_filename,
			chaos_file_serializer::fpos_packed))
	{
		LOG_ERR(
			"error : wave player::play - "
			"load error(%s)\n",
			new_wave_filename.c_str());
		return false;
	}
	size_t size = filedata.size();
	wave_buffer.resize(size);
	if (filedata.read(
			wave_buffer.data(), size) != size)
	{
		LOG_ERR(
			"error : wave player::play - "
			"read error\n");
		wave_buffer.clear_all();
		return false;
	}
	if (!sound.play_sound_from_memory(
			wave_buffer.data(),
			wave_buffer.size(),
			new_repeat))
	{
		LOG_ERR(
			"error : wave player::play - "
			"play error\n");
		wave_buffer.clear_all();
		return false;
	}
	playing_wave_filename = new_wave_filename;
	repeat = new_repeat;
	return true;
}

// stop wave audio

bool chaos_wave_player::stop() {
	sound.dispose();
	playing_wave_filename = "";
	repeat = false;
	wave_buffer.clear_all();
	return true;
}

// set sound volume

void chaos_wave_player::set_sound_volume(int new_sound_volume) {
	sound.set_volume(new_sound_volume);
}
