////////////////////////////////////////////////////////////////////////////////
// chaos_wave_player.h : wave player

#ifndef CHAOS_WAVE_PLAYER_H_INCLUDED
#define CHAOS_WAVE_PLAYER_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// declare

class chaos_wave_player;

////////////////////////////////////////////////////////////////////////////////
// include

#ifndef CHAOS_STRING_H_INCLUDED
#include "chaos_string.h"
#endif // CHAOS_STRING_H_INCLUDED

#ifndef CHAOS_BUFFER_H_INCLUDED
#include "chaos_buffer.h"
#endif // CHAOS_BUFFER_H_INCLUDED

#ifndef CHAOS_SOUND_MANAGER_H_INCLUDED
#include "chaos_sound_manager.h"
#endif // CHAOS_SOUND_MANAGER_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// chaos_wave_player : wave player class

class chaos_wave_player {
// attribute
private:
	// playing wave filename
	chaos_string playing_wave_filename;
	// repeat
	bool repeat;
	// wave buffer
	chaos_buffer wave_buffer;
	// sound object
	chaos_sound sound;

// create & destroy
private:
	// default constructor
	chaos_wave_player();

public:
	// destructor
	~chaos_wave_player();
	// dispose
	void dispose();

	// get singleton instance
	static chaos_wave_player* instance();

// get
public:
	// get playing wave filename
	const chaos_string& get_playing_wave_filename() const {
		return playing_wave_filename;
	}
	// is repeat
	bool is_repeat() const {
		return repeat;
	}
	// is playing
	bool is_playing() const {
		return wave_buffer.size() > 0;
	}

// play
public:
	// play wave audio
	bool play(
		const chaos_string& new_wave_filename,
		bool new_repeat);
	// stop wave audio
	bool stop();
	// notify wave audio
	bool notify();
	// set sound volume
	void set_sound_volume(int new_sound_volume);
};

#endif // CHAOS_WAVE_PLAYER_H_INCLUDED
