////////////////////////////////////////////////////////////////////////////////
// chaos_xml.h : xml data

#ifndef CHAOS_XML_H_INCLUDED
#define CHAOS_XML_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// declare

class chaos_xml;

////////////////////////////////////////////////////////////////////////////////
// include

#ifndef CHAOS_STRING_H_INCLUDED
#include "chaos_string.h"
#endif // CHAOS_STRING_H_INCLUDED

#ifndef CHAOS_FILE_DATA_H_INCLUDED
#include "chaos_file_data.h"
#endif // CHAOS_FILE_DATA_H_INCLUDED

#ifndef CHAOS_VALUE_MANAGER_H_INCLUDED
#include "chaos_value_manager.h"
#endif // CHAOS_VALUE_MANAGER_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// chaos_xml : xml data class

class chaos_xml {
// enum
private:
	enum {
		ENCODING_UNKNOWN,
		ENCODING_SJIS,
		ENCODING_EUC,
		ENCODING_UTF8,
		ENCODING_UTF16LE,
		ENCODING_UTF16BE,
		ENCODING_UTF8_BOM,
		ENCODING_UTF16LE_BOM,
		ENCODING_UTF16BE_BOM
	};

// attribute
private:
	// xml kind
	chaos_string xml_kind;
	// xml date
	chaos_string xml_date;
	// xml time
	chaos_string xml_time;
	// xml info
	chaos_string xml_info;
	// values version
	chaos_string values_version;
	// value
	chaos_value_manager<chaos_string>* values;

// create & destroy
public:
	// default constructor
	chaos_xml();
	// destructor
	~chaos_xml();
	// clear
	void clear();

// get & set
public:
	// get xml kind
	const chaos_string& get_kind() const {
		return xml_kind;
	}
	// set xml kind
	void set_kind(const chaos_string& new_kind) {
		xml_kind = new_kind;
	}
	// get xml date
	const chaos_string& get_date() const {
		return xml_date;
	}
	// set xml date
	void set_date(const chaos_string& new_date) {
		xml_date = new_date;
	}
	// get xml time
	const chaos_string& get_time() const {
		return xml_time;
	}
	// set xml time
	void set_time(const chaos_string& new_time) {
		xml_time = new_time;
	}
	// get xml info
	const chaos_string& get_info() const {
		return xml_info;
	}
	// set xml info
	void set_info(const chaos_string& new_info) {
		xml_info = new_info;
	}
	// get values version
	const chaos_string& get_values_version() const {
		return values_version;
	}
	// set values version
	void set_values_version(const chaos_string& new_version) {
		values_version = new_version;
	}
	// get values
	const chaos_value_manager<chaos_string>* get_values() const {
		return values;
	}
	chaos_value_manager<chaos_string>* get_values() {
		return values;
	}

// operation
private:
	// encoding check
	static bool encoding_check(
		int accept_encoding,
		int& detect_encoding,
		const chaos_string& encoding);
	// load in encoding
	bool load_in_encoding(
		chaos_file_data* filedata,
		int accept_encoding,
		int& detect_encoding);
public:
	// load
	bool load(chaos_file_data* filedata);
	// save
	bool save(chaos_file_data* filedata) const;
};

#endif // CHAOS_XML_H_INCLUDED
