////////////////////////////////////////////////////////////////////////////////
// stdafx.cpp : implement standard framework

////////////////////////////////////////////////////////////////////////////////
// include

#include "stdafx.h"

////////////////////////////////////////////////////////////////////////////////
// heap

#ifdef CHAOS_PLATFORM_WIN32

#ifdef CHAOS_DEBUG

#undef new

#endif // CHAOS_DEBUG

// new

void* operator new(size_t size) {

#ifdef CHAOS_DEBUG

	void* p = _malloc_dbg(size, _CLIENT_BLOCK, __FILE__, __LINE__);

#else // CHAOS_DEBUG

	void* p = malloc(size);

#endif // CHAOS_DEBUG

	if (p == NULL) {
		throw std::bad_alloc();
	}
	return p;
}

// delete

void operator delete(void* p) {

#ifdef CHAOS_DEBUG

	_free_dbg(p, _CLIENT_BLOCK);

#else // CHAOS_DEBUG

	free(p);

#endif // CHAOS_DEBUG

}

#ifdef CHAOS_DEBUG

// placement new

void* operator new(size_t size, const char* filename, int line) {
	void* p = _malloc_dbg(size, _CLIENT_BLOCK, filename, line);
	if (p == NULL) {
		throw std::bad_alloc();
	}
	return p;
}

// placement delete

void operator delete(void* p, const char* /*filename*/, int /*line*/) {
	_free_dbg(p, _CLIENT_BLOCK);
}

#define new \
	new(__FILE__, __LINE__)

#endif // CHAOS_DEBUG

#endif // CHAOS_PLATFORM_WIN32

////////////////////////////////////////////////////////////////////////////////
// debug

#ifdef CHAOS_DEBUG

////////////////////////////////////////////////////////////////////////////////
// attribute

// asserting

bool chaos_debug::asserting = false;

////////////////////////////////////////////////////////////////////////////////
// assert

// assert report

bool chaos_debug::assert_prompt(const char* filename, int line) {

#ifdef CHAOS_PLATFORM_WIN32

	char message[256+1];
	snprintf(
		message, sizeof(message)-1,
		"Assert Failed:\n%s : %d",
		filename, line);
	message[sizeof(message)-1] = '\0';
	asserting = true;
	int result = MessageBox(
		NULL, message, "ASSERT", MB_ABORTRETRYIGNORE);
	asserting = false;
	if (result == IDABORT) {
		exit(1);
	}
	return result == IDRETRY;

#else // CHAOS_PLATFORM_WIN32

	fprintf(
		stderr,
		"Assert Failed:\n%s : %d",
		filename, line);
	exit(1);
	return false;

#endif // CHAOS_PLATFORM_WIN32

}

////////////////////////////////////////////////////////////////////////////////
// trace

// trace report

void chaos_debug::trace_output(const char* format, ...) {

#ifdef CHAOS_GUI_WIN32

	va_list ap;
	va_start(ap, format);
	char output[256*2];
	vsnprintf(output, sizeof(output)/2, format, ap);
	output[sizeof(output)/2] = '\0';
	va_end(ap);
	char* lf = output;
	while ((lf = strchr(lf, '\n')) != NULL) {
		memmove(lf+1, lf, (strlen(lf)+1));
		*lf = '\r';
		lf += 2;
	}
	OutputDebugString(output);

#else // CHAOS_GUI_WIN32

	vfprintf(stderr, format, ap);

#endif // CHAOS_GUI_WIN32

}

#endif // CHAOS_DEBUG

////////////////////////////////////////////////////////////////////////////////
// log

////////////////////////////////////////////////////////////////////////////////
// create & destroy

// default constructor

chaos_log::chaos_log() :
	log_filename(NULL),
	log_fpt(NULL),
	log_level(loglv_none)
{

#ifdef CHAOS_GUI_WIN32

	log_wnd = NULL;

#else // CHAOS_GUI_WIN32

	output_stderr = false;

#endif // CHAOS_GUI_WIN32

}

// destructor

chaos_log::~chaos_log() {
	if (log_fpt != NULL) {
		fclose(log_fpt);
	}
	if (log_filename != NULL) {
		free(log_filename);
	}
}

// get singleton instance

chaos_log* chaos_log::instance() {
	static chaos_log log;
	return &log;
}

////////////////////////////////////////////////////////////////////////////////
// get & set

// set log filename

void chaos_log::set_log_filename(const char* new_log_filename) {
	if (log_filename != NULL) {
		free(log_filename);
		log_filename = NULL;
	}
	if (new_log_filename != NULL) {
		size_t length = strlen(new_log_filename);
		log_filename = (char*)malloc(length+1);
		if (log_filename != NULL) {
			strcpy(log_filename, new_log_filename);
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// output

// log output(variable args)

void chaos_log::log_output_v(const char* format, va_list ap) {

#ifdef CHAOS_GUI_WIN32

	if (get_log_wnd() != NULL) {
		char output[256*2];
		vsnprintf(output, sizeof(output)/2, format, ap);
		output[sizeof(output)/2] = '\0';
		char* lf = output;
		while ((lf = strchr(lf, '\n')) != NULL) {
			memmove(lf+1, lf, (strlen(lf)+1));
			*lf = '\r';
			lf += 2;
		}
		SendMessage(
			get_log_wnd(),
			WM_LOGOUT,
			0, (LPARAM)output);
		return;
	}

#else // CHAOS_GUI_WIN32

	if (is_output_stderr()) {
		vfprintf(stderr, format, ap);
		return;
	}

#endif // CHAOS_GUI_WIN32

	if (log_filename != NULL) {
		if (log_fpt == NULL) {
			log_fpt = fopen(log_filename, "at");
			if (log_fpt == NULL) {
				TRACE("cannot create log file\n");
				free(log_filename);
				log_filename = NULL;
				return;
			}
		}
		vfprintf(log_fpt, format, ap);
		fflush(log_fpt);
	}
}

// error log output

void chaos_log::error_log_output(const char* format, ...) {
	chaos_log* log = instance();
	if (log->get_log_level() >= loglv_error) {
		va_list ap;
		va_start(ap, format);
		log->log_output_v(format, ap);
		va_end(ap);
	}
}

// warning log output

void chaos_log::warning_log_output(const char* format, ...) {
	chaos_log* log = instance();
	if (log->get_log_level() >= loglv_warning) {
		va_list ap;
		va_start(ap, format);
		log->log_output_v(format, ap);
		va_end(ap);
	}
}

// report log output

void chaos_log::report_log_output(const char* format, ...) {
	chaos_log* log = instance();
	if (log->get_log_level() >= loglv_report) {
		va_list ap;
		va_start(ap, format);
		log->log_output_v(format, ap);
		va_end(ap);
	}
}
