////////////////////////////////////////////////////////////////////////////////
// stdafx.h : declare standard framework

#ifndef STDAFX_H_INCLUDED
#define STDAFX_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// compiler setting

#if defined(_MSC_VER) || defined(__BORLANDC__)

// Compiler is VisualC++ or BorlandC++.

#ifndef __BORLANDC__

#define CHAOS_COMPILER_VC

#else

#define CHAOS_COMPILER_BC

#endif

// conditional expression is constant
#pragma warning(disable : 4127)
// nonstandard extension used(in windows.h)
#pragma warning(disable : 4201)
// private operator= are good to have
#pragma warning(disable : 4512)
// delete unused inline function
#pragma warning(disable : 4514)
// unreachable code?（Unknown）
#pragma warning(disable : 4702)
// unreferenced inline/local function has been removed
#pragma warning(disable : 4710)
// identifier was truncated in the debug information
#pragma warning(disable : 4786)

#define CHAOS_PLATFORM_WIN32
#define CHAOS_SOURCE_CODE_SJIS
#define CHAOS_LOCALE_CODE_SJIS
#define CHAOS_FILENAME_CODE_SJIS

#ifndef CHAOS_SDL_WIN32

#define CHAOS_GUI_CODE_SJIS
#define CHAOS_GUI_WIN32
#define CHAOS_FONT_WIN32
#define CHAOS_SCREEN_WIN32
#define CHAOS_VIDEO_DIRECTX
#define CHAOS_SOUND_DIRECTX
#define CHAOS_CDAUDIO_WIN32
#define CHAOS_API_PNG
#define CHAOS_API_OGG_VORBIS
#define CHAOS_API_WIN32
#define CHAOS_API_DIRECTX
#define CHAOS_API_MMSYSTEM

#ifdef _DEBUG

#define CHAOS_DEBUG

#endif // _DEBUG

#else // CHAOS_SDL_WIN32

// unreferenced local function has been removed
#pragma warning(disable : 4505)

#define CHAOS_GUI_CODE_UTF8
#define CHAOS_GUI_GTK
#define CHAOS_FONT_PANGO
#define CHAOS_SCREEN_SDL
#define CHAOS_VIDEO_SDL
#define CHAOS_SOUND_SDL
#define CHAOS_CDAUDIO_SDL
#define CHAOS_API_PNG
#define CHAOS_API_GTK
#define CHAOS_API_SDL

#endif // CHAOS_SDL_WIN32

#elif defined(__GNUC__)

// Compiler is GNU C++.

#define CHAOS_COMPILER_GC

#define CHAOS_PLATFORM_UNIX
#define CHAOS_SOURCE_CODE_UTF8
#define CHAOS_LOCALE_CODE_GCONV
#define CHAOS_FILENAME_CODE_GCONV
#define CHAOS_GUI_CODE_UTF8
#define CHAOS_GUI_GTK
#define CHAOS_FONT_PANGO
#define CHAOS_SCREEN_SDL
#define CHAOS_VIDEO_SDL
#define CHAOS_SOUND_SDL
#define CHAOS_CDAUDIO_SDL
#define CHAOS_API_PNG
#define CHAOS_API_GTK
#define CHAOS_API_SDL

#else

#error unknown compiler

#endif // compiler

////////////////////////////////////////////////////////////////////////////////
// include

// standard library

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <memory.h>
#include <limits.h>
#include <fcntl.h>
#include <time.h>
#include <math.h>
#include <sys/types.h>

// stl

#include <iterator>
#include <vector>
#include <list>
#include <deque>
#include <set>
#include <map>
#include <queue>
#include <stack>
#include <algorithm>
#include <stdexcept>

#ifdef CHAOS_PLATFORM_WIN32

// windows

#include <direct.h>
#include <io.h>
#include <process.h>

#define _WINVER			0x0400
#define _WIN32_WINNT	0x0400
#define _WIN32_IE		0x0300

#include <windows.h>

#define snprintf	_snprintf
#define vsnprintf	_vsnprintf

#elif defined(CHAOS_PLATFORM_UNIX)

// unix

#include <unistd.h>
#include <sys/mman.h>

#endif

#ifdef CHAOS_API_PNG

// PNG

#include <png.h>

#endif // CHAOS_API_PNG

#ifdef CHAOS_API_OGG_VORBIS

// Ogg Vorbis

#include <vorbis/vorbisfile.h>

#endif // CHAOS_API_OGG_VORBIS

#ifdef CHAOS_GUI_WIN32

// GUI : Win32

#include <commctrl.h>
#include <prsht.h>
#include <imm.h>

typedef HWND chaos_wnd_handle;

#elif defined(CHAOS_GUI_GTK)

// GUI : GTK+

#include <glib.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>

typedef GtkWidget* chaos_wnd_handle;

#endif // CHAOS_GUI

#ifdef CHAOS_API_MMSYSTEM

// Win32 MultimediaSystem

#include <mmsystem.h>

#endif // CHAOS_API_MMSYSTEM

#ifdef CHAOS_API_DIRECTX

// DirectX

#define DIRECTDRAW_VERSION	0x0300
#define DIRECTSOUND_VERSION	0x0300
#define DIRECTINPUT_VERSION	0x0300

#include <ddraw.h>
#include <dsound.h>
#include <dshow.h>

#endif // CHAOS_API_DIRECTX

#ifdef CHAOS_API_SDL

// SDL

#include <SDL.h>
#include <SDL_mixer.h>

#endif // CHAOS_API_SDL

#ifdef CHAOS_PLATFORM_WIN32

// debug

#include <crtdbg.h>

#endif // CHAOS_PLATFORM

////////////////////////////////////////////////////////////////////////////////
// heap

#ifdef CHAOS_PLATFORM_WIN32

// new

void* operator new(size_t size);

// delete

void operator delete(void* p);

#ifdef CHAOS_DEBUG

// placement new

void* operator new(size_t size, const char* filename, int line);

// placement delete

void operator delete(void* p, const char* filename, int line);

// macro

#define malloc(s) \
	_malloc_dbg((s), _NORMAL_BLOCK, __FILE__, __LINE__)

#define calloc(n, s) \
	_calloc_dbg((n), (s), _NORMAL_BLOCK, __FILE__, __LINE__)

#define realloc(p, s) \
	_realloc_dbg((p), (s), _NORMAL_BLOCK, __FILE__, __LINE__)

#define free(p) \
	_free_dbg((p), _NORMAL_BLOCK)

#define new \
	new(__FILE__, __LINE__)

#endif // CHAOS_DEBUG

#endif // CHAOS_PLATFORM_WIN32

////////////////////////////////////////////////////////////////////////////////
// debug

#ifdef CHAOS_DEBUG

class chaos_debug {
// attribute
private:
	// asserting
	static bool asserting;

// assert
public:
	// is asserting
	static bool is_asserting() {
		return asserting;
	}
	// assert report
	static bool assert_prompt(const char* filename, int line);

// trace
public:
	// trace report
	static void trace_output(const char* format, ...);
};

// macro

#define ASSERT(f) \
	do { \
		if (!(f) && chaos_debug::assert_prompt(__FILE__, __LINE__)) { \
			__asm{ int 3 }; \
		} \
	} while (0) \

#define TRACE	chaos_debug::trace_output

#else // CHAOS_DEBUG

// trace report(dummy)

inline void trace_output(const char*, ...) {
}

// macro

#define ASSERT(f)

#define TRACE	true? (void)0: trace_output

#endif // CHAOS_DEBUG

////////////////////////////////////////////////////////////////////////////////
// log

class chaos_log {
// enum
public:
	// log level
	enum log_level_type {
		loglv_none		= 0,
		loglv_error		= 1,
		loglv_warning	= 2,
		loglv_report	= 3
	};

// attribute
private:
	// log filename
	char* log_filename;
	// log file
	FILE* log_fpt;
	// log output level
	log_level_type log_level;

#ifdef CHAOS_GUI_WIN32

	// log window
	chaos_wnd_handle log_wnd;

#else // CHAOS_GUI_WIN32

	// output stderr
	bool output_stderr;

#endif // CHAOS_GUI_WIN32

// create & destroy
private:
	// default constructor
	chaos_log();

public:
	// destructor
	~chaos_log();

	// get singleton instance
	static chaos_log* instance();

// get & set
public:
	// get log output level
	log_level_type get_log_level() const {
		return log_level;
	}
	// get log output level
	void set_log_level(log_level_type new_log_level) {
		log_level = new_log_level;
	}
	// get log filename
	const char* get_log_filename() const {
		return log_filename;
	}
	// set log filename
	void set_log_filename(const char* new_log_filename);

#ifdef CHAOS_GUI_WIN32

	// get log window haldle
	chaos_wnd_handle get_log_wnd() const {
		return log_wnd;
	}
	// set log window haldle
	void set_log_wnd(chaos_wnd_handle new_log_wnd) {
		log_wnd = new_log_wnd;
	}

#else // CHAOS_GUI_WIN32

	// is output stderr
	bool is_output_stderr() const {
		return output_stderr;
	}
	// set output stderr
	void set_output_stderr(bool new_output_stderr) {
		output_stderr = new_output_stderr;
	}

#endif // CHAOS_GUI_WIN32

// output
private:
	// log output(variable args)
	void log_output_v(const char* format, va_list ap);

public:
	// error log output
	static void error_log_output(const char* format, ...);
	// warning log output
	static void warning_log_output(const char* format, ...);
	// report log output
	static void report_log_output(const char* format, ...);
};

// log message

#ifdef CHAOS_GUI_WIN32

#define WM_LOGOUT	(WM_USER+100)

#endif // CHAOS_GUI_WIN32

// macro

#define LOG_ERR		chaos_log::error_log_output
#define LOG_WRN		chaos_log::warning_log_output
#define LOG_RPT		chaos_log::report_log_output

////////////////////////////////////////////////////////////////////////////////
// endian

#ifdef CHAOS_GUI_GTK

#if G_BYTE_ORDER == G_LITTLE_ENDIAN

#define CHAOS_ENDIAN_LITTLE

#else // G_BYTE_ORDER

#define CHAOS_ENDIAN_BIG

#endif // G_BYTE_ORDER

#else // CHAOS_GUI_GTK

#define CHAOS_ENDIAN_LITTLE

#endif // CHAOS_GUI_GTK

////////////////////////////////////////////////////////////////////////////////
// fixed type

#ifndef CHAOS_GUI_GTK

typedef signed char		chaos_int8;
typedef unsigned char	chaos_uint8;
typedef signed short	chaos_int16;
typedef unsigned short	chaos_uint16;
typedef signed long		chaos_int32;
typedef unsigned long	chaos_uint32;

#else // CHAOS_GUI_GTK

typedef gint8			chaos_int8;
typedef guint8			chaos_uint8;
typedef gint16			chaos_int16;
typedef guint16			chaos_uint16;
typedef gint32			chaos_int32;
typedef guint32			chaos_uint32;

#endif // CHAOS_GUI_GTK

////////////////////////////////////////////////////////////////////////////////
// rgba

#ifdef CHAOS_ENDIAN_LITTLE

#define CHAOS_RGBA_BLUE_OFS		0
#define CHAOS_RGBA_GREEN_OFS	1
#define CHAOS_RGBA_RED_OFS		2
#define CHAOS_RGBA_ALPHA_OFS	3

#define CHAOS_RGBA_BLUE_SHIFT	0
#define CHAOS_RGBA_GREEN_SHIFT	8
#define CHAOS_RGBA_RED_SHIFT	16
#define CHAOS_RGBA_ALPHA_SHIFT	24

#define CHAOS_RGBA_BLUE(c)		((c) & 0xFF)
#define CHAOS_RGBA_GREEN(c)		(((c) >> 8) & 0xFF)
#define CHAOS_RGBA_RED(c)		(((c) >> 16) & 0xFF)
#define CHAOS_RGBA_ALPHA(c)		(((c) >> 24) & 0xFF)

#define CHAOS_RGBA(r, g, b, a) \
	(chaos_uint32)( \
		(((b) & 0xFF) | (((g) & 0xFF) << 8) | \
			(((r) & 0xFF) << 16) | (((a) & 0xFF) << 24)))

#elif defined(CHAOS_ENDIAN_BIG)

#define CHAOS_RGBA_BLUE_OFS		0
#define CHAOS_RGBA_GREEN_OFS	1
#define CHAOS_RGBA_RED_OFS		2
#define CHAOS_RGBA_ALPHA_OFS	3

#define CHAOS_RGBA_BLUE_SHIFT	24
#define CHAOS_RGBA_GREEN_SHIFT	16
#define CHAOS_RGBA_RED_SHIFT	8
#define CHAOS_RGBA_ALPHA_SHIFT	0

#define CHAOS_RGBA_BLUE(c)		(((c) >> 24) & 0xFF)
#define CHAOS_RGBA_GREEN(c)		(((c) >> 16) & 0xFF)
#define CHAOS_RGBA_RED(c)		(((c) >> 8) & 0xFF)
#define CHAOS_RGBA_ALPHA(c)		((c) & 0xFF)

#define CHAOS_RGBA(r, g, b, a) \
	(chaos_uint32)( \
		((((b) & 0xFF) << 24) | (((g) & 0xFF) << 16) | \
			(((r) & 0xFF) << 8) | ((a) & 0xFF)))

#endif // CHAOS_ENDIAN

#endif // STDAFX_H_INCLUDED
